@@ -42,7 +42,7 @@ static const QChar tabChar('\t');
 static const QChar spaceChar(' ');
 
 KateRenderer::KateRenderer(KateDocument* doc, KateView *view)
-  : m_doc(doc), m_view (view), m_caretStyle(KateRenderer::Insert)
+  : m_doc(doc), m_view (view), m_caretStyle(KateRenderer::Line)
     , m_drawCaret(true)
     , m_showSelections(true)
     , m_showTabs(true)
@@ -231,22 +231,21 @@ void KateRenderer::paintTabstop(QPainter &paint, qreal x, qreal y)
 {
   QPen penBackup( paint.pen() );
   QPen pen( config()->tabMarkerColor() );
-  pen.setWidthF(qMax(0.5, spaceWidth() * .1));
-  pen.setCapStyle(Qt::RoundCap);
+  pen.setWidth(qMax(1u, spaceWidth() / 10));
   paint.setPen( pen );
+  paint.setRenderHint(QPainter::Antialiasing, false);
 
-  // FIXME: optimize for speed!
-  qreal dist = spaceWidth() * 0.3;
-  QPointF points[8];
-  points[0] = QPointF(x - dist, y - dist);
-  points[1] = QPointF(x, y);
-  points[2] = QPointF(x, y);
-  points[3] = QPointF(x - dist, y + dist);
+  int dist = spaceWidth() * 0.3;
+  QPoint points[8];
+  points[0] = QPoint(x - dist, y - dist);
+  points[1] = QPoint(x, y);
+  points[2] = QPoint(x, y);
+  points[3] = QPoint(x - dist, y + dist);
   x += spaceWidth() / 3.0;
-  points[4] = QPointF(x - dist, y - dist);
-  points[5] = QPointF(x, y);
-  points[6] = QPointF(x, y);
-  points[7] = QPointF(x - dist, y + dist);
+  points[4] = QPoint(x - dist, y - dist);
+  points[5] = QPoint(x, y);
+  points[6] = QPoint(x, y);
+  points[7] = QPoint(x - dist, y + dist);
   paint.drawLines(points, 4);
   paint.setPen( penBackup );
 }
@@ -308,7 +307,6 @@ QList<QTextLayout::FormatRange> KateRenderer::decorationsForLine( const KateText
       // Add arbitrary highlighting ranges to the list
       renderRanges.appendRanges(m_view->internalHighlights(), selectionsOnly, view());
       renderRanges.appendRanges(m_view->externalHighlights(), selectionsOnly, view());
-      renderRanges.appendRanges(m_doc->documentHighlights(), selectionsOnly, view());
 
     } else {
       // Add the code completion arbitrary highlight to the list
@@ -331,7 +329,7 @@ QList<QTextLayout::FormatRange> KateRenderer::decorationsForLine( const KateText
         selectionHighlight->addRange(new KTextEditor::Range(line, 0, line + 1, 0), backgroundAttribute);
       else
         if(m_view->blockSelection() && m_view->selectionRange().overlapsLine(line))
-          selectionHighlight->addRange(new KTextEditor::Range(line, m_view->selectionRange().start().column(), line, m_view->selectionRange().end().column()), backgroundAttribute);        
+          selectionHighlight->addRange(new KTextEditor::Range(line, m_view->selectionRange().start().column(), line, m_view->selectionRange().end().column()), backgroundAttribute);
         else
           selectionHighlight->addRange(new KTextEditor::Range(m_view->selectionRange()), backgroundAttribute);
 
@@ -350,7 +348,7 @@ QList<QTextLayout::FormatRange> KateRenderer::decorationsForLine( const KateText
       } else {
         KTextEditor::Range rangeNeeded = m_view->selectionRange().encompass(m_dynamicRegion.boundingRange());
         rangeNeeded &= KTextEditor::Range(line, 0, line + 1, 0);
-  
+
         currentPosition = qMax(KTextEditor::Cursor(line, 0), rangeNeeded.start());
         endPosition = qMin(KTextEditor::Cursor(line + 1, 0), rangeNeeded.end());
       }
@@ -384,19 +382,21 @@ QList<QTextLayout::FormatRange> KateRenderer::decorationsForLine( const KateText
         fr.length = textLine->length() - currentPosition.column() + 1;
       }
 
-      KTextEditor::Attribute a = renderRanges.generateAttribute();
-      fr.format = a;
+      KTextEditor::Attribute::Ptr a = renderRanges.generateAttribute();
+      if(a) {
+        fr.format = *a;
 
-      if(selectionsOnly) {
-        if(m_view->blockSelection()) {
-          int minSelectionColumn = qMin(m_view->selectionRange().start().column(), m_view->selectionRange().end().column());
-          int maxSelectionColumn = qMax(m_view->selectionRange().start().column(), m_view->selectionRange().end().column());
- 
-          if(currentPosition.column() >= minSelectionColumn && currentPosition.column() < maxSelectionColumn)
-            assignSelectionBrushesFromAttribute(fr, a);
-        
-        } else if (m_view->selection() && m_view->selectionRange().contains(currentPosition)) {
-          assignSelectionBrushesFromAttribute(fr, a);
+        if(selectionsOnly) {
+          if(m_view->blockSelection()) {
+            int minSelectionColumn = qMin(m_view->selectionRange().start().column(), m_view->selectionRange().end().column());
+            int maxSelectionColumn = qMax(m_view->selectionRange().start().column(), m_view->selectionRange().end().column());
+
+            if(currentPosition.column() >= minSelectionColumn && currentPosition.column() < maxSelectionColumn)
+              assignSelectionBrushesFromAttribute(fr, *a);
+
+          } else if (m_view->selection() && m_view->selectionRange().contains(currentPosition)) {
+            assignSelectionBrushesFromAttribute(fr, *a);
+          }
         }
       }
 
@@ -429,13 +429,12 @@ void KateRenderer::assignSelectionBrushesFromAttribute(QTextLayout::FormatRange&
 The ultimate line painting function.
 Currently missing features:
 - draw indent lines
-- draw input method hints
 */
 void KateRenderer::paintTextLine(QPainter& paint, KateLineLayoutPtr range, int xStart, int xEnd, const KTextEditor::Cursor* cursor)
 {
   Q_ASSERT(range->isValid());
 
-//   kDebug()<<"KateRenderer::paintTextLine";
+//   kDebug( 13033 )<<"KateRenderer::paintTextLine";
 
   // font data
   const QFontMetrics& fm = config()->fontMetrics();
@@ -448,7 +447,14 @@ void KateRenderer::paintTextLine(QPainter& paint, KateLineLayoutPtr range, int x
 
   // Draws the dashed underline at the start of a folded block of text.
   if (range->startsInvisibleBlock()) {
-    paint.setPen(QPen(config()->wordWrapMarkerColor(), 1, Qt::DashLine));
+    paint.setRenderHint(QPainter::Antialiasing, false);
+    QPen pen(config()->wordWrapMarkerColor());
+    pen.setCosmetic(true);
+    pen.setStyle(Qt::DashLine);
+    QVector<qreal> dash = pen.dashPattern(); // workaround for N226156
+    pen.setDashOffset(xStart);
+    pen.setDashPattern(dash);
+    paint.setPen(pen);
     paint.drawLine(0, (fm.height() * range->viewLineCount()) - 1, xEnd - xStart, (fm.height() * range->viewLineCount()) - 1);
   }
 
@@ -517,24 +523,30 @@ void KateRenderer::paintTextLine(QPainter& paint, KateLineLayoutPtr range, int x
 
       backgroundDetermined:
 
-      // Draw selection outside of areas where text is rendered
+      // Draw selection or background color outside of areas where text is rendered
       if (!m_printerFriendly ) {
+        bool draw = false;
+        QBrush drawBrush;
         if (m_view->selection() && !m_view->blockSelection() && m_view->lineEndSelected(line.end(true))) {
-          int width= xEnd - xStart;
-          int height= fm.height() * (i + 1);
+          draw = true;
+          drawBrush = config()->selectionColor();
+        } else if (backgroundBrushSet && !m_view->blockSelection()) {
+          draw = true;
+          drawBrush = backgroundBrush;
+        }
+
+        if (draw) {
           int fillStartX = line.endX() - line.startX() + line.xOffset() - xStart;
           int fillStartY = fm.height() * i;
- 
-          QRect area(fillStartX, fillStartY, width, height);
-          paint.fillRect(area, config()->selectionColor());
+          int width= xEnd - xStart - fillStartX;
+          int height= fm.height();
 
-        } else if (backgroundBrushSet && !m_view->blockSelection()) {
-          // Draw text background outside of areas where text is rendered.
-          QRect area(line.endX() /*+ line.xOffset()*/ - line.startX() +  (i==0?0:range->shiftX()) - xStart/*-(i*xEnd)*/, fm.height() * i, xEnd - xStart, fm.height() /** (i + 1)*/);
-          paint.fillRect(area, /*QBrush(Qt::red));*/backgroundBrush);
-  //         kDebug()<<i<<":backgroundBrush:"<<backgroundBrush<<"---"<<xEnd<<"/"<<xStart;
-  //         kDebug()<<line.endX()<<"---"<<line.xOffset();
-          //kDebug()<<i<<"----"<<area;
+          // reverse X for right-aligned lines
+          if (range->layout()->textOption().alignment() == Qt::AlignRight)
+            fillStartX = 0;
+
+          QRect area(fillStartX, fillStartY, width, height);
+          paint.fillRect(area, drawBrush);
         }
       }
       // Draw indent lines
@@ -593,7 +605,7 @@ void KateRenderer::paintTextLine(QPainter& paint, KateLineLayoutPtr range, int x
       // Make the caret the desired width
       int caretWidth = 2;
       QTextLine line = range->layout()->lineForTextPosition(cursor->column());
-      if (caretStyle() == Replace) {
+      if (caretStyle() == Block || (m_view->viInputMode() && m_view->getCurrentViMode() != InsertMode)) {
         if (line.isValid() && cursor->column() < range->length()) {
           caretWidth = int(line.cursorToX(cursor->column() + 1) - line.cursorToX(cursor->column()));
           if (caretWidth < 0)
@@ -610,18 +622,28 @@ void KateRenderer::paintTextLine(QPainter& paint, KateLineLayoutPtr range, int x
         c = m_caretOverrideColor;
 
       } else {
-        foreach (QTextLayout::FormatRange r, range->layout()->additionalFormats())
+        // search for the FormatRange that includes the cursor
+        foreach (const QTextLayout::FormatRange &r, range->layout()->additionalFormats()) {
           if ( (r.start <= cursor->column() ) && ( (r.start + r.length)  > cursor->column()) ) {
-            c = r.format.foreground().color();
+            // check for Qt::NoBrush, as the returned color is black() and no invalid QColor
+            QBrush foregroundBrush = r.format.foreground();
+            if (foregroundBrush != Qt::NoBrush) {
+              c = r.format.foreground().color();
+            }
             break;
           }
+        }
+
+        // still no color found, fall back to default style
         if (!c.isValid())
-          if (range->layout()->additionalFormats().count())
-            c = range->layout()->additionalFormats().last().format.foreground().color();
-          else
             c = attribute(KateExtendedAttribute::dsNormal)->foreground().color();
       }
 
+      // make it possible to see the selected character in the vi input mode's normal/visual mode
+      if (m_view->viInputMode() && m_view->getCurrentViMode() != InsertMode ) {
+        c.setAlpha(128);
+      }
+
       if (cursor->column() <= range->length()) {
         paint.save();
         paint.setPen(QPen(c, caretWidth));
@@ -638,7 +660,7 @@ void KateRenderer::paintTextLine(QPainter& paint, KateLineLayoutPtr range, int x
         const KateTextLayout& lastLine = range->viewLine(range->viewLineCount() - 1);
         int x = range->widthOfLastLine() + spaceWidth() * (cursor->column() - range->length());
         if ( (x >= xStart) && (x <= xEnd))
-          paint.fillRect(x, (int)lastLine.lineLayout().y(), caretWidth, fm.height(), c);
+          paint.fillRect(x-xStart, (int)lastLine.lineLayout().y(), caretWidth, fm.height(), c);
       }
     }
   }
@@ -737,13 +759,14 @@ void KateRenderer::layoutLine(KateLineLayoutPtr lineLayout, int maxwidth, bool c
 {
   // if maxwidth == -1 we have no wrap
 
-  Q_ASSERT(lineLayout->textLine());
+  KateTextLine::Ptr textLine = lineLayout->textLine();
+  Q_ASSERT(textLine);
 
   QTextLayout* l = lineLayout->layout();
   if (!l) {
-    l = new QTextLayout(lineLayout->textLine()->string(), config()->font());
+    l = new QTextLayout(textLine->string(), config()->font());
   } else {
-    l->setText(lineLayout->textLine()->string());
+    l->setText(textLine->string());
     l->setFont(config()->font());
   }
 
@@ -755,7 +778,7 @@ void KateRenderer::layoutLine(KateLineLayoutPtr lineLayout, int maxwidth, bool c
   QTextOption opt;
   opt.setFlags(QTextOption::IncludeTrailingSpaces);
   opt.setTabStop(m_tabWidth * config()->fontMetrics().width(spaceChar));
-  opt.setWrapMode(QTextOption::WrapAnywhere);//QTextOption::WrapAtWordBoundaryOrAnywhere);
+  opt.setWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
 
   // Find the first strong character in the string.
   // If it is an RTL character, set the base layout direction of the string to RTL.
@@ -764,13 +787,19 @@ void KateRenderer::layoutLine(KateLineLayoutPtr lineLayout, int maxwidth, bool c
   // Qt's text renderer ("scribe") version 4.2 assumes a "higher-level protocol"
   // (such as KatePart) will specify the paragraph level, so it does not apply P2 & P3
   // by itself. If this ever change in Qt, the next code block could be removed.
-  if (lineLayout->textLine()->string().isRightToLeft())
-    opt.setTextDirection(Qt::RightToLeft);
+  if (isLineRightToLeft(lineLayout)) {
+      opt.setAlignment( Qt::AlignRight );
+      opt.setTextDirection( Qt::RightToLeft );
+  }
+  else {
+      opt.setAlignment( Qt::AlignLeft );
+      opt.setTextDirection( Qt::LeftToRight );
+  }
 
   l->setTextOption(opt);
 
   // Syntax highlighting, inbuilt and arbitrary
-  l->setAdditionalFormats(decorationsForLine(lineLayout->textLine(), lineLayout->line()));
+  l->setAdditionalFormats(decorationsForLine(textLine, lineLayout->line()));
 
   // Begin layouting
   l->beginLayout();
@@ -794,7 +823,7 @@ void KateRenderer::layoutLine(KateLineLayoutPtr lineLayout, int maxwidth, bool c
     if (needShiftX) {
       needShiftX = false;
       // Determine x offset for subsequent-lines-of-paragraph indenting
-      int pos = lineLayout->textLine()->nextNonSpaceChar(0);
+      int pos = textLine->nextNonSpaceChar(0);
 
       if (pos > 0) {
         shiftX = (int)line.cursorToX(pos);
@@ -818,6 +847,50 @@ void KateRenderer::layoutLine(KateLineLayoutPtr lineLayout, int maxwidth, bool c
   lineLayout->setLayout(l);
 }
 
+
+// 1) QString::isRightToLeft() sux
+// 2) QString::isRightToLeft() is marked as internal (WTF?)
+// 3) QString::isRightToLeft() does not seem to work on my setup
+// 4) isStringRightToLeft() should behave much better than QString::isRightToLeft() therefore:
+// 5) isStringRightToLeft() kicks ass
+bool KateRenderer::isLineRightToLeft( KateLineLayoutPtr lineLayout ) const
+{
+  QString s = lineLayout->textLine()->string();
+  int i = 0;
+
+  // borrowed from QString::updateProperties()
+  while( i != s.length() )
+  {
+    QChar c = s.at(i);
+
+    switch(c.direction()) {
+      case QChar::DirL:
+      case QChar::DirLRO:
+      case QChar::DirLRE:
+          return false;
+
+      case QChar::DirR:
+      case QChar::DirAL:
+      case QChar::DirRLO:
+      case QChar::DirRLE:
+          return true;
+
+      default:
+          break;
+    }
+    i ++;
+  }
+
+   return false;
+#if 0
+  // or should we use the direction of the widget?
+  QWidget* display = qobject_cast<QWidget*>(view()->parent());
+  if (!display)
+    return false;
+  return display->layoutDirection() == Qt::RightToLeft;
+#endif
+}
+
 int KateRenderer::cursorToX(const KateTextLayout& range, int col) const
 {
   return cursorToX(range, KTextEditor::Cursor(range.line(), col));
@@ -830,20 +903,23 @@ int KateRenderer::cursorToX(const KateTextLayout& range, const KTextEditor::Curs
   return (int)range.lineLayout().cursorToX(pos.column());
 }
 
+int KateRenderer::cursorToX(const KateTextLayout& range, const KTextEditor::Cursor & pos, bool returnPastLine) const
+{
+  int x = cursorToX(range, pos);
+  int over = pos.column() * spaceWidth() - range.width();
+ 
+  if (returnPastLine && over > 0)
+    x += over;
+
+  return x;
+}
+
 KTextEditor::Cursor KateRenderer::xToCursor(const KateTextLayout & range, int x, bool returnPastLine ) const
 {
   Q_ASSERT(range.isValid());
+  KTextEditor::Cursor ret(range.line(), range.lineLayout().xToCursor(x));
 
-  int adjustedX = x;
-
-  if (adjustedX <  range.xOffset())
-    adjustedX = range.xOffset();
-  else if (adjustedX > range.width() + range.xOffset())
-    adjustedX = range.width() + range.xOffset();
-
-  Q_ASSERT(range.isValid());
-  KTextEditor::Cursor ret(range.line(), range.lineLayout().xToCursor(adjustedX));
-
+  // TODO wrong for RTL lines?
   if (returnPastLine && x > range.width() + range.xOffset())
     ret.setColumn(ret.column() + ((x - (range.width() + range.xOffset())) / spaceWidth()));
 
