@@ -16,7 +16,10 @@
    Boston, MA 02110-1301, USA.
 */
 
+#define QT_NO_CAST_FROM_ASCII
+
 #include "kcmdlineargs.h"
+#include <kdebug.h>
 
 #include <config.h>
 
@@ -46,6 +49,8 @@
 #include "kstringhandler.h"
 #include "kurl.h"
 
+#include "kuitsemantics_p.h" // for escaping arguments in i18n
+
 // -----------------------------------------------------------------------------
 // Design notes:
 //
@@ -81,13 +86,13 @@
 // Helper classes
 //
 
-class KCmdLineParsedOptions : public QHash<QString,QString>
+class KCmdLineParsedOptions : public QHash<QByteArray,QByteArray>
 {
 public:
    KCmdLineParsedOptions() { }
 };
 
-class KCmdLineParsedArgs : public QList<QString>
+class KCmdLineParsedArgs : public QList<QByteArray>
 {
 public:
    KCmdLineParsedArgs() { }
@@ -110,7 +115,7 @@ public:
 
 class KCmdLineOptionsPrivate {
     public:
-    QStringList names;
+    QList<QByteArray> names;
     QList<KLocalizedString> descriptions;
     QStringList defaults;
 };
@@ -141,7 +146,7 @@ KCmdLineOptions &KCmdLineOptions::add (const QByteArray &name,
                                        const KLocalizedString &description,
                                        const QByteArray &defaultValue)
 {
-    d->names.append(QString::fromUtf8(name));
+    d->names.append(name);
     d->descriptions.append(description);
     d->defaults.append(QString::fromUtf8(defaultValue));
     return *this;
@@ -169,7 +174,7 @@ class KCmdLineArgsStatic {
     char **argv; // The original argv
     bool parsed : 1; // Whether we have parsed the arguments since calling init
     bool ignoreUnknown : 1; // Ignore unknown options and arguments
-    QString mCwd; // Current working directory. Important for KUnqiueApp!
+    QByteArray mCwd; // Current working directory. Important for KUnqiueApp!
     KCmdLineArgs::StdCmdLineArgs mStdargs;
 
     KCmdLineOptions qt_options;
@@ -218,15 +223,15 @@ class KCmdLineArgsStatic {
      *
      *  +4 - no more options follow         // !fork
      */
-    static int findOption(const KCmdLineOptions &options, QString &opt,
-                          QString &opt_name, QString &def, bool &enabled);
+    static int findOption(const KCmdLineOptions &options, QByteArray &opt,
+                          QByteArray &opt_name, QString &def, bool &enabled);
 
     /**
      * @internal
      *
      * Checks what to do with a single option
      */
-    static void findOption(const QString &optv, const QString &_opt,
+    static void findOption(const QByteArray &optv, const QByteArray &_opt,
                            int &i, bool _enabled, bool &moreOptions);
 
     /**
@@ -244,7 +249,7 @@ class KCmdLineArgsStatic {
      *
      * @param id The name of the options to be removed.
      */
-    static void removeArgs(const QString &id);
+    static void removeArgs(const QByteArray &id);
 };
 
 K_GLOBAL_STATIC(KCmdLineArgsStatic, s)
@@ -254,7 +259,7 @@ KCmdLineArgsStatic::KCmdLineArgsStatic () {
     argsList = 0;
     argc = 0;
     argv = 0;
-    mCwd = QString();
+    mCwd.clear();
     about = 0;
     parsed = false;
     ignoreUnknown = false;
@@ -262,7 +267,6 @@ KCmdLineArgsStatic::KCmdLineArgsStatic () {
 
     // Text codec.
     codec = QTextCodec::codecForLocale();
-    setlocale(LC_ALL, ""); // need to initialize "System" codec, i.e. iconv
 
     // Qt options
     //FIXME: Check if other options are specific to Qt/X11
@@ -298,7 +302,12 @@ KCmdLineArgsStatic::KCmdLineArgsStatic () {
     qt_options.add("qws", ki18n("forces the application to run as QWS Server"));
 #endif
     qt_options.add("reverse", ki18n("mirrors the whole layout of widgets"));
-
+    qt_options.add("stylesheet <file.qss>", ki18n("applies the Qt stylesheet to the application widgets"));
+#if QT_VERSION >= 0x040a00
+# error Qt version larger than 4.9, please fix me
+#endif
+    if (qVersion()[2] >= '5')
+        qt_options.add("graphicssystem <system>", ki18n("use a different graphics system instead of the default one, options are raster and opengl (experimental)"));
     // KDE options
     kde_options.add("caption <caption>",   ki18n("Use 'caption' as name in the titlebar"));
     kde_options.add("icon <icon>",         ki18n("Use 'icon' as the application icon"));
@@ -309,7 +318,9 @@ KCmdLineArgsStatic::KCmdLineArgsStatic () {
 #endif
     kde_options.add("style <style>",       ki18n("sets the application GUI style"));
     kde_options.add("geometry <geometry>", ki18n("sets the client geometry of the main widget - see man X for the argument format"));
+#ifndef Q_WS_WIN
     kde_options.add("smkey <sessionKey>"); // this option is obsolete and exists only to allow smooth upgrades from sessions
+#endif
 }
 
 KCmdLineArgsStatic::~KCmdLineArgsStatic ()
@@ -327,14 +338,15 @@ class KCmdLineArgsPrivate
 {
     friend class KCmdLineArgsStatic;
 public:
-    KCmdLineArgsPrivate(const KCmdLineOptions &_options, const KLocalizedString &_name, const QString &_id)
+    KCmdLineArgsPrivate(const KCmdLineOptions &_options, const KLocalizedString &_name, const QByteArray &_id)
         : options(_options)
         , name(_name)
         , id(_id)
         , parsedOptionList(0)
         , parsedArgList(0)
         , isQt(id == "qt")
-    {}
+    {
+    }
     ~KCmdLineArgsPrivate()
     {
         delete parsedOptionList;
@@ -342,7 +354,7 @@ public:
     }
     const KCmdLineOptions options;
     const KLocalizedString name;
-    const QString id;
+    const QByteArray id;
     KCmdLineParsedOptions *parsedOptionList;
     KCmdLineParsedArgs *parsedArgList;
     bool isQt;
@@ -352,21 +364,21 @@ public:
      *
      *  Set a boolean option
      */
-    void setOption(const QString &option, bool enabled);
+    void setOption(const QByteArray &option, bool enabled);
 
     /**
      * @internal
      *
      *  Set a string option
      */
-    void setOption(const QString &option, const QString &value);
+    void setOption(const QByteArray &option, const QByteArray &value);
 
     /**
      * @internal
      *
      * Add an argument
      */
-    void addArgument(const QString &argument);
+    void addArgument(const QByteArray &argument);
 
     /**
      * @internal
@@ -460,13 +472,13 @@ KCmdLineArgs::init(int _argc, char **_argv, const KAboutData *_about, StdCmdLine
 
    s->about = _about;
    s->parsed = false;
-   s->mCwd = QDir::currentPath();
+   s->mCwd = QDir::currentPath().toLocal8Bit(); //currentPath() uses fromLocal8Bit internally apparently
    addStdCmdLineOptions(stdargs);
 }
 
 QString KCmdLineArgs::cwd()
 {
-   return s->mCwd;
+   return QString::fromLocal8Bit(s->mCwd);
 }
 
 QString KCmdLineArgs::appName()
@@ -490,14 +502,11 @@ void KCmdLineArgs::addStdCmdLineOptions(StdCmdLineArgs stdargs) {
 
 void
 KCmdLineArgs::addCmdLineOptions( const KCmdLineOptions &options, const KLocalizedString &name,
-         const QByteArray &_id, const QByteArray &_afterId)
+         const QByteArray &id, const QByteArray &afterId)
 {
    if (!s->argsList)
       s->argsList = new KCmdLineArgsList;
 
-   QString id = QString::fromUtf8(_id);
-   QString afterId = QString::fromUtf8(_afterId);
-
    int pos = s->argsList->count();
    // To make sure that the named options come before unnamed.
    if (pos > 0 && !id.isEmpty() && s->argsList->last()->d->name.isEmpty())
@@ -507,8 +516,9 @@ KCmdLineArgs::addCmdLineOptions( const KCmdLineOptions &options, const KLocalize
    int i = 0;
    for(args = s->argsList->begin(); args != s->argsList->end(); ++args, i++)
    {
-      if (id == (*args)->d->id)
+       if (id == (*args)->d->id) {
          return; // Options already present.
+      }
 
       // Only check for afterId if it has been given non-empty, as the
       // unnamed option group should come after all named groups.
@@ -518,7 +528,7 @@ KCmdLineArgs::addCmdLineOptions( const KCmdLineOptions &options, const KLocalize
 
    Q_ASSERT( s->parsed == false ); // You must add _ALL_ cmd line options
                                    // before accessing the arguments!
-   s->argsList->insert(pos, new KCmdLineArgs(options, name, id.toUtf8()));
+   s->argsList->insert(pos, new KCmdLineArgs(options, name, id));
 }
 
 void
@@ -530,9 +540,9 @@ KCmdLineArgs::saveAppArgs( QDataStream &ds)
    // Remove Qt and KDE options.
    s->removeArgs("qt");
    s->removeArgs("kde");
+   s->removeArgs("kuniqueapp");
 
-   QByteArray qCwd = QFile::encodeName(s->mCwd);
-   ds << qCwd;
+   ds << s->mCwd;
 
    uint count = s->argsList ? s->argsList->count() : 0;
    ds << count;
@@ -555,6 +565,7 @@ KCmdLineArgs::loadAppArgs( QDataStream &ds)
    // Remove Qt and KDE options.
    s->removeArgs("qt");
    s->removeArgs("kde");
+   s->removeArgs("kuniqueapp");
 
    KCmdLineArgsList::Iterator args;
    if ( s->argsList ) {
@@ -570,32 +581,36 @@ KCmdLineArgs::loadAppArgs( QDataStream &ds)
    QByteArray qCwd;
    ds >> qCwd;
 
-   s->mCwd = QFile::decodeName(qCwd); //FIXME: Is this proper decoding?
+   s->mCwd = qCwd;
 
    uint count;
    ds >> count;
 
    while(count--)
    {
-     QByteArray idRaw;
-     ds >> idRaw;
-     QString id = idRaw;  //FIXME: What about decoding?
+     QByteArray id;
+     ds >> id;
      Q_ASSERT( s->argsList );
+     bool found = false;
      for(args = s->argsList->begin(); args != s->argsList->end(); ++args)
      {
        if ((*args)->d->id  == id)
        {
           (*args)->d->load(ds);
+          found = true;
           break;
        }
      }
+     if (!found) {
+         kWarning() << "Argument definitions for" << id << "not found!";
+         // The next ds >> id will do nonsensical things...
+     }
    }
    s->parsed = true;
 }
 
-KCmdLineArgs *KCmdLineArgs::parsedArgs(const QByteArray &_id)
+KCmdLineArgs *KCmdLineArgs::parsedArgs(const QByteArray &id)
 {
-   QString id = QString::fromUtf8(_id);
    if (!s->argsList)
       return 0;
    KCmdLineArgsList::Iterator args = s->argsList->begin();
@@ -613,7 +628,7 @@ KCmdLineArgs *KCmdLineArgs::parsedArgs(const QByteArray &_id)
    return 0;
 }
 
-void KCmdLineArgsStatic::removeArgs(const QString &id)
+void KCmdLineArgsStatic::removeArgs(const QByteArray &id)
 {
    if (!s->argsList)
       return;
@@ -637,8 +652,8 @@ void KCmdLineArgsStatic::removeArgs(const QString &id)
 }
 
 int
-KCmdLineArgsStatic::findOption(const KCmdLineOptions &options, QString &opt,
-                               QString &opt_name, QString &def, bool &enabled)
+KCmdLineArgsStatic::findOption(const KCmdLineOptions &options, QByteArray &opt,
+                               QByteArray &opt_name, QString &def, bool &enabled)
 {
    int result;
    bool inverse;
@@ -647,7 +662,7 @@ KCmdLineArgsStatic::findOption(const KCmdLineOptions &options, QString &opt,
    {
       result = 0;
       inverse = false;
-      opt_name = options.d->names[i].toUtf8();
+      opt_name = options.d->names[i];
       if (opt_name.startsWith(':') || opt_name.isEmpty())
       {
          continue;
@@ -677,7 +692,7 @@ KCmdLineArgsStatic::findOption(const KCmdLineOptions &options, QString &opt,
                i++;
                if (i >= options.d->names.size())
                   return result+0;
-               QString nextOption = options.d->names[i].toUtf8();
+               QByteArray nextOption = options.d->names[i];
                int p = nextOption.indexOf(' ');
                if (p > 0)
                   nextOption = nextOption.left(p);
@@ -699,7 +714,7 @@ KCmdLineArgsStatic::findOption(const KCmdLineOptions &options, QString &opt,
          if (opt_name.startsWith(' '))
          {
             opt_name = opt_name.mid(1);
-            def = options.d->defaults[i].toUtf8();
+            def = options.d->defaults[i];
             return result+3;
          }
       }
@@ -708,14 +723,14 @@ KCmdLineArgsStatic::findOption(const KCmdLineOptions &options, QString &opt,
 }
 
 void
-KCmdLineArgsStatic::findOption(const QString &optv, const QString &_opt,
+KCmdLineArgsStatic::findOption(const QByteArray &optv, const QByteArray &_opt,
                                int &i, bool _enabled, bool &moreOptions)
 {
    KCmdLineArgsList::Iterator args = s->argsList->begin();
-   QString opt = _opt;
-   QString opt_name;
+   QByteArray opt = _opt;
+   QByteArray opt_name;
    QString def;
-   QString argument;
+   QByteArray argument;
    int j = opt.indexOf('=');
    if (j != -1)
    {
@@ -744,7 +759,7 @@ KCmdLineArgsStatic::findOption(const QString &optv, const QString &_opt,
       int p = 1;
       while (true)
       {
-         QString singleCharOption = " ";
+         QByteArray singleCharOption = " ";
          singleCharOption[0] = optv[p];
          args = s->argsList->begin();
          while (args != s->argsList->end())
@@ -787,11 +802,11 @@ KCmdLineArgsStatic::findOption(const QString &optv, const QString &_opt,
       if (s->ignoreUnknown)
          return;
 #ifdef Q_WS_MACX
-		if (_opt.startsWith("psn_", Qt::CaseInsensitive))
+		if (_opt.startsWith("psn_"))
 			return;
 #endif
       KCmdLineArgs::enable_i18n();
-      KCmdLineArgs::usageError( i18n("Unknown option '%1'.", _opt));
+      KCmdLineArgs::usageError( i18n("Unknown option '%1'.", QString::fromLocal8Bit(_opt)));
    }
 
    if ((result & 4) != 0)
@@ -807,11 +822,11 @@ KCmdLineArgsStatic::findOption(const QString &optv, const QString &_opt,
          if (s->ignoreUnknown)
             return;
 #ifdef Q_WS_MACX
-			if (_opt.startsWith("psn_", Qt::CaseInsensitive))
+			if (_opt.startsWith("psn_"))
 				return;
 #endif
          KCmdLineArgs::enable_i18n();
-         KCmdLineArgs::usageError( i18n("Unknown option '%1'.", _opt));
+         KCmdLineArgs::usageError( i18n("Unknown option '%1'.", QString::fromLocal8Bit(_opt)));
       }
       if (argument.isEmpty())
       {
@@ -819,9 +834,9 @@ KCmdLineArgsStatic::findOption(const QString &optv, const QString &_opt,
          if (i >= s->argc)
          {
             KCmdLineArgs::enable_i18n();
-            KCmdLineArgs::usageError( i18nc("@info:shell %1 is cmdoption name","'%1' missing.",  opt_name));
+            KCmdLineArgs::usageError( i18nc("@info:shell %1 is cmdoption name","'%1' missing.",  QString::fromLocal8Bit(opt_name)));
          }
-         argument = s->decodeInput(s->argv[i]);
+         argument = s->argv[i];
       }
       (*args)->d->setOption(opt, argument);
    }
@@ -840,16 +855,10 @@ KCmdLineArgsStatic::parseAllArgs()
    KCmdLineArgs *appOptions = s->argsList->last();
    if (appOptions->d->id.isEmpty())
    {
-     const KCmdLineOptions &option = appOptions->d->options;
-     for (int i = 0; i < option.d->names.size(); i++)
+     foreach(const QByteArray& name, appOptions->d->options.d->names)
      {
-       if (option.d->names[i].startsWith('+'))
-           allowArgs = true;
-       if ( option.d->names[i].startsWith("!+") )
-       {
-           allowArgs = true;
-           everythingAfterArgIsArgs = true;
-       }
+       everythingAfterArgIsArgs = everythingAfterArgIsArgs || name.startsWith("!+");
+       allowArgs = allowArgs || name.startsWith('+') || everythingAfterArgIsArgs;
      }
    }
    for(int i = 1; i < s->argc; i++)
@@ -860,8 +869,8 @@ KCmdLineArgsStatic::parseAllArgs()
       if ((s->argv[i][0] == '-') && s->argv[i][1] && inOptions)
       {
          bool enabled = true;
-         QString orig = decodeInput(s->argv[i]);
-         QString option = orig.mid(1);
+         QByteArray orig = s->argv[i];
+         QByteArray option = orig.mid(1);
          if (option.startsWith('-'))
          {
             option = option.mid(1);
@@ -878,19 +887,25 @@ KCmdLineArgsStatic::parseAllArgs()
          }
          else if (option.startsWith("help-"))
          {
-            KCmdLineArgs::usage(option.mid(5).toUtf8());
+            KCmdLineArgs::usage(option.mid(5));
          }
          else if ((option == "version") || (option == "v"))
          {
-            s->printQ( QString("Qt: %1\n").arg(qVersion()));
-            s->printQ( QString("KDE: %1\n").arg(KDE_VERSION_STRING));
-            s->printQ( QString("%1: %2\n"). arg(s->about->programName()).arg(s->about->version()));
+            KCmdLineArgs::enable_i18n();
+            s->printQ(i18nc("@info:shell message on appcmd --version; "
+                            "%3 application name, other %n version strings",
+                            "Qt: %1\n"
+                            "KDE: %2\n"
+                            "%3: %4\n",
+                            QString::fromLatin1(qVersion()),
+                            QString::fromLatin1(KDE_VERSION_STRING),
+                            s->about->programName(), s->about->version()));
             exit(0);
          } else if (option == "license")
          {
             KCmdLineArgs::enable_i18n();
-            s->printQ( s->about->license() );
-            s->printQ( "\n" );
+            s->printQ(s->about->license());
+            s->printQ(QString::fromLatin1("\n"));
             exit(0);
          } else if (option == "author") {
              KCmdLineArgs::enable_i18n();
@@ -901,8 +916,8 @@ KCmdLineArgsStatic::parseAllArgs()
            for (QList<KAboutPerson>::ConstIterator it = authors.begin(); it != authors.end(); ++it ) {
              QString email;
              if ( !(*it).emailAddress().isEmpty() )
-               email = " &lt;" + (*it).emailAddress() + "&gt;";
-             authorlist += QString("    ") + (*it).name() + email + '\n';
+               email = QString::fromLatin1(" &lt;") + (*it).emailAddress() + QLatin1String("&gt;");
+             authorlist += QString::fromLatin1("    ") + (*it).name() + email + QLatin1Char('\n');
            }
            s->printQ( i18nc("the 2nd argument is a list of name+address, one on each line","%1 was written by\n%2",   QString(s->about->programName()) ,  authorlist ) );
          }
@@ -913,11 +928,10 @@ KCmdLineArgsStatic::parseAllArgs()
        {
          if (!s->about->customAuthorTextEnabled ())
          {
-           if (s->about->bugAddress().isEmpty() || s->about->bugAddress() == "submit@bugs.kde.org" )
+           if (s->about->bugAddress().isEmpty() || s->about->bugAddress() == QLatin1String("submit@bugs.kde.org") )
              s->printQ( i18n( "Please use http://bugs.kde.org to report bugs.\n" ) );
-           else {
+           else
              s->printQ( i18n( "Please report bugs to %1.\n" , s->about->bugAddress()) );
-           }
          }
          else
          {
@@ -942,11 +956,11 @@ KCmdLineArgsStatic::parseAllArgs()
             if (s->ignoreUnknown)
                continue;
             KCmdLineArgs::enable_i18n();
-            KCmdLineArgs::usageError(i18n("Unexpected argument '%1'.", s->decodeInput(s->argv[i])));
+            KCmdLineArgs::usageError(i18n("Unexpected argument '%1'.", KuitSemantics::escape(s->decodeInput(s->argv[i]))));
          }
          else
          {
-            appOptions->d->addArgument(s->decodeInput(s->argv[i]));
+            appOptions->d->addArgument(s->argv[i]);
             if (everythingAfterArgIsArgs)
                 inOptions = false;
          }
@@ -1000,7 +1014,7 @@ KCmdLineArgs::qtArgv()
    if (!(s->mStdargs & KCmdLineArgs::CmdLineArgQt))
    {
      s_qt_argv = new char*[2];
-     s_qt_argv[0] = qstrdup(s->encodeOutput(appName()));
+     s_qt_argv[0] = qstrdup(s->argc?s->argv[0]:"");
      s_qt_argv[1] = 0;
 
      return s_qt_argv;
@@ -1017,12 +1031,13 @@ KCmdLineArgs::qtArgv()
       exit(255);
    }
 
-   s_qt_argv = new char*[ args->count() + 2 ];
-   s_qt_argv[0] = qstrdup(s->encodeOutput(appName()));
+   int count=args->count();
+   s_qt_argv = new char*[ count + 2 ];
+   s_qt_argv[0] = qstrdup(s->argc?s->argv[0]:"");
    int i = 0;
-   for(; i < args->count(); i++)
+   for(; i < count; i++)
    {
-      s_qt_argv[i+1] = qstrdup(s->encodeOutput(args->arg(i)));
+      s_qt_argv[i+1] = qstrdup(args->d->parsedArgList->at(i));
    }
    s_qt_argv[i+1] = 0;
 
@@ -1071,8 +1086,8 @@ KCmdLineArgs::usage(const QByteArray &id)
    Q_ASSERT(s->argsList != 0); // It's an error to call usage(...) without
                                // having done addCmdLineOptions first!
 
-   QString optionFormatString   = "  %1 %2\n";
-   QString optionFormatStringDef  = "  %1 %2 [%3]\n";
+   QString optionFormatString = QString::fromLatin1("  %1 %2\n");
+   QString optionFormatStringDef = QString::fromLatin1("  %1 %2 [%3]\n");
    QString tmp;
    QString usage;
 
@@ -1088,7 +1103,7 @@ KCmdLineArgs::usage(const QByteArray &id)
    {
       if (!(*args)->d->name.isEmpty())
       {
-         usage = i18n("[%1-options]", (*args)->d->name.toString())+' '+usage;
+         usage = i18n("[%1-options]", (*args)->d->name.toString())+QLatin1Char(' ')+usage;
       }
       if (args == s->argsList->begin())
          break;
@@ -1101,26 +1116,27 @@ KCmdLineArgs::usage(const QByteArray &id)
      const KCmdLineOptions &option = appOptions->d->options;
      for (int i = 0; i < option.d->names.size(); i++)
      {
-       QString opt_name = option.d->names[i];
+       QByteArray opt_name = option.d->names[i];
        if (opt_name.startsWith('+'))
-          usage = usage + (opt_name.mid(1)) + ' ';
+           usage += QString::fromLatin1(opt_name.mid(1)) + QLatin1Char(' ');
        else if ( opt_name.startsWith("!+") )
-          usage = usage + (opt_name.mid(2)) + ' ';
+          usage += QString::fromLatin1(opt_name.mid(2)) + QLatin1Char(' ');
      }
    }
 
-   s->printQ(i18n("Usage: %1 %2\n", s->argv[0], usage));
-   s->printQ('\n'+s->about->shortDescription()+'\n');
+   s->printQ(i18n("Usage: %1 %2\n", QString::fromLocal8Bit(s->argv[0]), KuitSemantics::escape(usage)));
+   s->printQ(QLatin1Char('\n')+s->about->shortDescription()+QLatin1Char('\n'));
 
    s->printQ(i18n("\nGeneric options:\n"));
-   s->printQ(optionFormatString.arg("--help", -25).arg(i18n("Show help about options")));
+   s->printQ(optionFormatString.arg(QString::fromLatin1("--help"), -25)
+             .arg(i18n("Show help about options")));
 
    args = s->argsList->begin();
    while(args != s->argsList->end())
    {
       if (!(*args)->d->name.isEmpty() && !(*args)->d->id.isEmpty())
       {
-         QString option = QString("--help-%1").arg((*args)->d->id);
+          QString option = QString::fromLatin1("--help-%1").arg(QString::fromLatin1((*args)->d->id));
          QString desc = i18n("Show %1 specific options", (*args)->d->name.toString());
 
          s->printQ(optionFormatString.arg(option, -25).arg(desc));
@@ -1128,11 +1144,11 @@ KCmdLineArgs::usage(const QByteArray &id)
       ++args;
    }
 
-   s->printQ(optionFormatString.arg("--help-all",-25).arg(i18n("Show all options")));
-   s->printQ(optionFormatString.arg("--author",-25).arg(i18n("Show author information")));
-   s->printQ(optionFormatString.arg("-v, --version",-25).arg(i18n("Show version information")));
-   s->printQ(optionFormatString.arg("--license",-25).arg(i18n("Show license information")));
-   s->printQ(optionFormatString.arg("--", -25).arg(i18n("End of options")));
+   s->printQ(optionFormatString.arg(QString::fromLatin1("--help-all"),-25).arg(i18n("Show all options")));
+   s->printQ(optionFormatString.arg(QString::fromLatin1("--author"),-25).arg(i18n("Show author information")));
+   s->printQ(optionFormatString.arg(QString::fromLatin1("-v, --version"),-25).arg(i18n("Show version information")));
+   s->printQ(optionFormatString.arg(QString::fromLatin1("--license"),-25).arg(i18n("Show license information")));
+   s->printQ(optionFormatString.arg(QString::fromLatin1("--"), -25).arg(i18n("End of options")));
 
    args = s->argsList->begin(); // Sets current to 1st.
 
@@ -1160,7 +1176,7 @@ KCmdLineArgs::usage(const QByteArray &id)
      while (args != s->argsList->end())
      {
        const KCmdLineOptions &option = (*args)->d->options;
-       QString opt;
+       QByteArray opt;
 
        for (int i = 0; i < option.d->names.size(); i++)
        {
@@ -1177,9 +1193,9 @@ KCmdLineArgs::usage(const QByteArray &id)
          {
             if (!descriptionFull.isEmpty())
             {
-               optionsHeader = '\n'+descriptionFull;
-               if (!optionsHeader.endsWith('\n'))
-                  optionsHeader.append('\n');
+               optionsHeader = QLatin1Char('\n')+descriptionFull;
+               if (!optionsHeader.endsWith(QLatin1Char('\n')))
+                  optionsHeader.append(QLatin1Char('\n'));
                hasOptions = false;
             }
             continue;
@@ -1190,9 +1206,9 @@ KCmdLineArgs::usage(const QByteArray &id)
          {
             if (!descriptionFull.isEmpty())
             {
-               tmp = '\n'+descriptionFull;
-               if (!tmp.endsWith('\n'))
-                  tmp.append('\n');
+               tmp = QLatin1Char('\n')+descriptionFull;
+               if (!tmp.endsWith(QLatin1Char('\n')))
+                  tmp.append(QLatin1Char('\n'));
                s->printQ(tmp);
             }
             continue;
@@ -1201,11 +1217,11 @@ KCmdLineArgs::usage(const QByteArray &id)
          // Options
          if (!descriptionFull.isEmpty())
          {
-            dl = descriptionFull.split( "\n", QString::KeepEmptyParts);
+            dl = descriptionFull.split(QLatin1Char('\n'), QString::KeepEmptyParts);
             description = dl.first();
             dl.erase( dl.begin() );
          }
-         QString name = option.d->names[i];
+         QByteArray name = option.d->names[i];
          if (name.startsWith('!'))
              name = name.mid(1);
 
@@ -1220,7 +1236,7 @@ KCmdLineArgs::usage(const QByteArray &id)
             name = name.mid(1);
             if (name.startsWith('[') && name.endsWith(']'))
                 name = name.mid(1, name.length()-2);
-            s->printQ(optionFormatString.arg(name, -25).arg(description));
+            s->printQ(optionFormatString.arg(QString::fromLocal8Bit(name), -25).arg(description));
          }
          else
          {
@@ -1243,21 +1259,21 @@ KCmdLineArgs::usage(const QByteArray &id)
                opt = opt + name;
                if (option.d->defaults[i].isEmpty())
                {
-                  s->printQ(optionFormatString.arg(QString( opt ), -25).arg(description));
+                   s->printQ(optionFormatString.arg(QString::fromLatin1(opt), -25).arg(description));
                }
                else
                {
-                  s->printQ(optionFormatStringDef.arg(QString( opt ), -25)
+                   s->printQ(optionFormatStringDef.arg(QString::fromLatin1(opt), -25)
                             .arg(description, option.d->defaults[i]));
                }
-               opt = "";
+               opt.clear();
             }
          }
          for(QStringList::Iterator it = dl.begin();
              it != dl.end();
              ++it)
          {
-            s->printQ(optionFormatString.arg("", -25).arg(*it));
+            s->printQ(optionFormatString.arg(QString(), -25).arg(*it));
          }
        }
 
@@ -1300,24 +1316,21 @@ KCmdLineArgs::~KCmdLineArgs()
 void
 KCmdLineArgs::setCwd( const QByteArray &cwd )
 {
-    s->mCwd = QString::fromUtf8(cwd);
+    s->mCwd = cwd;
 }
 
 void
 KCmdLineArgs::clear()
 {
-   delete d->parsedArgList;
-   d->parsedArgList = 0;
-   delete d->parsedOptionList;
-   d->parsedOptionList = 0;
+   delete d->parsedArgList;     d->parsedArgList = 0;
+   delete d->parsedOptionList;  d->parsedOptionList = 0;
 }
 
 void
 KCmdLineArgs::reset()
 {
    if ( s->argsList ) {
-      delete s->argsList;
-      s->argsList = 0;
+      delete s->argsList; s->argsList = 0;
    }
    s->parsed = false;
 }
@@ -1347,23 +1360,21 @@ KCmdLineArgsPrivate::load( QDataStream &ds)
 
    if (parsedOptionList->count() == 0)
    {
-      delete parsedOptionList;
-      parsedOptionList = 0;
+      delete parsedOptionList;  parsedOptionList = 0;
    }
    if (parsedArgList->count() == 0)
    {
-      delete parsedArgList;
-      parsedArgList = 0;
+      delete parsedArgList;     parsedArgList = 0;
    }
 }
 
 void
-KCmdLineArgsPrivate::setOption(const QString &opt, bool enabled)
+KCmdLineArgsPrivate::setOption(const QByteArray &opt, bool enabled)
 {
    if (isQt)
    {
       // Qt does it own parsing.
-      QString argString = "-";
+      QByteArray argString = "-";
       if( !enabled )
           argString += "no";
       argString += opt;
@@ -1374,18 +1385,18 @@ KCmdLineArgsPrivate::setOption(const QString &opt, bool enabled)
    }
 
    if (enabled)
-      parsedOptionList->insert( opt, QString::fromUtf8("t") );
+      parsedOptionList->insert( opt, "t" );
    else
-      parsedOptionList->insert( opt, QString::fromUtf8("f") );
+      parsedOptionList->insert( opt, "f" );
 }
 
 void
-KCmdLineArgsPrivate::setOption(const QString &opt, const QString &value)
+KCmdLineArgsPrivate::setOption(const QByteArray &opt, const QByteArray &value)
 {
    if (isQt)
    {
       // Qt does it's own parsing.
-      QString argString = "-";
+      QByteArray argString = "-";
       argString += opt;
       addArgument(argString);
       addArgument(value);
@@ -1394,7 +1405,7 @@ KCmdLineArgsPrivate::setOption(const QString &opt, const QString &value)
       // Hack coming up!
       if (argString == "-display")
       {
-         setenv(DISPLAY, s->encodeOutput(value).data(), true);
+         setenv(DISPLAY, value.data(), true);
       }
 #endif
    }
@@ -1408,17 +1419,17 @@ KCmdLineArgsPrivate::setOption(const QString &opt, const QString &value)
 QString
 KCmdLineArgs::getOption(const QByteArray &_opt) const
 {
-   QString opt = QString::fromUtf8(_opt);
-   QString value;
+   QByteArray opt = _opt;
+   QByteArray value;
    if (d->parsedOptionList)
    {
       value = d->parsedOptionList->value(opt);
    }
    if (!value.isEmpty())
-      return value;
+       return QString::fromLocal8Bit(value);
 
    // Look up the default.
-   QString opt_name;
+   QByteArray opt_name;
    QString def;
    bool dummy = true;
    int result = s->findOption( d->options, opt, opt_name, def, dummy) & ~4;
@@ -1427,7 +1438,7 @@ KCmdLineArgs::getOption(const QByteArray &_opt) const
    {
       fprintf(stderr, "\n\nFAILURE (KCmdLineArgs):\n");
       fprintf(stderr, "Application requests for getOption(\"%s\") but the \"%s\" option\n",
-                      s->encodeOutput(opt).data(), s->encodeOutput(opt).data());
+                      opt.data(), opt.data());
       fprintf(stderr, "has never been specified via addCmdLineOptions( ... )\n\n");
 
       Q_ASSERT( 0 );
@@ -1437,20 +1448,18 @@ KCmdLineArgs::getOption(const QByteArray &_opt) const
 }
 
 QStringList
-KCmdLineArgs::getOptionList(const QByteArray &_opt) const
+KCmdLineArgs::getOptionList(const QByteArray &opt) const
 {
-   QString opt = QString::fromUtf8(_opt);
-
    QStringList result;
    if (!d->parsedOptionList)
       return result;
 
    while(true)
    {
-      QString value = d->parsedOptionList->take(opt);
+      QByteArray value = d->parsedOptionList->take(opt);
       if (value.isEmpty())
          break;
-      result.prepend(value);
+      result.prepend(QString::fromLocal8Bit(value));
    }
 
    // Reinsert items in dictionary
@@ -1458,11 +1467,9 @@ KCmdLineArgs::getOptionList(const QByteArray &_opt) const
    // to the API like "you can only call this function once".
    // I can't access all items without taking them out of the list.
    // So taking them out and then putting them back is the only way.
-   for(QStringList::ConstIterator it=result.begin();
-       it != result.end();
-       ++it)
+   Q_FOREACH(const QString &str, result)
    {
-      d->parsedOptionList->insertMulti(opt, *it);
+      d->parsedOptionList->insertMulti(opt, str.toLocal8Bit());
    }
    return result;
 }
@@ -1471,8 +1478,8 @@ bool
 KCmdLineArgs::isSet(const QByteArray &_opt) const
 {
    // Look up the default.
-   QString opt = QString::fromUtf8(_opt);
-   QString opt_name;
+   QByteArray opt = _opt;
+   QByteArray opt_name;
    QString def;
    int result = 0;
    KCmdLineArgsList::Iterator args = s->argsList->begin();
@@ -1488,14 +1495,14 @@ KCmdLineArgs::isSet(const QByteArray &_opt) const
    {
       fprintf(stderr, "\n\nFAILURE (KCmdLineArgs):\n");
       fprintf(stderr, "Application requests for isSet(\"%s\") but the \"%s\" option\n",
-                      s->encodeOutput(opt).data(), s->encodeOutput(opt).data());
+                      opt.data(), opt.data());
       fprintf(stderr, "has never been specified via addCmdLineOptions( ... )\n\n");
 
       Q_ASSERT( 0 );
       exit(255);
    }
 
-   QString value;
+   QByteArray value;
    if (d->parsedOptionList)
    {
       value = d->parsedOptionList->value(opt);
@@ -1506,7 +1513,7 @@ KCmdLineArgs::isSet(const QByteArray &_opt) const
       if (result == 3)
          return true;
       else
-         return (value[0] == 't');
+         return (value.at(0) == 't');
    }
 
    if (result == 3)
@@ -1520,9 +1527,7 @@ KCmdLineArgs::isSet(const QByteArray &_opt) const
 int
 KCmdLineArgs::count() const
 {
-   if (!d->parsedArgList)
-      return 0;
-   return d->parsedArgList->count();
+   return d->parsedArgList?d->parsedArgList->count():0;
 }
 
 QString
@@ -1538,7 +1543,7 @@ KCmdLineArgs::arg(int n) const
       exit(255);
    }
 
-   return d->parsedArgList->at(n);
+   return QString::fromLocal8Bit(d->parsedArgList->at(n));
 }
 
 KUrl
@@ -1553,13 +1558,13 @@ KUrl KCmdLineArgs::makeURL(const QByteArray &_urlArg)
     QFileInfo fileInfo(urlArg);
     if (!fileInfo.isRelative()) { // i.e. starts with '/', on unix
         KUrl result;
-        result.setPath(urlArg);
+        result.setPath(QDir::fromNativeSeparators(urlArg));
         return result; // Absolute path.
     }
 
     if ( KUrl::isRelativeUrl(urlArg) || fileInfo.exists() ) {
         KUrl result;
-        result.setPath( cwd()+'/'+urlArg );
+        result.setPath(cwd()+QLatin1Char('/')+urlArg);
         result.cleanPath();
         return result;  // Relative path
     }
@@ -1568,7 +1573,7 @@ KUrl KCmdLineArgs::makeURL(const QByteArray &_urlArg)
 }
 
 void
-KCmdLineArgsPrivate::addArgument(const QString &argument)
+KCmdLineArgsPrivate::addArgument(const QByteArray &argument)
 {
    if (!parsedArgList)
       parsedArgList = new KCmdLineParsedArgs;
@@ -1587,7 +1592,5 @@ KCmdLineArgs::addTempFileOption()
 bool KCmdLineArgs::isTempFileSet()
 {
     KCmdLineArgs* args = KCmdLineArgs::parsedArgs( "kde-tempfile" );
-    if ( args )
-        return args->isSet( "tempfile" );
-    return false;
+    return args && args->isSet( "tempfile" );
 }
