@@ -38,13 +38,6 @@ QTEST_KDEMAIN_CORE( KUrlTest )
 #include <stdio.h>
 #include <stdlib.h>
 
-// needed to build a valid path on windows
-#ifdef Q_OS_WIN
-#define LOCAL_FILE_PREFIX "C:"
-#else
-#define LOCAL_FILE_PREFIX
-#endif
-
 void KUrlTest::testEmptyURL()
 {
   KUrl emptyURL;
@@ -139,6 +132,7 @@ void KUrlTest::testSetHTMLRef()
   QCOMPARE( url1.htmlRef(), QString("kde=rocks&a=b") );
   url1.setHTMLRef( "#" );
   QCOMPARE( url1.htmlRef(), QString("#") );
+  QCOMPARE( url1.ref(), QString("%23") ); // it's encoded
   url1.setHTMLRef( "" );
   QCOMPARE( url1.htmlRef(), QString("") );
   url1.setHTMLRef( QString() );
@@ -233,8 +227,12 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
 
   u1 = "file:///home/dfaure/my#%2f";
   url1 = u1;
-  // KDE3: was %2f, but this is OK too
+  // KDE3: was %2f, Qt-4.0 to 4.4: #/, bad. 4.5: %2f again, good
+#if QT_VERSION < 0x040500
   QCOMPARE( url1.url(), QString("file:///home/dfaure/my#/") );
+#else
+  QCOMPARE( url1.url(), QString("file:///home/dfaure/my#%2f") );
+#endif
   QVERIFY( url1.hasRef() );
   QVERIFY( url1.hasHTMLRef() );
   QVERIFY( !url1.hasSubUrl() );
@@ -278,6 +276,7 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
   QVERIFY( url1.hasRef() );
   QVERIFY( url1.hasHTMLRef() );
   QVERIFY( !url1.hasSubUrl() );
+  QCOMPARE( url1.ref(), QString("QObject%3A%3Aconnect") );
   QCOMPARE( url1.htmlRef(), QString("QObject::connect") );
   QCOMPARE( url1.upUrl().url(), QString("file:///opt/kde2/qt2/doc/html/") );
 
@@ -287,6 +286,7 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
   QVERIFY( url1.hasRef() );
   QVERIFY( url1.hasHTMLRef() );
   QVERIFY( !url1.hasSubUrl() );
+  QCOMPARE( url1.ref(), QString("QObject%3A%3Aconnect") );
   QCOMPARE( url1.htmlRef(), QString("QObject::connect") );
   QCOMPARE( url1.upUrl().url(), QString("file:///opt/kde2/qt2/doc/html/") );
 
@@ -296,6 +296,7 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
   QVERIFY( url1.hasRef() );
   QVERIFY( url1.hasHTMLRef() );
   QVERIFY( !url1.hasSubUrl() );
+  QCOMPARE( url1.ref(), QString("QObject%3Aconnect") );
   QCOMPARE( url1.htmlRef(), QString("QObject:connect") );
   QCOMPARE( url1.upUrl().url(), QString("file:///opt/kde2/qt2/doc/html/") );
 
@@ -319,6 +320,10 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
   KUrl charles2("file:/home/charles/foo%20moo");
   QCOMPARE( charles2.path(), QString("/home/charles/foo moo") );
 
+  //NOTE this test should be ran in UTF8 locale
+  KUrl percentEncodedQuery( "http://mail.yandex.ru/message_part/%D0%9A%D1%80%D0%B8%D1%82%D0%B5%D1%80%D0%B8%D0%B8%20%D0%BE%D1%86%D0%B5%D0%BD%D0%B8%D0%B2%D0%B0%D0%BD%D0%B8%D1%8F%20%D0%BE%D1%80%D0%BB%D0%BE%D0%B2%D0%BE%D0%B9.rar?hid=1.1&mid=391.56424458.99241672611486679803334485488&name=%D0%9A%D1%80%D0%B8%D1%82%D0%B5%D1%80%D0%B8%D0%B8%20%D0%BE%D1%86%D0%B5%D0%BD%D0%B8%D0%B2%D0%B0%D0%BD%D0%B8%D1%8F%20%D0%BE%D1%80%D0%BB%D0%BE%D0%B2%D0%BE%D0%B9.rar" );
+  QCOMPARE( percentEncodedQuery.prettyUrl(), QString::fromUtf8("http://mail.yandex.ru/message_part/Критерии оценивания орловой.rar?hid=1.1&mid=391.56424458.99241672611486679803334485488&name=Критерии оценивания орловой.rar"));
+
 #ifdef Q_WS_WIN
 #ifdef Q_CC_MSVC
 #pragma message ("port KUser")
@@ -502,6 +507,12 @@ void KUrlTest::testPathAndQuery()
   KUrl tobi0("http://some.host.net/path/to/file#fragmentPrecedes?theQuery");
   QCOMPARE( tobi0.ref(), QString("fragmentPrecedes") );
   QCOMPARE( tobi0.query(), QString("?theQuery") );
+#else
+  // So we treat it as part of the fragment
+  KUrl tobi0("http://some.host.net/path/to/file#foo?bar");
+  QCOMPARE( tobi0.ref(), QString("foo%3Fbar") );
+  QCOMPARE( tobi0.query(), QString() );
+  QCOMPARE( tobi0.prettyUrl(), QString("http://some.host.net/path/to/file#foo?bar") );
 #endif
 
   KUrl tobi1( "http://host.net/path?myfirstquery#andsomeReference" );
@@ -527,6 +538,18 @@ void KUrlTest::testPathAndQuery()
   KUrl uloc( "file:///home/dfaure/konqtests/Mat%C3%A9riel" );
   QCOMPARE( uloc.encodedPathAndQuery(), QString( "/home/dfaure/konqtests/Mat%C3%A9riel" ) );
 
+  KUrl urlWithAccent( "file:///home/dfaure/konqtests/Matériel" );
+  QCOMPARE(urlWithAccent.encodedPathAndQuery(), QString("/home/dfaure/konqtests/Mat%C3%A9riel"));
+
+  KUrl urlWithUnicodeChar(QString::fromUtf8("file:///home/dfaure/konqtests/Matériel"));
+  QCOMPARE(urlWithUnicodeChar.encodedPathAndQuery(), QString("/home/dfaure/konqtests/Mat%C3%A9riel"));
+
+  KUrl maelcum(QString::fromUtf8("http://a.b.c/äöu"));
+  QCOMPARE(maelcum.encodedPathAndQuery(), QString("/%C3%A4%C3%B6u"));
+}
+
+void KUrlTest::testUpUrl()
+{
   KUrl url1( "ftp://user%40host.com@ftp.host.com/var/www/" );
   QCOMPARE( url1.user(), QString("user@host.com" ) );
   QCOMPARE( url1.host(), QString("ftp.host.com" ) );
@@ -536,6 +559,12 @@ void KUrlTest::testPathAndQuery()
   QCOMPARE( up.url(), QString("ftp://user%40host.com@ftp.host.com/") );
   up = up.upUrl();
   QCOMPARE( up.url(), QString("ftp://user%40host.com@ftp.host.com/") ); // unchanged
+
+  // Going up from a relative url is not supported (#170695)
+  KUrl invalid("tmp");
+  QVERIFY(invalid.isValid());
+  up = invalid.upUrl();
+  QVERIFY(!up.isValid());
 }
 
 void KUrlTest::testSetFileName() // and addPath
@@ -596,6 +625,14 @@ void KUrlTest::testSetFileName() // and addPath
   KUrl lastUrl = lst.last();
   QString dir = lastUrl.directory();
   QCOMPARE(  dir, QString("/dir1/dir2") );
+
+  // files without directories
+  KUrl singleFile( "foo.txt" );
+  QCOMPARE( singleFile.path(), QString("foo.txt") );
+  QCOMPARE( singleFile.pathOrUrl(), QString("foo.txt") );
+  singleFile.setFileName( "bar.bin" );
+  QCOMPARE( singleFile.path(), QString("bar.bin") );
+  QCOMPARE( singleFile.pathOrUrl(), QString("bar.bin") );
 }
 
 void KUrlTest::testDirectory()
@@ -666,6 +703,29 @@ void KUrlTest::testPrettyURL()
 
   KUrl xmppUri("xmpp:ogoffart@kde.org");
   QCOMPARE( xmppUri.prettyUrl(), QString::fromLatin1( "xmpp:ogoffart@kde.org" ) );
+
+  QUrl offEagleqUrl;
+  offEagleqUrl.setEncodedUrl("http://www.sejlsport.dk/Pr%F8v%20noget%20nyt%20dokumenter.pdf", QUrl::TolerantMode);
+  const QString offEaglePath = offEagleqUrl.path();
+  QCOMPARE((int)offEaglePath.at(2).unicode(), (int)'r');
+#if 0 // CURRENTLY BROKEN, PENDING PRETTYURL REWRITE AND QT-4.5 (in thiago's hands)
+  QCOMPARE((int)offEaglePath.at(3).unicode(), (int)0xf8);
+
+  KUrl offEagle("http://www.sejlsport.dk/graphics/ds/DSUngdom/PDF/Pr%F8v%20noget%20nyt%20dokumenter/Invitation_Kerteminde_11.07.08.pdf");
+  QCOMPARE(offEagle.path(), QString::fromLatin1("/graphics/ds/DSUngdom/PDF/Pr%F8v noget nyt dokumenter/Invitation_Kerteminde_11.07.08.pdf"));
+  QCOMPARE(offEagle.url(), QString::fromLatin1("http://www.sejlsport.dk/graphics/ds/DSUngdom/PDF/Pr%F8v%20noget%20nyt%20dokumenter/Invitation_Kerteminde_11.07.08.pdf"));
+  QCOMPARE(offEagle.prettyUrl(), QString::fromLatin1("http://www.sejlsport.dk/graphics/ds/DSUngdom/PDF/Pr%F8v noget nyt dokumenter/Invitation_Kerteminde_11.07.08.pdf"));
+#endif
+
+  KUrl openWithUrl("kate --use %25U");
+  QCOMPARE(openWithUrl.url(), QString::fromLatin1("kate%20--use%20%25U"));
+  QCOMPARE(openWithUrl.prettyUrl(), QString::fromLatin1("kate --use %25U"));
+  QCOMPARE(openWithUrl.path(), QString::fromLatin1("kate --use %U"));
+  QCOMPARE(openWithUrl.pathOrUrl(), QString::fromLatin1("kate --use %25U")); // caused #153894; better not use KUrl for this.
+
+  KUrl ipv6Address( "http://[::ffff:129.144.52.38]:81/index.html" );
+  QCOMPARE( ipv6Address.prettyUrl(), QString::fromLatin1( "http://[::ffff:129.144.52.38]:81/index.html" ) );
+
 }
 
 void KUrlTest::testIsRelative()
@@ -681,6 +741,13 @@ void KUrlTest::testIsRelative()
   QCOMPARE(something.url(), QString("something"));
   QCOMPARE(something.protocol(), QString());
   QVERIFY(!something.isLocalFile());
+
+  // Now let's test QUrl::isRelative.
+  QVERIFY(!KUrl("file:///blah").isRelative());
+  QVERIFY(!KUrl("/blah").isRelative());
+  QVERIFY(KUrl("blah").isRelative());
+  QVERIFY(!KUrl("http://www.kde.org").isRelative());
+  QVERIFY(KUrl("foo/bar").isRelative());
 }
 
 void KUrlTest::testRelativePath()
@@ -879,9 +946,6 @@ void KUrlTest::testBaseURL() // those are tests for the KUrl(base,relative) cons
   KUrl rel_url2( baseURL, "mailto:bastian@kde.org" );
   QCOMPARE( rel_url2.url(), QString("mailto:bastian@kde.org" ));
 
-  baseURL = "mailto:bastian@kde.org?subject=hello";
-  QCOMPARE( baseURL.url(), QString("mailto:bastian@kde.org?subject=hello" ));
-
   baseURL = "file:/usr/local/src/kde2/kdelibs/kio/";
   KUrl url2( baseURL, "../../////kdebase/konqueror" );
   QCOMPARE( url2.url(), QString("file:///usr/local/src/kde2/////kdebase/konqueror" ));
@@ -958,6 +1022,10 @@ void KUrlTest::testBaseURL() // those are tests for the KUrl(base,relative) cons
      KUrl waba2( waba1, "relative.html");
      QCOMPARE( waba2.url(), QString("http://www.website.com/directory/relative.html") );
   }
+  {
+     KUrl waba2( waba1, "./relative.html");
+     QCOMPARE( waba2.url(), QString("http://www.website.com/directory/relative.html") );
+  }
   {
      KUrl waba2( waba1, "../relative.html");
      QCOMPARE( waba2.url(), QString("http://www.website.com/relative.html") );
@@ -966,6 +1034,10 @@ void KUrlTest::testBaseURL() // those are tests for the KUrl(base,relative) cons
      KUrl waba2( waba1, "down/relative.html");
      QCOMPARE( waba2.url(), QString("http://www.website.com/directory/down/relative.html") );
   }
+  {
+     KUrl waba2( waba1, "down/./relative.html");
+     QCOMPARE( waba2.url(), QString("http://www.website.com/directory/down/relative.html") );
+  }
   {
      KUrl waba2( waba1, "/down/relative.html");
      QCOMPARE( waba2.url(), QString("http://www.website.com/down/relative.html") );
@@ -1016,6 +1088,25 @@ void KUrlTest::testBaseURL() // those are tests for the KUrl(base,relative) cons
   //QEXPECT_FAIL("","Issue N183630, task ID 183874", Continue); // Fixed by _setEncodedUrl
   QCOMPARE(dxOffEagle.url(), QString("http://something/newpage.html?%5B%7B%22foo:%20bar%22%7D%5D") );
 
+#if QT_VERSION >= 0x040500 // QtSw issue 243557
+  QByteArray tsdgeos("http://google.com/c?c=Translation+%C2%BB+trunk|");
+  QUrl tsdgeosQUrl;
+  tsdgeosQUrl.setEncodedUrl(tsdgeos, QUrl::TolerantMode);
+  QVERIFY(tsdgeosQUrl.isValid()); // failed in Qt-4.4, works in Qt-4.5
+  QByteArray tsdgeosExpected("http://google.com/c?c=Translation+%C2%BB+trunk%7C");
+  //QCOMPARE(tsdgeosQUrl.toEncoded(), tsdgeosExpected); // unusable output from qtestlib...
+  QCOMPARE(QString(tsdgeosQUrl.toEncoded()), QString(tsdgeosExpected));
+
+  KUrl tsdgeosUrl(tsdgeos);
+  QCOMPARE(tsdgeosUrl.url(), QString(tsdgeosExpected));
+
+  QByteArray pipesAgain("http://translate.google.com/translate_t#en|uk|demo");
+  QUrl pipesUrl;
+  pipesUrl.setEncodedUrl(pipesAgain, QUrl::TolerantMode);
+  QVERIFY(pipesUrl.isValid());
+  QCOMPARE(QString(pipesUrl.toEncoded()), QString("http://translate.google.com/translate_t#en%7Cuk%7Cdemo"));
+#endif
+
   // Shows up in nspluginviewer/flash
   QString flashRel = "javascript:window.location+\"__flashplugin_unique__\"";
   KUrl flashUrl(flashRel);
@@ -1025,11 +1116,46 @@ void KUrlTest::testBaseURL() // those are tests for the KUrl(base,relative) cons
   QCOMPARE(flashComposed.url(), QString("javascript:window.location+%22__flashplugin_unique__%22"));
 }
 
+void KUrlTest::testSetEncodedFragment_data()
+{
+    QTest::addColumn<QByteArray>("base");
+    QTest::addColumn<QByteArray>("fragment");
+    QTest::addColumn<QByteArray>("expected");
+    typedef QByteArray BA;
+    QTest::newRow("basic test") << BA("http://www.kde.org") << BA("abc") << BA("http://www.kde.org#abc");
+    QTest::newRow("initial url has fragment") << BA("http://www.kde.org#old") << BA("new") << BA("http://www.kde.org#new");
+    QTest::newRow("encoded fragment") << BA("http://www.kde.org") << BA("a%20c") << BA("http://www.kde.org#a%20c");
+    QTest::newRow("with #") << BA("http://www.kde.org") << BA("a#b") << BA("http://www.kde.org#a#b");
+    QTest::newRow("empty") << BA("http://www.kde.org") << BA("") << BA("http://www.kde.org#");
+}
+
+void KUrlTest::testSetEncodedFragment()
+{
+    // QtSw task number: TODO (mail sent)
+    // Bug fixed in 4.5.1 by Thiago
+    QSKIP("Bug in Qt-4.4/4.5-rc1: setEncodedFragment doesn't work if the initial url has no fragment", SkipAll);
+
+    QFETCH(QByteArray, base);
+    QFETCH(QByteArray, fragment);
+    QFETCH(QByteArray, expected);
+    QUrl u;
+    u.setEncodedUrl(base, QUrl::TolerantMode);
+    QVERIFY(u.isValid());
+    u.setEncodedFragment(fragment);
+    QVERIFY(u.isValid());
+    QCOMPARE(QString::fromLatin1(u.toEncoded()), QString::fromLatin1(expected));
+}
+
 void KUrlTest::testSubURL()
 {
   QString u1 = "file:/home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/#myref";
   KUrl url1(u1);
-  QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/%23tar:/%23myref") ); // KDE3 would say # 3 times
+  // KDE3: was #,#,#, Qt-4.0 to 4.4: #,%23,%23 . 4.5: #,#,#, good
+#if QT_VERSION < 0x040500
+  QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/%23tar:/%23myref") );
+#else
+  QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/#myref") );
+#endif
   QVERIFY( url1.hasRef() );
   QVERIFY( !url1.isLocalFile() );  // Not strictly local!
   QVERIFY( url1.hasSubUrl() );
@@ -1043,12 +1169,11 @@ void KUrlTest::testSubURL()
   QCOMPARE( splitted[1].url(), QString("gzip:/#myref") );
   QCOMPARE( splitted[2].url(), QString("tar:/#myref") );
 
-  QSKIP( "Multiple sub urls not supported at the moment, due to #-escaping", SkipSingle );
-
-  // FAILS, due to the #->%23 escaping, see the comment in KUrl::join.
-  // But do we really want to keep the suburl stuff?
-  KUrl rejoined = KUrl::join( splitted );
-  QCOMPARE( rejoined.url(), url1.url() );
+#if QT_VERSION < 0x040500
+  QSKIP( "Multiple sub urls not supported with Qt < 4.5", SkipSingle );
+#endif
+  KUrl rejoined = KUrl::join(splitted);
+  QCOMPARE(rejoined.url(), url1.url());
 
   u1 = "error:/?error=14&errText=Unknown%20host%20asdfu.adgi.sdfgoi#http://asdfu.adgi.sdfgoi";
   url1 = u1;
@@ -1060,7 +1185,7 @@ void KUrlTest::testSubURL()
 
   u1 = "file:/home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/";
   url1 = u1;
-  QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/%23tar:/") ); // KDE3: #tar
+  QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/") );
   QVERIFY( url1.hasRef() );
   QVERIFY( !url1.hasHTMLRef() );
   QVERIFY( url1.hasSubUrl() );
@@ -1069,7 +1194,7 @@ void KUrlTest::testSubURL()
 
   u1 = "file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/";
   url1 = u1;
-  QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/%23tar:/") ); // KDE3: #tar
+  QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/") );
   QVERIFY( url1.hasRef() );
   QVERIFY( !url1.hasHTMLRef() );
   QVERIFY( url1.hasSubUrl() );
@@ -1090,12 +1215,22 @@ void KUrlTest::testSubURL()
 
   u1 = "file:/home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/README";
   url1 = u1;
-  QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/%23tar:/README") );
+  QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/README") );
   QVERIFY( url1.hasRef() );
   QVERIFY( !url1.hasHTMLRef() );
   QVERIFY( url1.hasSubUrl() );
   QCOMPARE( url1.htmlRef(), QString("") );
-  QCOMPARE( url1.upUrl().url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/%23tar:/") );
+  const KUrl::List url1Splitted = KUrl::split( url1 );
+  QCOMPARE( url1Splitted.count(), 3 );
+  kDebug() << url1Splitted.toStringList();
+  QCOMPARE(url1Splitted[0].url(), QString("file:///home/dfaure/my%20tar%20file.tgz"));
+  QCOMPARE(url1Splitted[1].url(), QString("gzip:/"));
+  QCOMPARE(url1Splitted[2].url(), QString("tar:/README"));
+  const KUrl url1Rejoined = KUrl::join(url1Splitted);
+  // Bug fixed in 4.5.1 by Thiago
+  QSKIP("Bug in Qt-4.4/4.5-rc1: setEncodedFragment doesn't work if the initial url has no fragment", SkipAll);
+  QCOMPARE(url1Rejoined.url(), url1.url());
+  QCOMPARE(url1.upUrl().url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/"));
 }
 
 void KUrlTest::testSetUser()
@@ -1107,9 +1242,7 @@ void KUrlTest::testSetUser()
 
   KUrl emptyUserTest1( "http://www.foobar.com/");
   QVERIFY( emptyUserTest1.user().isEmpty() );
-#if QT_VERSION >= 0x040400
   QVERIFY( emptyUserTest1.user().isNull() ); // Expected result. This was fixed in Qt-4.4
-#endif
   KUrl emptyUserTest2( "http://www.foobar.com/");
   emptyUserTest2.setUser( "" );
   //QVERIFY( emptyUserTest2.user().isNull() );
@@ -1177,6 +1310,7 @@ void KUrlTest::testStreaming()
 {
   // Streaming operators
   KUrl origURL( "http://www.website.com/directory/?#ref" );
+  KUrl accentuated(QString::fromUtf8("trash:/été"));
   KUrl empty( "" );
   KUrl invalid( "ptal://mlc:usb" );
   QVERIFY( !invalid.isValid() );
@@ -1185,6 +1319,7 @@ void KUrlTest::testStreaming()
   {
       QDataStream stream( &buffer, QIODevice::WriteOnly );
       stream << origURL
+             << accentuated
              << empty
              << invalid
              << waba1; // the IPv6 one
@@ -1194,6 +1329,8 @@ void KUrlTest::testStreaming()
       KUrl restoredURL;
       stream >> restoredURL; // streaming valid url
       QCOMPARE( restoredURL.url(), origURL.url() );
+      stream >> restoredURL; // streaming valid url with accents
+      QCOMPARE( restoredURL.url(), accentuated.url() );
       stream >> restoredURL; // streaming empty url
       QVERIFY( !restoredURL.isValid() );
       QVERIFY( restoredURL.isEmpty() );
@@ -1315,26 +1452,32 @@ void KUrlTest::testMoreBrokenStuff()
   weird = "http://strange<username>@strange<hostname>/";
   QVERIFY( !weird.isValid() );
 
-  weird = "http://strange<username>@ok_hostname/";
+  weird = "http://strange<username>@hostname/";
   QVERIFY( weird.isValid() ); // KDE3: was valid. Fixed by _setEncodedUrl.
-  QCOMPARE( weird.host(), QString("ok_hostname") );
+  QCOMPARE( weird.host(), QString("hostname") );
 
   weird = "http://strange;hostname/";
+#if QT_VERSION < 0x040600
   QVERIFY( weird.isValid() ); // KDE3: was invalid. bah*2.
+#endif
 
   weird = "http://strange;username@strange;hostname/";
+#if QT_VERSION < 0x040600
   QVERIFY( weird.isValid() ); // KDE3: was invalid. bah*3.
+#endif
 
-  weird = "http://strange;username@ok_hostname/";
+  weird = "http://strange;username@hostname/";
   QVERIFY( weird.isValid() );
-  QCOMPARE( weird.host(), QString("ok_hostname") );
+  QCOMPARE( weird.host(), QString("hostname") );
 
   weird = "http://strange;username:password@strange;hostname/";
+#if QT_VERSION < 0x040600
   QVERIFY( weird.isValid() ); // KDE3: was invalid
+#endif
 
-  weird = "http://strange;username:password@ok_hostname/";
+  weird = "http://strange;username:password@hostname/";
   QVERIFY( weird.isValid() );
-  QCOMPARE( weird.host(), QString("ok_hostname") );
+  QCOMPARE( weird.host(), QString("hostname") );
 
   weird = "http://[strange;hostname]/";
   QVERIFY( !weird.isValid() );
@@ -1382,16 +1525,21 @@ void KUrlTest::testMoreBrokenStuff()
 
   QUrl dxOffEagle3;
   dxOffEagle3.setEncodedUrl( "http://something/newpage.html?[{\"foo: bar\"}]", QUrl::TolerantMode);
+#if QT_VERSION < 0x040500
   QEXPECT_FAIL("","Issue N183630, task ID 183874; works with setUrl so we do that in _setEncodedUrl now", Continue);
+#endif
   QVERIFY(dxOffEagle3.isValid());
-  //QEXPECT_FAIL("","Issue N183630, task ID 183874", Continue);
-  //QCOMPARE(dxOffEagle.toEncoded(), dxOffEagle3.toEncoded());
+#if QT_VERSION >= 0x040500
+  QCOMPARE(dxOffEagle.toEncoded(), dxOffEagle3.toEncoded());
+#endif
 
   QUrl javascript;
   javascript.setUrl("javascript:window.location+\"__flashplugin_unique__\"", QUrl::TolerantMode);
   QVERIFY(javascript.isValid());
   javascript.setEncodedUrl("javascript:window.location+\"__flashplugin_unique__\"", QUrl::TolerantMode);
+#if QT_VERSION < 0x040500
   QEXPECT_FAIL("","Issue N183630, task ID 183874", Continue);
+#endif
   QVERIFY(javascript.isValid());
 }
 
@@ -1423,6 +1571,12 @@ void KUrlTest::testMailto()
   QCOMPARE( url1.url(), QString("mailto:user@host.com") );
   QCOMPARE( url1.url(KUrl::LeaveTrailingSlash), QString("mailto:user@host.com") );
 
+  KUrl mailtoUrl("mailto:null@kde.org?subject=hello");
+  QCOMPARE( mailtoUrl.url(), QString("mailto:null@kde.org?subject=hello" ));
+
+  QUrl qurl("mailto:null@kde.org?subject=hello#world"); // #80165: is #world part of fragment or query? RFC-3986 says: fragment.
+  QCOMPARE(QString::fromLatin1(qurl.encodedQuery()), QString("subject=hello"));
+
 #if 0
   // I wrote this test in the very first kurltest, but there's no proof that it's actually valid.
   // Andreas says this is broken, i.e. against rfc2368.
@@ -1432,6 +1586,10 @@ void KUrlTest::testMailto()
   QCOMPARE( umail2.path(), QString("Faure David <faure@kde.org>") );
   QVERIFY( !KUrl::isRelativeUrl("mailto:faure@kde.org") );
 #endif
+
+  KUrl url183433("mailto:test[at]gmail[dot]com");
+  QCOMPARE(url183433.prettyUrl(), QString("mailto:test[at]gmail[dot]com"));
+  QCOMPARE(url183433.url(), QString("mailto:test[at]gmail[dot]com"));
 }
 
 void KUrlTest::testSmb()
@@ -1448,6 +1606,13 @@ void KUrlTest::testSmb()
   smb = "smb:///";
   QVERIFY( smb.isValid() );
 
+  KUrl implicitSmb("file://host/path");
+  QVERIFY(!implicitSmb.isLocalFile()); // -> kio_file will redirect to smb (by default)
+  QCOMPARE(implicitSmb.host(), QString("host"));
+
+  KUrl noImplicitSmb("//path1/path2");
+  QVERIFY(noImplicitSmb.isLocalFile());
+  QCOMPARE(noImplicitSmb.path(), QString("//path1/path2"));
 }
 
 void KUrlTest::testOtherProtocols()
@@ -1467,10 +1632,16 @@ void KUrlTest::testOtherProtocols()
   QCOMPARE( leo.path(), QString("text/html,http://www.invalid/" ) );
 
   KUrl ptal( "ptal://mlc:usb@PC_970" ); // User=mlc, password=usb, host=PC_970
+#if QT_VERSION >= 0x040600
+  QCOMPARE(ptal.url(), QString("ptal://mlc:usb@")); // The host "PC_970" is invalid according to STD3 validation
+  KUrl ptalSimpler("ptal://mlc:usb@pc123");
+  QCOMPARE(ptalSimpler.url(), QString("ptal://mlc:usb@pc123"));
+#else
   QVERIFY( ptal.isValid() );
   QCOMPARE( ptal.host(), QString("pc_970") );
   QCOMPARE( ptal.user(), QString("mlc") );
   QCOMPARE( ptal.pass(), QString("usb") );
+#endif
 }
 
 void KUrlTest::testUtf8()
@@ -1508,9 +1679,18 @@ void KUrlTest::testUtf8()
   KUrl umlaut2("http://www.clever-tanken.de/liste.asp?ort=N%FCrnberg&typ=Diesel"); // was ,106
   QCOMPARE(umlaut2.url(), QString("http://www.clever-tanken.de/liste.asp?ort=N%FCrnberg&typ=Diesel"));
 
+  KUrl urlWithUnicodeChar( QString::fromUtf8("file:///home/dfaure/Matériel") );
+  QCOMPARE( uloc.url(), QString("file:///home/dfaure/Mat%C3%A9riel") );
+
   KUrl wkai(QString::fromUtf8("/tmp/魔"));
   QCOMPARE(wkai.url(), QString("file:///tmp/%E9%AD%94"));
   QCOMPARE(wkai.prettyUrl(), QString::fromUtf8("file:///tmp/魔"));
+
+  // Show that the character "fraction slash" (U+2044) cannot appear in url(),
+  // so it's ok to use that to encode urls as filenames (e.g. in kio_http cache)
+  KUrl fractionSlash(QString("http://kde.org/a")+QChar(0x2044)+"b");
+  QCOMPARE(fractionSlash.url(), QString("http://kde.org/a%E2%81%84b"));
+  QCOMPARE(fractionSlash.prettyUrl(), QString("http://kde.org/a")+QChar(0x2044)+"b");
 }
 
 void KUrlTest::testOtherEncodings()
@@ -1535,10 +1715,11 @@ void KUrlTest::testOtherEncodings()
 void KUrlTest::testPathOrURL()
 {
   // passing path or url to the constructor: both work
-  KUrl uloc( LOCAL_FILE_PREFIX"/home/dfaure/konqtests/Mat%C3%A9riel" );
-  QCOMPARE( uloc.path(), QString(LOCAL_FILE_PREFIX"/home/dfaure/konqtests/Mat%C3%A9riel") );
+  KUrl uloc( "/home/dfaure/konqtests/Mat%C3%A9riel" );
+  QCOMPARE( uloc.path(), QString("/home/dfaure/konqtests/Mat%C3%A9riel") );
   uloc = KUrl( "http://www.kde.org" );
   QCOMPARE( uloc.pathOrUrl(), uloc.url() );
+  QCOMPARE( uloc.pathOrUrl(KUrl::AddTrailingSlash), QString("http://www.kde.org/") );
   uloc = KUrl( QString("www.kde.org" ) );
   QVERIFY( uloc.isValid() ); // KDE3: was invalid. But it's now a url with path="www.kde.org", ok.
   uloc = KUrl( "index.html" );
@@ -1560,7 +1741,7 @@ void KUrlTest::testPathOrURL()
 #endif
 
   // pathOrUrl tests
-  uloc = KUrl( LOCAL_FILE_PREFIX"/home/dfaure/konqtests/Mat%C3%A9riel" );
+  uloc = KUrl( "/home/dfaure/konqtests/Mat%C3%A9riel" );
   QCOMPARE( uloc.pathOrUrl(), uloc.path() );
   uloc = "http://www.kde.org";
   QCOMPARE( uloc.url(), QString("http://www.kde.org") );
@@ -1568,19 +1749,33 @@ void KUrlTest::testPathOrURL()
   QCOMPARE( uloc.pathOrUrl(), QString::fromUtf8("file:///home/dfaure/konq tests/Matériel#ref" ) );
   uloc = "file:///home/dfaure/konq%20tests/Mat%C3%A9riel?query";
   QCOMPARE( uloc.pathOrUrl(), QString::fromUtf8("file:///home/dfaure/konq tests/Matériel?query" ) );
-  uloc = KUrl( LOCAL_FILE_PREFIX"/home/dfaure/file#with#hash" );
-  QCOMPARE( uloc.pathOrUrl(), QString(LOCAL_FILE_PREFIX"/home/dfaure/file#with#hash" ) );
+  uloc = KUrl( "/home/dfaure/file#with#hash" );
+  QCOMPARE( uloc.pathOrUrl(), QString("/home/dfaure/file#with#hash" ) );
+
+  // test creation of url from pathOrUrl
+  uloc = KUrl( QString::fromUtf8("http://www.kde.org/home/andreas/täst") );
+  QCOMPARE( KUrl( uloc.pathOrUrl() ), uloc );
+  uloc = KUrl( "http://www.kde.org/home/andreas/t%C3%A4st");
+  QCOMPARE( KUrl( uloc.pathOrUrl() ), uloc );
+  uloc = KUrl( QString::fromUtf8("file:///home/andreas/täst") );
+  QCOMPARE( KUrl( uloc.pathOrUrl() ), uloc );
+  uloc = KUrl( "file:///home/andreas/t%C3%A4st");
+  QCOMPARE( KUrl( uloc.pathOrUrl() ), uloc );
+  uloc = KUrl( "http://www.kde.org/home/kde?foobar#test" );
+  QCOMPARE( KUrl( uloc.pathOrUrl() ), uloc );
+  uloc = KUrl( "http://www.kde.org/home/%andreas");
+  QCOMPARE( KUrl( uloc.pathOrUrl() ), uloc );
 }
 
 void KUrlTest::testAssignment()
 {
   // passing path or url to the constructor: both work
   KUrl uloc;
-  uloc = LOCAL_FILE_PREFIX"/home/dfaure/konqtests/Mat%C3%A9riel";
-  QCOMPARE( uloc.path(), QString(LOCAL_FILE_PREFIX"/home/dfaure/konqtests/Mat%C3%A9riel") );
+  uloc = "/home/dfaure/konqtests/Mat%C3%A9riel";
+  QCOMPARE( uloc.path(), QString("/home/dfaure/konqtests/Mat%C3%A9riel") );
   KUrl u2;
   u2 = uloc;
-  QCOMPARE( u2.path(), QString(LOCAL_FILE_PREFIX"/home/dfaure/konqtests/Mat%C3%A9riel") );
+  QCOMPARE( u2.path(), QString("/home/dfaure/konqtests/Mat%C3%A9riel") );
   uloc = "http://www.kde.org";
   QCOMPARE( uloc.pathOrUrl(), uloc.url() );
   uloc = QString("www.kde.org" );
@@ -1648,14 +1843,10 @@ void KUrlTest::testEncodeString()
   // Needed for #49616
   QCOMPARE( QUrl::toPercentEncoding( "C++" ), QByteArray("C%2B%2B") );
   QCOMPARE( QUrl::fromPercentEncoding( "C%2B%2B" ), QString("C++") );
-#if QT_VERSION < 0x040400
-  QCOMPARE( QUrl::fromPercentEncoding( "C%00%A" ), QString("C") ); // stop at %00, like in kde3
-#else
   QString output = QUrl::fromPercentEncoding( "C%00%0A" );
   QString expected = QString::fromLatin1("C\0\n", 3); // no reason to stop at %00, in fact
   QCOMPARE( output.size(), expected.size() );
   QCOMPARE( output, expected );
-#endif
   QCOMPARE( QUrl::fromPercentEncoding( "C%A" ), QString("C%A") ); // % A is not percent-encoding  (pct-encoded = "%" HEXDIG HEXDIG)
 
   QCOMPARE( QUrl::toPercentEncoding( "%" ), QByteArray("%25") );
