@@ -1,3 +1,4 @@
+// vim: expandtab sw=4 ts=4
 /*  This file is part of the KDE libraries
  *  Copyright (C) 1999 David Faure <faure@kde.org>
  *  Copyright (C) 2000 Waldo Bastian <bastian@kde.org>
@@ -20,6 +21,7 @@
 #include "kded.h"
 #include "kdedadaptor.h"
 #include "kdedmodule.h"
+#include "kbuildsycoca.h"
 
 #include "kresourcelist.h"
 #include <kcrash.h>
@@ -48,6 +50,7 @@
 #include <kstandarddirs.h>
 #include <kservicetypetrader.h>
 #include <ktoolinvocation.h>
+#include <kde_file.h>
 #include "klauncher_iface.h"
 
 #ifdef Q_WS_X11
@@ -64,6 +67,10 @@ Kded *Kded::_self = 0;
 
 static bool checkStamps = true;
 static bool delayedCheck = false;
+static int HostnamePollInterval;
+static bool bCheckSycoca;
+static bool bCheckUpdates;
+static bool bCheckHostname;
 
 extern QDBUS_EXPORT void qDBusAddSpyHook(void (*)(const QDBusMessage&));
 
@@ -89,6 +96,10 @@ static void runBuildSycoca(QObject *callBackObj=0, const char *callBackSlot=0)
    else
    {
       KToolInvocation::kdeinitExecWait( exe, args );
+
+      if (callBackObj && callBackSlot) {
+          QTimer::singleShot(0, callBackObj, callBackSlot);
+      }
    }
 }
 
@@ -105,9 +116,8 @@ static void runDontChangeHostname(const QByteArray &oldName, const QByteArray &n
    KToolInvocation::kdeinitExecWait( "kdontchangethehostname", args );
 }
 
-Kded::Kded(bool checkUpdates)
-  : b_checkUpdates(checkUpdates),
-    m_needDelayedCheck(false)
+Kded::Kded()
+  : m_needDelayedCheck(false)
 {
   _self = this;
 
@@ -157,6 +167,10 @@ Kded::~Kded()
 // calls are delivered to objects
 void Kded::messageFilter(const QDBusMessage &message)
 {
+  // This happens when kded goes down and some modules try to clean up.
+  if (!self())
+     return;
+
   if (message.type() != QDBusMessage::MethodCallMessage)
      return;
 
@@ -164,33 +178,43 @@ void Kded::messageFilter(const QDBusMessage &message)
   if (!obj.startsWith(MODULES_PATH))
      return;
 
+  // Remove the <MODULES_PATH> part
   obj = obj.mid(strlen(MODULES_PATH));
   if (obj == "ksycoca")
      return; // Ignore this one.
 
+  // Remove the part after the modules name
+  int index = obj.indexOf('/');
+  if (index!=-1) {
+      obj = obj.left(index);
+  }
+
   if (self()->m_dontLoad.value(obj, 0))
      return;
 
   KDEDModule *module = self()->loadModule(obj, true);
+  if (!module) {
+      kDebug(7020) << "Failed to load module for " << obj;
+  }
   Q_UNUSED(module);
 }
 
 void Kded::initModules()
 {
      m_dontLoad.clear();
-     KSharedConfig::Ptr config = KGlobal::config();
-     bool kde_running = !( getenv( "KDE_FULL_SESSION" ) == NULL || getenv( "KDE_FULL_SESSION" )[ 0 ] == '\0' );
+     bool kde_running = !qgetenv( "KDE_FULL_SESSION" ).isEmpty();
     // not the same user like the one running the session (most likely we're run via sudo or something)
-    if( getenv( "KDE_SESSION_UID" ) != NULL && uid_t( atoi( getenv( "KDE_SESSION_UID" ))) != getuid())
+    const QByteArray sessionUID = qgetenv( "KDE_SESSION_UID" );
+    if( !sessionUID.isEmpty() && uid_t( sessionUID.toInt() ) != getuid())
         kde_running = false;
      // Preload kded modules.
-     KService::List kdedModules = KServiceTypeTrader::self()->query("KDEDModule");
+     const KService::List kdedModules = KServiceTypeTrader::self()->query("KDEDModule");
      for(KService::List::ConstIterator it = kdedModules.begin(); it != kdedModules.end(); ++it)
      {
          KService::Ptr service = *it;
-         bool autoload = service->property("X-KDE-Kded-autoload", QVariant::Bool).toBool();
-         KConfigGroup cg(config, QString("Module-%1").arg(service->desktopEntryName()));
-         autoload = cg.readEntry("autoload", autoload);
+         // Should the service load on startup?
+         bool autoload = isModuleAutoloaded(service);
+
          // see ksmserver's README for description of the phases
          QVariant phasev = service->property("X-KDE-Kded-phase", QVariant::Int );
          int phase = phasev.isValid() ? phasev.toInt() : 2;
@@ -208,17 +232,22 @@ void Kded::initModules()
                  prevent_autoload = true;
                  break;
          }
-         if (autoload && !prevent_autoload)
-            loadModule(service, false);
 
-         bool dontLoad = false;
-         QVariant p = service->property("X-KDE-Kded-load-on-demand", QVariant::Bool);
-         if (p.isValid() && (p.toBool() == false))
-            dontLoad = true;
-         if (dontLoad)
+        // Load the module if necessary and allowed
+         if (autoload && !prevent_autoload) {
+            if (!loadModule(service, false)) {
+                continue;
+            }
+         }
+
+         // Remember if the module is allowed to load on demand
+         bool loadOnDemand = isModuleLoadedOnDemand(service);
+         if (!loadOnDemand)
             noDemandLoad(service->desktopEntryName());
 
-         if (dontLoad && !autoload)
+         // In case of reloading the configuration it is possible for a module
+         // to run even if it is now allowed to. Stop it then.
+         if (!loadOnDemand && !autoload)
             unloadModule(service->desktopEntryName().toLatin1());
      }
 }
@@ -228,7 +257,7 @@ void Kded::loadSecondPhase()
      kDebug(7020) << "Loading second phase autoload";
      KSharedConfig::Ptr config = KGlobal::config();
      KService::List kdedModules = KServiceTypeTrader::self()->query("KDEDModule");
-     for(KService::List::ConstIterator it = kdedModules.begin(); it != kdedModules.end(); ++it)
+     for(KService::List::ConstIterator it = kdedModules.constBegin(); it != kdedModules.constEnd(); ++it)
      {
          KService::Ptr service = *it;
          bool autoload = service->property("X-KDE-Kded-autoload", QVariant::Bool).toBool();
@@ -246,8 +275,58 @@ void Kded::noDemandLoad(const QString &obj)
   m_dontLoad.insert(obj.toLatin1(), this);
 }
 
+void Kded::setModuleAutoloading(const QString &obj, bool autoload)
+{
+    KSharedConfig::Ptr config = KGlobal::config();
+    // Ensure the service exists.
+    KService::Ptr service = KService::serviceByDesktopPath("kded/"+obj+".desktop");
+    if (!service)
+        return;
+    KConfigGroup cg(config, QString("Module-%1").arg(service->desktopEntryName()));
+    cg.writeEntry("autoload", autoload);
+    cg.sync();
+}
+
+bool Kded::isModuleAutoloaded(const QString &obj) const
+{
+    KService::Ptr s = KService::serviceByDesktopPath("kded/"+obj+".desktop");
+    if (!s)
+        return false;
+    return isModuleAutoloaded(s);
+}
+
+bool Kded::isModuleAutoloaded(const KService::Ptr &module) const
+{
+    KSharedConfig::Ptr config = KGlobal::config();
+    bool autoload = module->property("X-KDE-Kded-autoload", QVariant::Bool).toBool();
+    KConfigGroup cg(config, QString("Module-%1").arg(module->desktopEntryName()));
+    autoload = cg.readEntry("autoload", autoload);
+    return autoload;
+}
+
+bool Kded::isModuleLoadedOnDemand(const QString &obj) const
+{
+    KService::Ptr s = KService::serviceByDesktopPath("kded/"+obj+".desktop");
+    if (!s)
+        return false;
+    return isModuleLoadedOnDemand(s);
+}
+
+bool Kded::isModuleLoadedOnDemand(const KService::Ptr &module) const
+{
+    KSharedConfig::Ptr config = KGlobal::config();
+    bool loadOnDemand = true;
+    QVariant p = module->property("X-KDE-Kded-load-on-demand", QVariant::Bool);
+    if (p.isValid() && (p.toBool() == false))
+        loadOnDemand = false;
+    return loadOnDemand;
+}
+
 KDEDModule *Kded::loadModule(const QString &obj, bool onDemand)
 {
+  // Make sure this method is only called with valid module names.
+  Q_ASSERT(obj.indexOf('/')==-1);
+
   KDEDModule *module = m_modules.value(obj, 0);
   if (module)
      return module;
@@ -340,7 +419,7 @@ void Kded::slotKDEDModuleRemoved(KDEDModule *module)
 }
 
 void Kded::slotApplicationRemoved(const QString &name, const QString &oldOwner,
-                                  const QString &)
+                                  const QString &newOwner)
 {
 #if 0 // see kdedmodule.cpp (KDED_OBJECTS)
   foreach( KDEDModule* module, m_modules )
@@ -348,7 +427,7 @@ void Kded::slotApplicationRemoved(const QString &name, const QString &oldOwner,
      module->removeAll(appId);
   }
 #endif
-  if (oldOwner.isEmpty())
+  if (oldOwner.isEmpty() || !newOwner.isEmpty())
      return;
 
   const QList<qlonglong> windowIds = m_windowIdList.value(name);
@@ -367,7 +446,7 @@ void Kded::slotApplicationRemoved(const QString &name, const QString &oldOwner,
 
 void Kded::updateDirWatch()
 {
-  if (!b_checkUpdates) return;
+  if (!bCheckUpdates) return;
 
   delete m_pDirWatch;
   m_pDirWatch = new KDirWatch;
@@ -380,8 +459,8 @@ void Kded::updateDirWatch()
            this, SLOT(dirDeleted(const QString&)));
 
   // For each resource
-  for( QStringList::ConstIterator it = m_allResourceDirs.begin();
-       it != m_allResourceDirs.end();
+  for( QStringList::ConstIterator it = m_allResourceDirs.constBegin();
+       it != m_allResourceDirs.constEnd();
        ++it )
   {
      readDirectory( *it );
@@ -390,9 +469,9 @@ void Kded::updateDirWatch()
 
 void Kded::updateResourceList()
 {
-  delete KSycoca::self();
+  KBuildSycoca::clearCaches();
 
-  if (!b_checkUpdates) return;
+  if (!bCheckUpdates) return;
 
   if (delayedCheck) return;
 
@@ -410,17 +489,6 @@ void Kded::updateResourceList()
   }
 }
 
-void Kded::crashHandler(int)
-{
-   if (_self) // Don't restart if we were closing down
-      system(KDED_EXENAME);
-}
-
-void Kded::installCrashHandler()
-{
-   KCrash::setEmergencySaveFunction(crashHandler);
-}
-
 void Kded::recreate()
 {
    recreate(false);
@@ -448,7 +516,8 @@ void Kded::recreate(bool initial)
    {
       if(!delayedCheck)
          updateDirWatch(); // this would search all the directories
-      runBuildSycoca();
+      if (bCheckSycoca)
+         runBuildSycoca();
       recreateDone();
       if(delayedCheck)
       {
@@ -459,7 +528,6 @@ void Kded::recreate(bool initial)
       }
       else
          m_needDelayedCheck = false;
-      QTimer::singleShot(100, this, SLOT(installCrashHandler()));
    }
 }
 
@@ -576,6 +644,7 @@ void Kded::registerWindowId(qlonglong windowId, const QString &sender)
 
   foreach( KDEDModule* module, m_modules )
   {
+     //kDebug() << module->moduleName();
      emit module->windowRegistered(windowId);
   }
 }
@@ -595,6 +664,7 @@ void Kded::unregisterWindowId(qlonglong windowId, const QString &sender)
 
   foreach( KDEDModule* module, m_modules )
   {
+    //kDebug() << module->moduleName();
     emit module->windowUnregistered(windowId);
   }
 }
@@ -602,7 +672,7 @@ void Kded::unregisterWindowId(qlonglong windowId, const QString &sender)
 
 static void sighandler(int /*sig*/)
 {
-    if (kapp)
+    if (qApp)
        qApp->quit();
 }
 
@@ -615,7 +685,7 @@ KUpdateD::KUpdateD()
     QObject::connect( m_pDirWatch, SIGNAL(dirty(const QString&)),
            this, SLOT(slotNewUpdateFile()));
 
-    QStringList dirs = KGlobal::dirs()->findDirs("data", "kconf_update");
+    const QStringList dirs = KGlobal::dirs()->findDirs("data", "kconf_update");
     for( QStringList::ConstIterator it = dirs.begin();
          it != dirs.end();
          ++it )
@@ -693,7 +763,7 @@ public:
   virtual bool process(const DCOPCString &fun, const QByteArray &data,
                        DCOPCString& replyType, QByteArray &replyData)
     {
-      if ( kapp && (fun == "quit()") )
+      if (qApp && (fun == "quit()"))
       {
         qApp->quit();
         replyType = "void";
@@ -733,7 +803,61 @@ public:
     {
        if (startup) {
           startup = false;
-          Kded::self()->initModules();
+
+          // This long initialization has to be here, not in kdemain.
+          // If it was in main, it would cause a dbus timeout when
+          // our parent from KUniqueApplication tries to call our
+          // newInstance method.
+
+          Kded *kded = Kded::self();
+
+          kded->recreate(true); // initial
+
+          if (bCheckUpdates)
+            (void) new KUpdateD; // Watch for updates
+
+#ifdef Q_WS_X11
+          XEvent e;
+          e.xclient.type = ClientMessage;
+          e.xclient.message_type = XInternAtom( QX11Info::display(), "_KDE_SPLASH_PROGRESS", False );
+          e.xclient.display = QX11Info::display();
+          e.xclient.window = QX11Info::appRootWindow();
+          e.xclient.format = 8;
+          strcpy( e.xclient.data.b, "kded" );
+          XSendEvent( QX11Info::display(), QX11Info::appRootWindow(), False, SubstructureNotifyMask, &e );
+#endif
+
+          runKonfUpdate(); // Run it once.
+
+#ifdef Q_WS_X11
+          e.xclient.type = ClientMessage;
+          e.xclient.message_type = XInternAtom( QX11Info::display(), "_KDE_SPLASH_PROGRESS", False );
+          e.xclient.display = QX11Info::display();
+          e.xclient.window = QX11Info::appRootWindow();
+          e.xclient.format = 8;
+          strcpy( e.xclient.data.b, "confupdate" );
+          XSendEvent( QX11Info::display(), QX11Info::appRootWindow(), False, SubstructureNotifyMask, &e );
+#endif
+
+          if (bCheckHostname)
+            (void) new KHostnameD(HostnamePollInterval); // Watch for hostname changes
+
+          QObject::connect(QDBusConnection::sessionBus().interface(),
+                          SIGNAL(serviceOwnerChanged(QString,QString,QString)),
+                          kded, SLOT(slotApplicationRemoved(QString,QString,QString)));
+
+          // During startup kdesktop waits for KDED to finish.
+          // Send a notifyDatabaseChanged signal even if the database hasn't
+          // changed.
+          // If the database changed, kbuildsycoca's signal didn't go anywhere
+          // anyway, because it was too early, so let's send this signal
+          // unconditionnally (David)
+
+          QDBusMessage msg = QDBusMessage::createSignal("/kbuildsycoca", "org.kde.KSycoca", "notifyDatabaseChanged" );
+          msg << QStringList();
+          QDBusConnection::sessionBus().send(msg);
+
+          kded->initModules();
        } else
           runBuildSycoca();
 
@@ -747,7 +871,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char *argv[])
 {
      KAboutData aboutData( "kded" /*don't change this one to kded4! dbus registration should be org.kde.kded etc.*/,
         "kdelibs4", ki18n("KDE Daemon"),
-        "$Id: kded.cpp 770025 2008-02-02 16:20:39Z dfaure $",
+        "$Id: kded.cpp 1031638 2009-10-05 16:59:11Z lunakl $",
         ki18n("KDE Daemon - triggers Sycoca database updates when needed"));
 
      KCmdLineOptions options;
@@ -788,71 +912,27 @@ extern "C" KDE_EXPORT int kdemain(int argc, char *argv[])
      // Thiago: reenable if such a thing exists in QtDBus in the future
      //KUniqueApplication::dcopClient()->setQtBridgeEnabled(false);
 
-     int HostnamePollInterval = cg.readEntry("HostnamePollInterval", 5000);
-     bool bCheckSycoca = cg.readEntry("CheckSycoca", true);
-     bool bCheckUpdates = cg.readEntry("CheckUpdates", true);
-     bool bCheckHostname = cg.readEntry("CheckHostname", true);
+     HostnamePollInterval = cg.readEntry("HostnamePollInterval", 5000);
+     bCheckSycoca = cg.readEntry("CheckSycoca", true);
+     bCheckUpdates = cg.readEntry("CheckUpdates", true);
+     bCheckHostname = cg.readEntry("CheckHostname", true);
      checkStamps = cg.readEntry("CheckFileStamps", true);
      delayedCheck = cg.readEntry("DelayedCheck", false);
 
-     Kded *kded = new Kded(bCheckSycoca); // Build data base
+     Kded *kded = new Kded(); // Build data base
 
-     signal(SIGTERM, sighandler);
-     signal(SIGHUP, sighandler);
+     KDE_signal(SIGTERM, sighandler);
+     KDE_signal(SIGHUP, sighandler);
      KDEDApplication k;
      k.setQuitOnLastWindowClosed(false);
 
+     KCrash::setFlags(KCrash::AutoRestart);
+
      // Not sure why kded is created before KDEDApplication
-     // but if it has to be, then it needsto be moved to the main thread
+     // but if it has to be, then it needs to be moved to the main thread
      // before it can use timers (DF)
      kded->moveToThread( k.thread() );
 
-     kded->recreate(true); // initial
-
-     if (bCheckUpdates)
-        (void) new KUpdateD; // Watch for updates
-
-#ifdef Q_WS_X11
-     XEvent e;
-     e.xclient.type = ClientMessage;
-     e.xclient.message_type = XInternAtom( QX11Info::display(), "_KDE_SPLASH_PROGRESS", False );
-     e.xclient.display = QX11Info::display();
-     e.xclient.window = QX11Info::appRootWindow();
-     e.xclient.format = 8;
-     strcpy( e.xclient.data.b, "kded" );
-     XSendEvent( QX11Info::display(), QX11Info::appRootWindow(), False, SubstructureNotifyMask, &e );
-#endif
-
-     runKonfUpdate(); // Run it once.
-
-#ifdef Q_WS_X11
-     e.xclient.type = ClientMessage;
-     e.xclient.message_type = XInternAtom( QX11Info::display(), "_KDE_SPLASH_PROGRESS", False );
-     e.xclient.display = QX11Info::display();
-     e.xclient.window = QX11Info::appRootWindow();
-     e.xclient.format = 8;
-     strcpy( e.xclient.data.b, "confupdate" );
-     XSendEvent( QX11Info::display(), QX11Info::appRootWindow(), False, SubstructureNotifyMask, &e );
-#endif
-
-     if (bCheckHostname)
-        (void) new KHostnameD(HostnamePollInterval); // Watch for hostname changes
-
-     QObject::connect(QDBusConnection::sessionBus().interface(),
-                      SIGNAL(serviceOwnerChanged(QString,QString,QString)),
-                      kded, SLOT(slotApplicationRemoved(QString,QString,QString)));
-
-     // During startup kdesktop waits for KDED to finish.
-     // Send a notifyDatabaseChanged signal even if the database hasn't
-     // changed.
-     // If the database changed, kbuildsycoca's signal didn't go anywhere
-     // anyway, because it was too early, so let's send this signal
-     // unconditionnally (David)
-
-     // FIXME: rename the signal
-     QDBusMessage msg = QDBusMessage::createSignal("/kbuildsycoca", "org.kde.KSycoca", "notifyDatabaseChanged" );
-     msg << QStringList();
-     QDBusConnection::sessionBus().send(msg);
      int result = k.exec(); // keep running
 
      delete kded;
