@@ -28,6 +28,7 @@
 #include "kurltogglebutton_p.h"
 
 #include <kfileitem.h>
+#include <kfileplacesmodel.h>
 #include <kglobalsettings.h>
 #include <kicon.h>
 #include <klineedit.h>
@@ -41,10 +42,15 @@
 #include <QtCore/QLinkedList>
 #include <QtCore/QTimer>
 #include <QtGui/QApplication>
-#include <QtGui/QClipboard>
-#include <QtGui/QKeyEvent>
 #include <QtGui/QBoxLayout>
+#include <QtGui/QClipboard>
+#include <QtGui/QDropEvent>
+#include <QtGui/QKeyEvent>
 #include <QtGui/QLabel>
+#include <QtGui/QPainter>
+#include <QtGui/QStyleOption>
+
+#include <fixx11h.h>
 
 /**
  * @brief Represents the history element of an URL.
@@ -132,75 +138,6 @@ inline int HistoryElem::contentsY() const
     return m_contentsY;
 }
 
-/**
- * @brief Line editor for the host part of the URL.
- *
- * Extends the class KLineEdit to return a preferred
- * width in HostLineEdit::sizeHint() which assures that
- * the host part is always fully visible.
- */
-class HostLineEdit : public KLineEdit
-{
-public:
-    explicit HostLineEdit(const QString& text, KUrlNavigator* parent = 0);
-    virtual ~HostLineEdit();
-    virtual QSize sizeHint() const;
-    inline void setOptimizeWidth(bool optimize);
-
-protected:
-    virtual void mousePressEvent(QMouseEvent* event);
-
-private:
-    bool m_optimizeWidth;
-    KUrlNavigator* m_urlNavigator;
-};
-
-HostLineEdit::HostLineEdit(const QString& text, KUrlNavigator* parent) :
-    KLineEdit(text, parent),
-    m_optimizeWidth(true),
-    m_urlNavigator(parent)
-{
-}
-
-HostLineEdit::~HostLineEdit()
-{
-}
-
-void HostLineEdit::setOptimizeWidth(bool optimize)
-{
-    m_optimizeWidth = optimize;
-}
-
-QSize HostLineEdit::sizeHint() const
-{
-    QSize size = KLineEdit::sizeHint();
-
-    if (m_optimizeWidth) {
-        const QFontMetrics fm(font());
-        const int width = fm.width(text()) + 32;
-        if (width > size.width()) {
-            size.setWidth(width);
-        }
-    }
-
-    return size;
-}
-
-void HostLineEdit::mousePressEvent(QMouseEvent* event)
-{
-    KLineEdit::mousePressEvent(event);
-
-    if (event->button() == Qt::LeftButton) {
-        // behave like a button when a mouse click has been done
-        // inside the host editor and go to the host URL
-        const KUrl currentUrl = m_urlNavigator->url();
-        const KUrl newUrl(currentUrl.protocol() + "://" + text());
-        if (currentUrl != newUrl) {
-            m_urlNavigator->setUrl(newUrl);
-        }
-    }
-}
-
 ////
 
 class KUrlNavigator::Private
@@ -208,7 +145,6 @@ class KUrlNavigator::Private
 public:
     Private(KUrlNavigator* q, KFilePlacesModel* placesModel);
 
-    void slotReturnPressed(const QString&);
     void slotReturnPressed();
     void slotRemoteHostActivated();
     void slotProtocolChanged(const QString&);
@@ -229,7 +165,7 @@ public:
     void switchView();
 
     /** Emits the signal urlsDropped(). */
-    void dropUrls(const KUrl::List& urls, const KUrl& destination);
+    void dropUrls(const KUrl& destination, QDropEvent* event);
 
     void updateContent();
 
@@ -250,6 +186,8 @@ public:
      */
     void updateButtonVisibility();
 
+    void switchToBreadcrumbMode();
+
     /**
      * Deletes all URL navigator buttons. m_navButtons is
      * empty after this operation.
@@ -257,13 +195,13 @@ public:
     void deleteButtons();
 
     /**
-     * Retrieves the place path for the path \a path.
+     * Retrieves the place path for the current path.
      * E. g. for the path "fish://root@192.168.0.2/var/lib" the string
-     * "fish://root@192.168.0.2/" will be returned, which leads to the
+     * "fish://root@192.168.0.2" will be returned, which leads to the
      * navigation indication 'Custom Path > var > lib". For e. g.
-     * "settings:///System/" the path "settings:///" will be returned.
+     * "settings:///System/" the path "settings://" will be returned.
      */
-    QString retrievePlacePath(const QString& path) const;
+    QString retrievePlacePath() const;
 
     /**
      * Returns true, if the MIME type of the path represents a
@@ -271,9 +209,12 @@ public:
      */
     bool isCompressedPath(const KUrl& path) const;
 
-    bool m_editable;
-    bool m_active;
-    bool m_showPlacesSelector;
+    void removeTrailingSlash(QString& url) const;
+
+    bool m_editable : 1;
+    bool m_active : 1;
+    bool m_showPlacesSelector : 1;
+    bool m_showFullPath : 1;
     int m_historyIndex;
 
     QHBoxLayout* m_layout;
@@ -282,20 +223,13 @@ public:
     KFilePlacesSelector* m_placesSelector;
     KUrlComboBox* m_pathBox;
     KProtocolCombo* m_protocols;
-    HostLineEdit* m_host;
+    KLineEdit* m_host;
     KUrlDropDownButton* m_dropDownButton;
     QLinkedList<KUrlNavigatorButton*> m_navButtons;
     KUrlButton* m_toggleEditableMode;
     QString m_homeUrl;
     QStringList m_customProtocols;
     KUrlNavigator* q;
-
-private:
-    /**
-     * Creates an instance of the class HostLineEdit and
-     * assigns it to m_host.
-     */
-    void createHostLineEdit(const QString& text);
 };
 
 
@@ -303,6 +237,7 @@ KUrlNavigator::Private::Private(KUrlNavigator* q, KFilePlacesModel* placesModel)
     m_editable(false),
     m_active(true),
     m_showPlacesSelector(placesModel != 0),
+    m_showFullPath(false),
     m_historyIndex(0),
     m_layout(new QHBoxLayout),
     m_placesSelector(0),
@@ -324,27 +259,49 @@ KUrlNavigator::Private::Private(KUrlNavigator* q, KFilePlacesModel* placesModel)
         m_placesSelector = new KFilePlacesSelector(q, placesModel);
         connect(m_placesSelector, SIGNAL(placeActivated(const KUrl&)),
                 q, SLOT(setUrl(const KUrl&)));
+
+        connect(placesModel, SIGNAL(rowsInserted(QModelIndex, int, int)),
+                q, SLOT(updateContent()));
+        connect(placesModel, SIGNAL(rowsRemoved(QModelIndex, int, int)),
+                q, SLOT(updateContent()));
+        connect(placesModel, SIGNAL(dataChanged(QModelIndex, QModelIndex)),
+                q, SLOT(updateContent()));
     }
 
+    // create protocol combo
+    m_protocols = new KProtocolCombo(QString(), q);
+    connect(m_protocols, SIGNAL(activated(QString)),
+            q, SLOT(slotProtocolChanged(QString)));
+
+    // create editor for editing the host
+    m_host = new KLineEdit(QString(), q);
+    m_host->setClearButtonShown(true);
+    connect(m_host, SIGNAL(editingFinished()),
+            q, SLOT(slotRemoteHostActivated()));
+    connect(m_host, SIGNAL(returnPressed()),
+            q, SIGNAL(returnPressed()));
+
+    // create drop down button for accessing all paths of the URL
     m_dropDownButton = new KUrlDropDownButton(q);
     connect(m_dropDownButton, SIGNAL(clicked()),
             q, SLOT(openPathSelectorMenu()));
 
     // initialize the path box of the traditional view
     m_pathBox = new KUrlComboBox(KUrlComboBox::Both, true, q);
-    m_pathBox->setMinimumWidth(50);
+    m_pathBox->setSizeAdjustPolicy(QComboBox::AdjustToMinimumContentsLength);
+    m_pathBox->installEventFilter(q);
 
     KUrlCompletion* kurlCompletion = new KUrlCompletion(KUrlCompletion::DirCompletion);
     m_pathBox->setCompletionObject(kurlCompletion);
     m_pathBox->setAutoDeleteCompletionObject(true);
 
-    connect(m_pathBox, SIGNAL(returnPressed(QString)),
-            q, SLOT(slotReturnPressed(QString)));
     connect(m_pathBox, SIGNAL(returnPressed()),
             q, SLOT(slotReturnPressed()));
     connect(m_pathBox, SIGNAL(urlActivated(KUrl)),
             q, SLOT(setUrl(KUrl)));
 
+    // create toggle button which allows to switch between
+    // the breadcrumb and traditional view
     m_toggleEditableMode = new KUrlToggleButton(q);
     m_toggleEditableMode->setMinimumWidth(20);
     connect(m_toggleEditableMode, SIGNAL(clicked()),
@@ -353,7 +310,10 @@ KUrlNavigator::Private::Private(KUrlNavigator* q, KFilePlacesModel* placesModel)
     if (m_placesSelector != 0) {
         m_layout->addWidget(m_placesSelector);
     }
+    m_layout->addWidget(m_protocols);
     m_layout->addWidget(m_dropDownButton);
+    m_layout->addWidget(m_host);
+    m_layout->setStretchFactor(m_host, 1);
     m_layout->addWidget(m_pathBox, 1);
     m_layout->addWidget(m_toggleEditableMode);
 }
@@ -363,7 +323,7 @@ void KUrlNavigator::Private::appendWidget(QWidget* widget, int stretch)
     m_layout->insertWidget(m_layout->count() - 1, widget, stretch);
 }
 
-void KUrlNavigator::Private::slotReturnPressed(const QString& text)
+void KUrlNavigator::Private::slotReturnPressed()
 {
     // Parts of the following code have been taken
     // from the class KateFileSelector located in
@@ -372,11 +332,7 @@ void KUrlNavigator::Private::slotReturnPressed(const QString& text)
     // Copyright (C) 2001 Joseph Wenninger <jowenn@kde.org>
     // Copyright (C) 2001 Anders Lund <anders.lund@lund.tdcadsl.dk>
 
-    KUrl typedUrl(text);
-    if (typedUrl.hasPass()) {
-        typedUrl.setPass(QString());
-    }
-
+    const KUrl typedUrl = q->uncommittedUrl();
     QStringList urls = m_pathBox->urls();
     urls.removeAll(typedUrl.url());
     urls.prepend(typedUrl.url());
@@ -388,12 +344,13 @@ void KUrlNavigator::Private::slotReturnPressed(const QString& text)
     m_pathBox->setUrl(q->url());
 
     emit q->returnPressed();
-}
 
-void KUrlNavigator::Private::slotReturnPressed()
-{
-    const QString text = q->uncommittedUrl().prettyUrl();
-    slotReturnPressed(text);
+    if (QApplication::keyboardModifiers() & Qt::ControlModifier) {
+        // Pressing Ctrl+Return automatically switches back to the breadcrumb mode.
+        // The switch must be done asynchronously, as we are in the context of the
+        // editor.
+        QMetaObject::invokeMethod(q, "switchToBreadcrumbMode", Qt::QueuedConnection);
+    }
 }
 
 void KUrlNavigator::Private::slotRemoteHostActivated()
@@ -440,11 +397,7 @@ void KUrlNavigator::Private::slotProtocolChanged(const QString& protocol)
     if (KProtocolInfo::protocolClass(protocol) == ":local") {
         q->setUrl(url);
     } else {
-        if (m_host == 0) {
-            createHostLineEdit("");
-        } else {
-            m_host->setText("");
-        }
+        m_host->setText(QString());
         m_host->show();
         m_host->setFocus();
     }
@@ -452,26 +405,46 @@ void KUrlNavigator::Private::slotProtocolChanged(const QString& protocol)
 
 void KUrlNavigator::Private::openPathSelectorMenu()
 {
-    KMenu* popup = new KMenu(q);
-
-    QString spacer;
-    QLinkedList<KUrlNavigatorButton*>::iterator it = m_navButtons.begin();
-    const QLinkedList<KUrlNavigatorButton*>::const_iterator itEnd = m_navButtons.end();
-    while (it != itEnd) {
-        const QString text = spacer + (*it)->text();
-        spacer.append("  ");
-
-        QAction* action = new QAction(text, popup);
-        action->setData(QVariant((*it)->index()));
-        popup->addAction(action);
-
-        ++it;
+    if (m_navButtons.count() <= 0) {
+        return;
     }
 
-    const QAction* activatedAction = popup->exec(QCursor::pos());
+    const KUrl firstVisibleUrl = q->url(m_navButtons.first()->index());
+
+    QString spacer;
+    KMenu* popup = new KMenu(q);
+    popup->setLayoutDirection(Qt::LeftToRight);
+
+    const QString placePath = retrievePlacePath();
+    int idx = placePath.count('/'); // idx points to the first directory
+                                    // after the place path
+
+    const QString path = q->url().pathOrUrl();
+    QString dirName = path.section('/', idx, idx);
+    if (dirName.isEmpty()) {
+        dirName = QChar('/');
+    }
+    do {
+        const QString text = spacer + dirName;
+
+        QAction* action = new QAction(text, popup);
+        const KUrl currentUrl = q->url(idx);
+        if (currentUrl == firstVisibleUrl) {
+            popup->addSeparator();
+        }
+        action->setData(QVariant(currentUrl.prettyUrl()));
+        popup->addAction(action);
+
+        ++idx;
+        spacer.append("  ");
+        dirName = path.section('/', idx, idx);
+    } while (!dirName.isEmpty());
+
+    const QPoint pos = q->mapToGlobal(m_dropDownButton->geometry().bottomRight());
+    const QAction* activatedAction = popup->exec(pos);
     if (activatedAction != 0) {
-        const int index = activatedAction->data().toInt();
-        q->setUrl(q->url(index));
+        const KUrl url = KUrl(activatedAction->data().toString());
+        q->setUrl(url);
     }
 
     popup->deleteLater();
@@ -486,13 +459,21 @@ void KUrlNavigator::Private::switchView()
     if (q->isUrlEditable()) {
         m_pathBox->setFocus();
     }
+
     emit q->requestActivation();
+    emit q->editableStateChanged(m_editable);
 }
 
-void KUrlNavigator::Private::dropUrls(const KUrl::List& urls,
-                                      const KUrl& destination)
+void KUrlNavigator::Private::dropUrls(const KUrl& destination, QDropEvent* event)
 {
-    emit q->urlsDropped(urls, destination);
+    const KUrl::List urls = KUrl::List::fromMimeData(event->mimeData());
+    if (!urls.isEmpty()) {
+        emit q->urlsDropped(destination, event);
+
+        // KDE5: remove, as the signal has been replaced by
+        // urlsDropped(const KUrl& destination, QDropEvent* event)
+        emit q->urlsDropped(urls, destination);
+    }
 }
 
 void KUrlNavigator::Private::updateContent()
@@ -502,96 +483,71 @@ void KUrlNavigator::Private::updateContent()
     }
 
     if (m_editable) {
-        delete m_protocols;
-        m_protocols = 0;
-        delete m_host;
-        m_host = 0;
-
+        m_protocols->hide();
+        m_host->hide();
         m_dropDownButton->hide();
+
         deleteButtons();
         m_toggleEditableMode->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
-
         q->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
+
         m_pathBox->show();
         m_pathBox->setUrl(q->url());
     } else {
-        const QString path = q->url().pathOrUrl();
-
-        q->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+        m_dropDownButton->setVisible(!m_showFullPath);
         m_pathBox->hide();
-        m_toggleEditableMode->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
 
-        // get the data from the currently selected place
-        KUrl placeUrl = KUrl();
-        if (m_placesSelector != 0) {
+        const KUrl currentUrl = q->url();
+        QString path = currentUrl.pathOrUrl();
+        removeTrailingSlash(path);
+
+        m_toggleEditableMode->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
+        q->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+
+        // The URL consists of a protocol, a host and a variable number of directories.
+        // The directories are mapped to URL navigator buttons represented by m_buttons.
+        // - If a part of the URL is represented by a Places bookmark, this part will
+        //   be also represented as button if m_showFullPath is false.
+        // - If no part of the URL is represented by a Places bookmark, a protocols-combo
+        //   and a host-editor will be shown if no subdirectories are available.
+        KUrl placeUrl;
+        if ((m_placesSelector != 0) && !m_showFullPath) {
             placeUrl = m_placesSelector->selectedPlaceUrl();
         }
 
-        const QString placePath = placeUrl.isValid() ? placeUrl.pathOrUrl() : retrievePlacePath(path);
-        const uint len = placePath.length();
+        QString placePath = placeUrl.isValid() ? placeUrl.pathOrUrl() : retrievePlacePath();
+        removeTrailingSlash(placePath);
 
-        // calculate the start point for the URL navigator buttons by counting
-        // the slashs inside the place URL
-        int slashCount = 0;
-        for (uint i = 0; i < len; ++i) {
-            if (placePath.at(i) == QChar('/')) {
-                ++slashCount;
-            }
+        // update the protocol-combo
+        const QString protocol = currentUrl.scheme();
+        m_protocols->setProtocol(protocol);
+
+        // update the host-editor
+        QString hostText = currentUrl.host();
+        if (!currentUrl.user().isEmpty()) {
+            hostText = currentUrl.user() + '@' + hostText;
         }
-        if ((len > 0) && placePath.at(len - 1) == QChar('/')) {
-            Q_ASSERT(slashCount > 0);
-            --slashCount;
+        if (currentUrl.port() != -1) {
+            hostText = hostText + ':' + QString::number(currentUrl.port());
         }
+        m_host->setText(hostText);
 
-        const KUrl currentUrl = q->url();
-        if (!currentUrl.isLocalFile() && !placeUrl.isValid()) {
-            QString protocol = currentUrl.scheme();
-            if (m_protocols == 0) {
-                deleteButtons();
+        // check whether the protocol-combo and the host-editor should be shown
+        const bool hasPlaceItem = currentUrl.isLocalFile() || placeUrl.isValid();
+        const bool isVisible = !hasPlaceItem &&
+                               (placePath == path) &&
+                               (KProtocolInfo::protocolClass(protocol) != ":local");
 
-                m_protocols = new KProtocolCombo(protocol, q);
-                const int index = m_layout->indexOf(m_dropDownButton);
-                m_layout->insertWidget(index, m_protocols);
+        m_host->setVisible(isVisible);
+        m_protocols->setVisible(isVisible);
 
-                if (!m_customProtocols.isEmpty()) {
-                    m_protocols->setCustomProtocols(m_customProtocols);
-                }
-                connect(m_protocols, SIGNAL(activated(QString)),
-                        q, SLOT(slotProtocolChanged(QString)));
-            } else {
-                m_protocols->setProtocol(protocol);
-            }
-            m_protocols->show();
-
-            if (KProtocolInfo::protocolClass(protocol) != ":local") {
-                QString hostText = currentUrl.host();
-
-                if (!currentUrl.user().isEmpty()) {
-                    hostText = currentUrl.user() + '@' + hostText;
-                }
-
-                if (currentUrl.port() != -1) {
-                    hostText = hostText + ':' + QString::number(currentUrl.port());
-                }
-
-                if (m_host == 0) {
-                    createHostLineEdit(hostText);
-                } else {
-                    m_host->setText(hostText);
-                }
-                m_host->show();
-            } else {
-                delete m_host;
-                m_host = 0;
-            }
-        } else if (m_protocols != 0) {
-            m_protocols->hide();
-            if (m_host != 0) {
-                m_host->hide();
-            }
+        // calculate the start index for the directories that should be shown as buttons
+        // and create the buttons
+        int startIndex = placePath.count('/');
+        if (!isVisible && !hasPlaceItem && hostText.isEmpty()) {
+            --startIndex;
         }
-
-        updateButtons(path, slashCount);
+        updateButtons(path, startIndex);
     }
 }
 
@@ -615,14 +571,18 @@ void KUrlNavigator::Private::updateButtons(const QString& path, int startIndex)
             if (isFirstButton) {
                 // the first URL navigator button should get the name of the
                 // place instead of the directory name
-                if (m_placesSelector != 0) {
+                if ((m_placesSelector != 0) && !m_showFullPath) {
                     const KUrl placeUrl = m_placesSelector->selectedPlaceUrl();
                     text = m_placesSelector->selectedPlaceText();
                 }
                 if (text.isEmpty()) {
                     if (currentUrl.isLocalFile()) {
-                        text = i18n("Custom Path");
+                        text = m_showFullPath ? "/" : i18n("Custom Path");
+                    } else if (!m_host->isVisible() && !m_host->text().isEmpty()) {
+                        text = m_host->text();
                     } else {
+                        // The host is already displayed by the m_host widget,
+                        // no button may be added for this index.
                         ++idx;
                         continue;
                     }
@@ -632,8 +592,8 @@ void KUrlNavigator::Private::updateButtons(const QString& path, int startIndex)
             KUrlNavigatorButton* button = 0;
             if (createButton) {
                 button = new KUrlNavigatorButton(idx, q);
-                connect(button, SIGNAL(urlsDropped(const KUrl::List&, const KUrl&)),
-                        q, SLOT(dropUrls(const KUrl::List&, const KUrl&)));
+                connect(button, SIGNAL(urlsDropped(const KUrl&, QDropEvent*)),
+                        q, SLOT(dropUrls(const KUrl&, QDropEvent*)));
                 appendWidget(button);
             } else {
                 button = *it;
@@ -642,7 +602,6 @@ void KUrlNavigator::Private::updateButtons(const QString& path, int startIndex)
 
             if (isFirstButton) {
                 button->setText(text);
-                button->updateMinimumWidth();
             }
 
             if (createButton) {
@@ -673,25 +632,11 @@ void KUrlNavigator::Private::updateButtonVisibility()
     }
 
     const int buttonsCount = m_navButtons.count();
-    if (m_host != 0) {
-        const bool optimize = (buttonsCount != 0);
-        m_host->setOptimizeWidth(optimize);
-        if (optimize) {
-            m_host->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
-            m_layout->setStretchFactor(m_host, 0);
-        } else {
-            m_host->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
-            m_layout->setStretchFactor(m_host, 1);
-        }
-    }
-
     if (buttonsCount == 0) {
         m_dropDownButton->hide();
         return;
     }
 
-    int hiddenButtonsCount = 0;
-
     // subtract all widgets from the available width, that must be shown anyway
     int availableWidth = q->width() - m_toggleEditableMode->minimumWidth();
 
@@ -703,7 +648,7 @@ void KUrlNavigator::Private::updateButtonVisibility()
         availableWidth -= m_protocols->width();
     }
 
-    if ((m_host != 0) && m_host->isVisible()) {
+    if (m_host->isVisible()) {
         availableWidth -= m_host->width();
     }
 
@@ -722,35 +667,45 @@ void KUrlNavigator::Private::updateButtonVisibility()
     // hide buttons...
     QLinkedList<KUrlNavigatorButton*>::iterator it = m_navButtons.end();
     const QLinkedList<KUrlNavigatorButton*>::const_iterator itBegin = m_navButtons.begin();
+    bool isLastButton = true;
+    bool hasHiddenButtons = false;
+
+    QLinkedList<KUrlNavigatorButton*> buttonsToShow;
     while (it != itBegin) {
         --it;
         KUrlNavigatorButton* button = (*it);
         availableWidth -= button->minimumWidth();
-        if (availableWidth <= 0) {
+        if ((availableWidth <= 0) && !isLastButton) {
             button->hide();
-            ++hiddenButtonsCount;
+            hasHiddenButtons = true;
         }
         else {
-            button->show();
+            button->setActive(isLastButton);
+            // Don't show the button immediately, as setActive()
+            // might change the size and a relayout gets triggered
+            // after showing the button. So the showing of all buttons
+            // is postponed until all buttons have the correct
+            // activation state.
+            buttonsToShow.append(button);
         }
+        isLastButton = false;
     }
 
-    Q_ASSERT(hiddenButtonsCount <= buttonsCount);
-    if (hiddenButtonsCount == buttonsCount) {
-        // assure that at least one button is visible
-        hiddenButtonsCount = buttonsCount - 1;
+    // all buttons have the correct activation state and
+    // can be shown now
+    foreach (KUrlNavigatorButton* button, buttonsToShow) {
+        button->show();
     }
 
-    int index = 0;
-    it = m_navButtons.begin();
-    const QLinkedList<KUrlNavigatorButton*>::const_iterator itEnd = m_navButtons.end();
-    while (it != itEnd) {
-        (*it)->setVisible(index >= hiddenButtonsCount);
-        ++it;
-        ++index;
-    }
+    const int startIndex = retrievePlacePath().count('/');
+    const bool showDropDownButton = hasHiddenButtons ||
+                                    (!hasHiddenButtons && (m_navButtons.front()->index() > startIndex));
+    m_dropDownButton->setVisible(showDropDownButton);
+}
 
-    m_dropDownButton->setVisible(hiddenButtonsCount != 0);
+void KUrlNavigator::Private::switchToBreadcrumbMode()
+{
+    q->setUrlEditable(false);
 }
 
 void KUrlNavigator::Private::deleteButtons()
@@ -762,8 +717,9 @@ void KUrlNavigator::Private::deleteButtons()
     m_navButtons.clear();
 }
 
-QString KUrlNavigator::Private::retrievePlacePath(const QString& path) const
+QString KUrlNavigator::Private::retrievePlacePath() const
 {
+    const QString path = q->url().pathOrUrl();
     int idx = path.indexOf(QLatin1String("///"));
     if (idx >= 0) {
         idx += 3;
@@ -771,7 +727,10 @@ QString KUrlNavigator::Private::retrievePlacePath(const QString& path) const
         idx = path.indexOf(QLatin1String("//"));
         idx = path.indexOf(QLatin1Char('/'), (idx < 0) ? 0 : idx + 2);
     }
-    return (idx < 0) ? path : path.left(idx);
+
+    QString placePath = (idx < 0) ? path : path.left(idx);
+    removeTrailingSlash(placePath);
+    return placePath;
 }
 
 bool KUrlNavigator::Private::isCompressedPath(const KUrl& url) const
@@ -780,6 +739,8 @@ bool KUrlNavigator::Private::isCompressedPath(const KUrl& url) const
     // Note: this list of MIME types depends on the protocols implemented by kio_archive
     return  mime->is("application/x-compressed-tar") ||
             mime->is("application/x-bzip-compressed-tar") ||
+	    mime->is("application/x-lzma-compressed-tar") ||
+	    mime->is("application/x-xz-compressed-tar") ||
             mime->is("application/x-tar") ||
             mime->is("application/x-tarz") ||
             mime->is("application/x-tzo") || // (not sure KTar supports those?)
@@ -787,20 +748,12 @@ bool KUrlNavigator::Private::isCompressedPath(const KUrl& url) const
             mime->is("application/x-archive");
 }
 
-void KUrlNavigator::Private::createHostLineEdit(const QString& text)
+void KUrlNavigator::Private::removeTrailingSlash(QString& url) const
 {
-    Q_ASSERT(m_host == 0);
-
-    m_host = new HostLineEdit(text, q);
-    m_host->setClearButtonShown(true);
-
-    const int index = m_layout->indexOf(m_dropDownButton);
-    m_layout->insertWidget(index, m_host);
-
-    connect(m_host, SIGNAL(editingFinished()),
-            q, SLOT(slotRemoteHostActivated()));
-    connect(m_host, SIGNAL(returnPressed()),
-            q, SIGNAL(returnPressed()));
+    const int length = url.length();
+    if ((length > 0) && (url.at(length - 1) == QChar('/'))) {
+        url.remove(length -1, 1);
+    }
 }
 
 ////
@@ -812,6 +765,7 @@ KUrlNavigator::KUrlNavigator(KFilePlacesModel* placesModel,
     d(new Private(this, placesModel))
 {
     d->m_history.prepend(HistoryElem(url));
+    setLayoutDirection(Qt::LeftToRight);
 
     const QFont font = KGlobalSettings::generalFont();
     setFont(font);
@@ -839,7 +793,7 @@ const KUrl& KUrlNavigator::url() const
 KUrl KUrlNavigator::uncommittedUrl() const
 {
     if (isUrlEditable()) {
-        return KUrl(d->m_pathBox->currentText());
+        return KUrl(d->m_pathBox->currentText().trimmed());
     } else {
         return KUrl(d->m_protocols->currentProtocol() + "://" + d->m_host->text());
     }
@@ -851,8 +805,8 @@ KUrl KUrlNavigator::url(int index) const
         index = 0;
     }
 
-    // keep scheme, hostname etc. maybe we will need this in the future
-    // for e.g. browsing ftp repositories.
+    // keep scheme, hostname etc. as this is needed for e. g. browsing
+    // FTP directories
     KUrl newUrl = url();
     newUrl.setPath(QString());
 
@@ -940,6 +894,20 @@ bool KUrlNavigator::isUrlEditable() const
     return d->m_editable;
 }
 
+void KUrlNavigator::setShowFullPath(bool show)
+{
+    if (d->m_showFullPath != show) {
+        d->m_showFullPath = show;
+        d->updateContent();
+    }
+}
+
+bool KUrlNavigator::showFullPath() const
+{
+    return d->m_showFullPath;
+}
+
+
 void KUrlNavigator::setActive(bool active)
 {
     if (active != d->m_active) {
@@ -977,12 +945,17 @@ bool KUrlNavigator::isPlacesSelectorVisible() const
     return d->m_showPlacesSelector;
 }
 
-void KUrlNavigator::setUrl(const KUrl& url)
+void KUrlNavigator::setUrl(const KUrl& newUrl)
 {
-    QString urlStr(url.pathOrUrl());
+    if (newUrl == url()) {
+        return;
+    }
 
-    //kDebug() << "setUrl(" << url << ")";
-    if (urlStr.length() > 0 && urlStr.at(0) == '~') {
+    KUrl url = newUrl;
+    url.cleanPath();
+
+    QString urlStr = KUrlCompletion::replacedPath(url.url(), true, true);
+    if ((urlStr.length() > 0) && (urlStr.at(0) == '~')) {
         // replace '~' by the home directory
         urlStr.remove(0, 1);
         urlStr.insert(0, QDir::homePath());
@@ -1014,32 +987,40 @@ void KUrlNavigator::setUrl(const KUrl& url)
 
     const KUrl transformedUrl(urlStr);
 
+    // Check whether current history element has the same URL.
+    // If this is the case, just ignore setting the URL.
+    const HistoryElem& historyElem = d->m_history[d->m_historyIndex];
+    const bool isUrlEqual = transformedUrl.equals(historyElem.url(), KUrl::CompareWithoutTrailingSlash) ||
+                            (!transformedUrl.isValid() && (urlStr == historyElem.url().url()));
+    if (isUrlEqual) {
+        return;
+    }
+
     if (d->m_historyIndex > 0) {
-        // Check whether the previous element of the history has the same Url.
-        // If yes, just go forward instead of inserting a duplicate history
-        // element.
-        HistoryElem& prevHistoryElem = d->m_history[d->m_historyIndex - 1];
-        if (transformedUrl == prevHistoryElem.url()) {
-            goForward();
-            return;
-        }
+        // If an URL is set when the history index is not at the end (= 0),
+        // then clear all previous history elements so that a new history
+        // tree is started from the current position.
+        QList<HistoryElem>::iterator begin = d->m_history.begin();
+        QList<HistoryElem>::iterator end = begin + d->m_historyIndex;
+        d->m_history.erase(begin, end);
+        d->m_historyIndex = 0;
     }
 
-    if (this->url() != transformedUrl) {
-        // don't insert duplicate history elements
-        d->m_history.insert(d->m_historyIndex, HistoryElem(transformedUrl));
+    Q_ASSERT(d->m_historyIndex == 0);
+    d->m_history.insert(0, HistoryElem(transformedUrl));
 
-        emit historyChanged();
-        emit urlChanged(transformedUrl);
-
-        // Prevent an endless growing of the history: remembering
-        // the last 100 Urls should be enough...
-        if (d->m_historyIndex > 100) {
-            d->m_history.removeFirst();
-            --d->m_historyIndex;
-        }
+    // Prevent an endless growing of the history: remembering
+    // the last 100 Urls should be enough...
+    const int historyMax = 100;
+    if (d->m_history.size() > historyMax) {
+        QList<HistoryElem>::iterator begin = d->m_history.begin() + historyMax;
+        QList<HistoryElem>::iterator end = d->m_history.end();
+        d->m_history.erase(begin, end);
     }
 
+    emit historyChanged();
+    emit urlChanged(transformedUrl);
+
     d->updateContent();
 
     requestActivation();
@@ -1090,6 +1071,16 @@ void KUrlNavigator::resizeEvent(QResizeEvent* event)
     QWidget::resizeEvent(event);
 }
 
+bool KUrlNavigator::eventFilter(QObject* watched, QEvent* event)
+{
+    if ((watched == d->m_pathBox) && (event->type() == QEvent::FocusIn)) {
+        requestActivation();
+        setFocus();
+    }
+
+    return QWidget::eventFilter(watched, event);
+}
+
 int KUrlNavigator::historySize() const
 {
     return d->m_history.count();
@@ -1100,6 +1091,13 @@ int KUrlNavigator::historyIndex() const
     return d->m_historyIndex;
 }
 
+KUrl KUrlNavigator::historyUrl(int historyIndex) const
+{
+    Q_ASSERT(historyIndex >= 0);
+    Q_ASSERT(historyIndex < historySize());
+    return d->m_history[historyIndex].url();
+}
+
 const KUrl& KUrlNavigator::savedRootUrl() const
 {
     const HistoryElem& histElem = d->m_history[d->m_historyIndex];
@@ -1120,7 +1118,6 @@ KUrlComboBox* KUrlNavigator::editor() const
 void KUrlNavigator::setCustomProtocols(const QStringList &protocols)
 {
     d->m_customProtocols = protocols;
-
     d->m_protocols->setCustomProtocols(d->m_customProtocols);
 }
 
