@@ -1,6 +1,6 @@
 /* This file is part of the KDE libraries
     Copyright (C) 2000 Stephan Kulow <coolo@kde.org>
-                       David Faure <faure@kde.org>
+                  2000-2009 David Faure <faure@kde.org>
                        Waldo Bastian <bastian@kde.org>
 
     This library is free software; you can redistribute it and/or
@@ -61,15 +61,9 @@ extern "C" {
 #include "kprotocolmanager.h"
 #include "filejob.h"
 
-#include "kssl/ksslcsessioncache.h"
-
 #include <kdirnotify.h>
 #include <ktemporaryfile.h>
 
-#ifdef Q_OS_UNIX
-#include <utime.h>
-#endif
-
 using namespace KIO;
 
 static inline Slave *jobSlave(SimpleJob *job)
@@ -113,8 +107,8 @@ bool Job::addSubjob(KJob *jobBase)
         job->mergeMetaData(d->m_outgoingMetaData);
 
         // Forward information from that subjob.
-        connect( job, SIGNAL(speed( KJob*, unsigned long )),
-                 SLOT(slotSpeed(KJob*, unsigned long)) );
+        connect(job, SIGNAL(speed(KJob*,ulong)),
+                SLOT(slotSpeed(KJob*,ulong)));
 
         if (ui() && job->ui()) {
             job->ui()->setWindow( ui()->window() );
@@ -133,15 +127,15 @@ bool Job::removeSubjob( KJob *jobBase )
 void JobPrivate::emitMoving(KIO::Job * job, const KUrl &src, const KUrl &dest)
 {
     emit job->description(job, i18nc("@title job","Moving"),
-                          qMakePair(i18n("Source"), src.prettyUrl()),
-                          qMakePair(i18n("Destination"), dest.prettyUrl()));
+                          qMakePair(i18nc("The source of a file operation", "Source"), src.prettyUrl()),
+                          qMakePair(i18nc("The destination of a file operation", "Destination"), dest.prettyUrl()));
 }
 
 void JobPrivate::emitCopying(KIO::Job * job, const KUrl &src, const KUrl &dest)
 {
     emit job->description(job, i18nc("@title job","Copying"),
-                          qMakePair(i18n("Source"), src.prettyUrl()),
-                          qMakePair(i18n("Destination"), dest.prettyUrl()));
+                          qMakePair(i18nc("The source of a file operation", "Source"), src.prettyUrl()),
+                          qMakePair(i18nc("The destination of a file operation", "Destination"), dest.prettyUrl()));
 }
 
 void JobPrivate::emitCreatingDir(KIO::Job * job, const KUrl &dir)
@@ -158,14 +152,14 @@ void JobPrivate::emitDeleting(KIO::Job *job, const KUrl &url)
 
 void JobPrivate::emitStating(KIO::Job *job, const KUrl &url)
 {
-    emit job->description(job, i18nc("@title job","Stating"),
+    emit job->description(job, i18nc("@title job","Examining"),
                           qMakePair(i18n("File"), url.prettyUrl()));
 }
 
 void JobPrivate::emitTransferring(KIO::Job *job, const KUrl &url)
 {
     emit job->description(job, i18nc("@title job","Transferring"),
-                          qMakePair(i18n("Source"), url.prettyUrl()));
+                          qMakePair(i18nc("The source of a file operation", "Source"), url.prettyUrl()));
 }
 
 void JobPrivate::emitMounting(KIO::Job * job, const QString &dev, const QString &point)
@@ -183,13 +177,10 @@ void JobPrivate::emitUnmounting(KIO::Job * job, const QString &point)
 
 bool Job::doKill()
 {
-  kDebug(7007) << "this=" << this << metaObject()->className();
   // kill all subjobs, without triggering their result slot
-  const QList<KJob *> &jobs = subjobs();
-  QList<KJob *>::const_iterator it = jobs.constBegin();
-  const QList<KJob *>::const_iterator end = jobs.constEnd();
-  for ( ; it != end ; ++it )
-    (*it)->kill( KJob::Quietly );
+  Q_FOREACH( KJob* it, subjobs()) {
+      it->kill( KJob::Quietly );
+  }
   clearSubjobs();
 
   return true;
@@ -197,12 +188,8 @@ bool Job::doKill()
 
 bool Job::doSuspend()
 {
-    const QList<KJob *> &jobs = subjobs();
-    QList<KJob *>::const_iterator it = jobs.constBegin();
-    const QList<KJob *>::const_iterator end = jobs.constEnd();
-    for ( ; it != end ; ++it )
-    {
-        if (!(*it)->suspend())
+    Q_FOREACH(KJob* it, subjobs()) {
+        if (!it->suspend())
             return false;
     }
 
@@ -211,12 +198,9 @@ bool Job::doSuspend()
 
 bool Job::doResume()
 {
-    const QList<KJob *> &jobs = subjobs();
-    QList<KJob *>::const_iterator it = jobs.constBegin();
-    const QList<KJob *>::const_iterator end = jobs.constEnd();
-    for ( ; it != end ; ++it )
+    Q_FOREACH ( KJob* it, subjobs() )
     {
-        if (!(*it)->resume())
+        if (!it->resume())
             return false;
     }
 
@@ -329,6 +313,7 @@ void SimpleJobPrivate::simpleJobInit()
 
 bool SimpleJob::doKill()
 {
+    //kDebug() << this;
     Q_D(SimpleJob);
     Scheduler::cancelJob( this ); // deletes the slave if not 0
     d->m_slave = 0; // -> set to 0
@@ -383,9 +368,9 @@ SimpleJob::~SimpleJob()
         d->m_slave->kill();
         Scheduler::jobFinished( this, d->m_slave ); // deletes the slave
 #endif
-        Scheduler::cancelJob( this );
         d->m_slave = 0; // -> set to 0
     }
+    Scheduler::cancelJob( this );
 }
 
 void SimpleJobPrivate::start(Slave *slave)
@@ -393,34 +378,34 @@ void SimpleJobPrivate::start(Slave *slave)
     Q_Q(SimpleJob);
     m_slave = slave;
 
-    q->connect( slave, SIGNAL( error( int , const QString & ) ),
-                SLOT( slotError( int , const QString & ) ) );
+    q->connect( slave, SIGNAL(error(int,QString)),
+                SLOT(slotError(int,QString)) );
 
-    q->connect( slave, SIGNAL( warning( const QString & ) ),
-                SLOT( slotWarning( const QString & ) ) );
+    q->connect( slave, SIGNAL(warning(QString)),
+                SLOT(slotWarning(QString)) );
 
-    q->connect( slave, SIGNAL( infoMessage( const QString & ) ),
-                SLOT( _k_slotSlaveInfoMessage( const QString & ) ) );
+    q->connect( slave, SIGNAL(infoMessage(QString)),
+                SLOT(_k_slotSlaveInfoMessage(QString)) );
 
-    q->connect( slave, SIGNAL( connected() ),
-                SLOT( slotConnected() ) );
+    q->connect( slave, SIGNAL(connected()),
+                SLOT(slotConnected()));
 
-    q->connect( slave, SIGNAL( finished() ),
-                SLOT( slotFinished() ) );
+    q->connect( slave, SIGNAL(finished()),
+                SLOT(slotFinished()) );
 
-    if ((m_extraFlags & EF_TransferJobDataSent) == 0)
+    if ((m_extraFlags & EF_TransferJobDataSent) == 0) // this is a "get" job
     {
-        q->connect( slave, SIGNAL( totalSize( KIO::filesize_t ) ),
-                    SLOT( slotTotalSize( KIO::filesize_t ) ) );
+        q->connect( slave, SIGNAL(totalSize(KIO::filesize_t)),
+                    SLOT(slotTotalSize(KIO::filesize_t)) );
 
-        q->connect( slave, SIGNAL( processedSize( KIO::filesize_t ) ),
-                    SLOT( slotProcessedSize( KIO::filesize_t ) ) );
+        q->connect( slave, SIGNAL(processedSize(KIO::filesize_t)),
+                    SLOT(slotProcessedSize(KIO::filesize_t)) );
 
-        q->connect( slave, SIGNAL( speed( unsigned long ) ),
-                    SLOT( slotSpeed( unsigned long ) ) );
+        q->connect( slave, SIGNAL(speed(ulong)),
+                    SLOT(slotSpeed(ulong)) );
     }
-    q->connect( slave, SIGNAL(metaData( const KIO::MetaData& ) ),
-                SLOT( slotMetaData( const KIO::MetaData& ) ) );
+    q->connect( slave, SIGNAL(metaData(KIO::MetaData)),
+                SLOT(slotMetaData(KIO::MetaData)) );
 
     if (ui() && ui()->window())
     {
@@ -432,12 +417,6 @@ void SimpleJobPrivate::start(Slave *slave)
         m_outgoingMetaData.insert("user-timestamp", QString::number(ui()->userTimestamp()));
     }
 
-    QString sslSession = KSSLCSessionCache::getSessionForUrl(m_url);
-    if ( !sslSession.isNull() )
-    {
-        m_outgoingMetaData.insert("ssl_session_id", sslSession);
-    }
-
     if (ui() == 0)              // not interactive
     {
         m_outgoingMetaData.insert("no-auth-prompt", "true");
@@ -555,13 +534,7 @@ void SimpleJob::slotMetaData( const KIO::MetaData &_metaData )
 
 void SimpleJob::storeSSLSessionFromJob(const KUrl &redirectionURL)
 {
-    Q_D(SimpleJob);
-    QString sslSession = queryMetaData("ssl_session_id");
-
-    if ( !sslSession.isNull() ) {
-	    const KUrl &queryURL = redirectionURL.isEmpty() ? d->m_url : redirectionURL;
-	    KSSLCSessionCache::putSessionForUrl(queryURL, sslSession);
-    }
+    Q_UNUSED(redirectionURL);
 }
 
 //////////
@@ -879,7 +852,7 @@ StatJob *KIO::stat(const KUrl& url, JobFlags flags)
 
 StatJob *KIO::stat(const KUrl& url, bool sideIsSource, short int details, JobFlags flags )
 {
-    kDebug(7007) << "stat" << url;
+    //kDebug(7007) << "stat" << url;
     KIO_ARGS << url;
     StatJob * job = StatJobPrivate::newJob(url, CMD_STAT, packedArgs, flags);
     job->setSide( sideIsSource ? StatJob::SourceSide : StatJob::DestinationSide );
@@ -889,7 +862,7 @@ StatJob *KIO::stat(const KUrl& url, bool sideIsSource, short int details, JobFla
 
 StatJob *KIO::stat(const KUrl& url, KIO::StatJob::StatSide side, short int details, JobFlags flags )
 {
-    kDebug(7007) << "stat" << url;
+    //kDebug(7007) << "stat" << url;
     KIO_ARGS << url;
     StatJob * job = StatJobPrivate::newJob(url, CMD_STAT, packedArgs, flags);
     job->setSide( side );
@@ -903,7 +876,6 @@ SimpleJob *KIO::http_update_cache( const KUrl& url, bool no_cache, time_t expire
     // Send http update_cache command (2)
     KIO_ARGS << (int)2 << url << no_cache << qlonglong(expireDate);
     SimpleJob * job = SimpleJobPrivate::newJob(url, CMD_SPECIAL, packedArgs);
-    job->setUiDelegate(new JobUiDelegate());
     Scheduler::scheduleJob(job);
     return job;
 }
@@ -913,6 +885,10 @@ SimpleJob *KIO::http_update_cache( const KUrl& url, bool no_cache, time_t expire
 TransferJob::TransferJob(TransferJobPrivate &dd)
     : SimpleJob(dd)
 {
+    Q_D(TransferJob);
+    if (d->m_command == CMD_PUT) {
+        d->m_extraFlags |= JobPrivate::EF_TransferJobDataSent;
+    }
 }
 
 TransferJob::~TransferJob()
@@ -927,6 +903,11 @@ void TransferJob::slotData( const QByteArray &_data)
       emit data( this, _data);
 }
 
+void KIO::TransferJob::setTotalSize(KIO::filesize_t bytes)
+{
+    setTotalAmount(KJob::Bytes, bytes);
+}
+
 // Slave got a redirection request
 void TransferJob::slotRedirection( const KUrl &url)
 {
@@ -1036,7 +1017,7 @@ void TransferJob::sendAsyncData(const QByteArray &dataForSlave)
     if (d->m_extraFlags & JobPrivate::EF_TransferJobNeedData)
     {
        d->m_slave->send( MSG_DATA, dataForSlave );
-       if (d->m_extraFlags & JobPrivate::EF_TransferJobDataSent)
+       if (d->m_extraFlags & JobPrivate::EF_TransferJobDataSent) // put job -> emit progress
        {
            KIO::filesize_t size = processedAmount(KJob::Bytes)+dataForSlave.size();
            setProcessedAmount(KJob::Bytes, size);
@@ -1248,13 +1229,43 @@ TransferJob *KIO::get( const KUrl& url, LoadType reload, JobFlags flags )
     return job;
 }
 
+class KIO::StoredTransferJobPrivate: public TransferJobPrivate
+{
+public:
+    StoredTransferJobPrivate(const KUrl& url, int command,
+                             const QByteArray &packedArgs,
+                             const QByteArray &_staticData)
+        : TransferJobPrivate(url, command, packedArgs, _staticData),
+          m_uploadOffset( 0 )
+        {}
+    QByteArray m_data;
+    int m_uploadOffset;
+
+    void slotStoredData( KIO::Job *job, const QByteArray &data );
+    void slotStoredDataReq( KIO::Job *job, QByteArray &data );
+
+    Q_DECLARE_PUBLIC(StoredTransferJob)
+
+    static inline StoredTransferJob *newJob(const KUrl &url, int command,
+                                            const QByteArray &packedArgs,
+                                            const QByteArray &staticData, JobFlags flags)
+    {
+        StoredTransferJob *job = new StoredTransferJob(
+            *new StoredTransferJobPrivate(url, command, packedArgs, staticData));
+        job->setUiDelegate(new JobUiDelegate);
+        if (!(flags & HideProgressInfo))
+            KIO::getJobTracker()->registerJob(job);
+        return job;
+    }
+};
+
 namespace KIO {
-    class PostErrorJob : public TransferJob
+    class PostErrorJob : public StoredTransferJob
     {
     public:
 
         PostErrorJob(int _error, const QString& url, const QByteArray &packedArgs, const QByteArray &postData)
-            : TransferJob(*new TransferJobPrivate(KUrl(), CMD_SPECIAL, packedArgs, postData))
+            : StoredTransferJob(*new StoredTransferJobPrivate(KUrl(), CMD_SPECIAL, packedArgs, postData))
             {
                 setError( _error );
                 setErrorText( url );
@@ -1263,7 +1274,7 @@ namespace KIO {
     };
 }
 
-TransferJob *KIO::http_post( const KUrl& url, const QByteArray &postData, JobFlags flags )
+static KIO::PostErrorJob* precheckHttpPost( const KUrl& url, const QByteArray& postData, JobFlags flags )
 {
     int _error = 0;
 
@@ -1347,8 +1358,8 @@ TransferJob *KIO::http_post( const KUrl& url, const QByteArray &postData, JobFla
             *overriden_ports = cfg.group(QString()).readEntry( "OverriddenPorts", QList<int>() );
             override_loaded = true;
         }
-        for( QList< int >::ConstIterator it = overriden_ports->begin();
-                it != overriden_ports->end();
+        for( QList< int >::ConstIterator it = overriden_ports->constBegin();
+                it != overriden_ports->constEnd();
                 ++it ) {
             if( overriden_ports->contains( url.port())) {
                 _error = 0;
@@ -1360,6 +1371,27 @@ TransferJob *KIO::http_post( const KUrl& url, const QByteArray &postData, JobFla
     if ((url.protocol() != "http") && (url.protocol() != "https" ))
         _error = KIO::ERR_POST_DENIED;
 
+    if (!_error && !KAuthorized::authorizeUrlAction("open", KUrl(), url))
+        _error = KIO::ERR_ACCESS_DENIED;
+
+    // if request is not valid, return an invalid transfer job
+    if (_error)
+    {
+        KIO_ARGS << (int)1 << url;
+        PostErrorJob * job = new PostErrorJob(_error, url.prettyUrl(), packedArgs, postData);
+        job->setUiDelegate(new JobUiDelegate());
+        if (!(flags & HideProgressInfo)) {
+            KIO::getJobTracker()->registerJob(job);
+        }
+        return job;
+    }
+
+    // all is ok, return 0
+    return 0;
+}
+
+TransferJob *KIO::http_post( const KUrl& url, const QByteArray &postData, JobFlags flags )
+{
     bool redirection = false;
     KUrl _url(url);
     if (_url.path().isEmpty())
@@ -1368,24 +1400,13 @@ TransferJob *KIO::http_post( const KUrl& url, const QByteArray &postData, JobFla
       _url.setPath("/");
     }
 
-    if (!_error && !KAuthorized::authorizeUrlAction("open", KUrl(), _url))
-        _error = KIO::ERR_ACCESS_DENIED;
-
-    // if request is not valid, return an invalid transfer job
-    if (_error)
-    {
-        KIO_ARGS << (int)1 << url;
-        TransferJob * job = new PostErrorJob(_error, url.prettyUrl(), packedArgs, postData);
-        job->setUiDelegate(new JobUiDelegate());
-        if (!(flags & HideProgressInfo)) {
-            KIO::getJobTracker()->registerJob(job);
-        }
+    TransferJob* job = precheckHttpPost(_url, postData, flags);
+    if (job)
         return job;
-    }
 
     // Send http post command (1), decoded path and encoded query
     KIO_ARGS << (int)1 << _url;
-    TransferJob * job = TransferJobPrivate::newJob(_url, CMD_SPECIAL, packedArgs, postData, flags);
+    job = TransferJobPrivate::newJob(_url, CMD_SPECIAL, packedArgs, postData, flags);
 
     if (redirection)
       QTimer::singleShot(0, job, SLOT(slotPostRedirection()) );
@@ -1393,6 +1414,26 @@ TransferJob *KIO::http_post( const KUrl& url, const QByteArray &postData, JobFla
     return job;
 }
 
+StoredTransferJob *KIO::storedHttpPost( const QByteArray& postData, const KUrl& url, JobFlags flags )
+{
+    bool redirection = false;
+    KUrl _url(url);
+    if (_url.path().isEmpty())
+    {
+      redirection = true;
+      _url.setPath("/");
+    }
+
+    StoredTransferJob* job = precheckHttpPost(_url, postData, flags);
+    if (job)
+        return job;
+
+    // Send http post command (1), decoded path and encoded query
+    KIO_ARGS << (int)1 << _url;
+    job = StoredTransferJobPrivate::newJob(_url, CMD_SPECIAL, packedArgs, postData, flags );
+    return job;
+}
+
 // http post got redirected from http://host to http://host/ by TransferJob
 // We must do this redirection ourselves because redirections by the
 // slave change post jobs into get jobs.
@@ -1413,35 +1454,6 @@ TransferJob *KIO::put( const KUrl& url, int permissions, JobFlags flags )
 
 //////////
 
-class KIO::StoredTransferJobPrivate: public TransferJobPrivate
-{
-public:
-    StoredTransferJobPrivate(const KUrl& url, int command,
-                             const QByteArray &packedArgs,
-                             const QByteArray &_staticData)
-        : TransferJobPrivate(url, command, packedArgs, _staticData),
-          m_uploadOffset( 0 )
-        {}
-    QByteArray m_data;
-    int m_uploadOffset;
-
-    void slotStoredData( KIO::Job *job, const QByteArray &data );
-    void slotStoredDataReq( KIO::Job *job, QByteArray &data );
-
-    Q_DECLARE_PUBLIC(StoredTransferJob)
-
-    static inline StoredTransferJob *newJob(const KUrl &url, int command,
-                                            const QByteArray &packedArgs, JobFlags flags)
-    {
-        StoredTransferJob *job = new StoredTransferJob(
-            *new StoredTransferJobPrivate(url, command, packedArgs, QByteArray()));
-        job->setUiDelegate(new JobUiDelegate);
-        if (!(flags & HideProgressInfo))
-            KIO::getJobTracker()->registerJob(job);
-        return job;
-    }
-};
-
 StoredTransferJob::StoredTransferJob(StoredTransferJobPrivate &dd)
     : TransferJob(dd)
 {
@@ -1461,6 +1473,7 @@ void StoredTransferJob::setData( const QByteArray& arr )
     Q_ASSERT( d->m_data.isNull() ); // check that we're only called once
     Q_ASSERT( d->m_uploadOffset == 0 ); // no upload started yet
     d->m_data = arr;
+    setTotalSize( d->m_data.size() );
 }
 
 QByteArray StoredTransferJob::data() const
@@ -1503,7 +1516,7 @@ StoredTransferJob *KIO::storedGet( const KUrl& url, LoadType reload, JobFlags fl
 {
     // Send decoded path and encoded query
     KIO_ARGS << url;
-    StoredTransferJob * job = StoredTransferJobPrivate::newJob(url, CMD_GET, packedArgs, flags);
+    StoredTransferJob * job = StoredTransferJobPrivate::newJob(url, CMD_GET, packedArgs, QByteArray(), flags);
     if (reload == Reload)
        job->addMetaData("cache", "reload");
     return job;
@@ -1513,7 +1526,7 @@ StoredTransferJob *KIO::storedPut( const QByteArray& arr, const KUrl& url, int p
                                    JobFlags flags )
 {
     KIO_ARGS << url << qint8( (flags & Overwrite) ? 1 : 0 ) << qint8( (flags & Resume) ? 1 : 0 ) << permissions;
-    StoredTransferJob * job = StoredTransferJobPrivate::newJob(url, CMD_PUT, packedArgs, flags );
+    StoredTransferJob * job = StoredTransferJobPrivate::newJob(url, CMD_PUT, packedArgs, QByteArray(), flags );
     job->setData( arr );
     return job;
 }
@@ -1843,7 +1856,7 @@ void FileCopyJobPrivate::startRenameJob(const KUrl &slave_url)
     Q_Q(FileCopyJob);
     m_mustChmod = true;  // CMD_RENAME by itself doesn't change permissions
     KIO_ARGS << m_src << m_dest << (qint8) (m_flags & Overwrite);
-    m_moveJob = SimpleJobPrivate::newJob(slave_url, CMD_RENAME, packedArgs);
+    m_moveJob = SimpleJobPrivate::newJobNoUi(slave_url, CMD_RENAME, packedArgs);
     q->addSubjob( m_moveJob );
     connectSubjob( m_moveJob );
 }
@@ -1973,10 +1986,15 @@ void FileCopyJobPrivate::slotCanResume( KIO::Job* job, KIO::filesize_t offset )
               offset = 0;
             else if ( res == R_CANCEL )
             {
-                if ( job == m_putJob )
+                if ( job == m_putJob ) {
                     m_putJob->kill( FileCopyJob::Quietly );
-                else
+                    q->removeSubjob(m_putJob);
+                    m_putJob = 0;
+                } else {
                     m_copyJob->kill( FileCopyJob::Quietly );
+                    q->removeSubjob(m_copyJob);
+                    m_copyJob = 0;
+                }
                 q->setError( ERR_USER_CANCELED );
                 q->emitResult();
                 return;
@@ -2063,6 +2081,8 @@ void FileCopyJobPrivate::slotDataReq( KIO::Job * , QByteArray &data)
        q->setError( ERR_INTERNAL );
        q->setErrorText( "'Put' job did not send canResume or 'Get' job did not send data!" );
        m_putJob->kill( FileCopyJob::Quietly );
+       q->removeSubjob(m_putJob);
+       m_putJob = 0;
        q->emitResult();
        return;
    }
@@ -2085,6 +2105,7 @@ void FileCopyJob::slotResult( KJob *job)
 {
    Q_D(FileCopyJob);
    //kDebug(7007) << "this=" << this << "job=" << job;
+   removeSubjob(job);
    // Did job have an error ?
    if ( job->error() )
    {
@@ -2092,14 +2113,12 @@ void FileCopyJob::slotResult( KJob *job)
       {
          d->m_moveJob = 0;
          d->startBestCopyMethod();
-         removeSubjob(job);
          return;
       }
       else if ((job == d->m_copyJob) && (job->error() == ERR_UNSUPPORTED_ACTION))
       {
          d->m_copyJob = 0;
          d->startDataPump();
-         removeSubjob(job);
          return;
       }
       else if (job == d->m_getJob)
@@ -2186,7 +2205,6 @@ void FileCopyJob::slotResult( KJob *job)
        d->m_chmodJob = 0; // Finished
    }
 
-   removeSubjob(job);
    if ( !hasSubjobs() )
        emitResult();
 }
@@ -2248,6 +2266,11 @@ public:
             KIO::getJobTracker()->registerJob(job);
         return job;
     }
+    static inline ListJob *newJobNoUi(const KUrl& u, bool _recursive, const QString &_prefix,
+                                      bool _includeHidden)
+    {
+        return new ListJob(*new ListJobPrivate(u, _recursive, _prefix, _includeHidden));
+    }
 };
 
 ListJob::ListJob(ListJobPrivate &dd)
@@ -2297,11 +2320,10 @@ void ListJobPrivate::slotListEntries( const KIO::UDSEntryList& list )
                 const QString filename = itemURL.fileName();
                 // skip hidden dirs when listing if requested
                 if (filename != ".." && filename != "." && (includeHidden || filename[0] != '.')) {
-                    ListJob *job = ListJobPrivate::newJob(itemURL,
+                    ListJob *job = ListJobPrivate::newJobNoUi(itemURL,
                                                true /*recursive*/,
                                                prefix + filename + '/',
                                                includeHidden);
-                    job->setUiDelegate(new JobUiDelegate());
                     Scheduler::scheduleJob(job);
                     q->connect(job, SIGNAL(entries( KIO::Job *, const KIO::UDSEntryList& )),
                                SLOT( gotEntries( KIO::Job*, const KIO::UDSEntryList& )));
