@@ -2,7 +2,7 @@
    Copyright (C) 1999 Torben Weis <weis@kde.org>
    Copyright (C) 2000- Waldo Bastain <bastain@kde.org>
    Copyright (C) 2000- Dawit Alemayehu <adawit@kde.org>
-   Copyright (C) 2008 Jaroslaw Staniek <js@iidea.pl>
+   Copyright (C) 2008 Jarosław Staniek <staniek@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -92,10 +92,10 @@ void KProtocolManager::reparseConfiguration()
     if (d->config) {
         d->config->reparseConfiguration();
     }
-    d->protocol = QString();
-    d->proxy = QString();
-    d->modifiers = QString();
-    d->useragent = QString();
+    d->protocol.clear();
+    d->proxy.clear();
+    d->modifiers.clear();
+    d->useragent.clear();
     d->url.clear();
 
   // Force the slave config to re-read its config...
@@ -315,6 +315,8 @@ static bool revmatch(const char *host, const char *nplist)
     {
       if ( nptr==nplist || nptr[-1]==',' || nptr[-1]==' ')
         return true;
+      if ( nptr[-1]=='/' && hptr == host ) // "bugs.kde.org" vs "http://bugs.kde.org", the config UI says URLs are ok
+        return true;
       if ( hptr == host ) // e.g. revmatch("bugs.kde.org","mybugs.kde.org")
         return false;
 
@@ -384,7 +386,7 @@ QString KProtocolManager::slaveProtocol(const KUrl &url, QString &proxy)
         if ( (!useRevProxy && !isRevMatch) || (useRevProxy && isRevMatch) )
         {
            d->url = proxy;
-           if ( d->url.isValid() )
+           if (d->url.isValid() && !d->url.protocol().isEmpty())
            {
               // The idea behind slave protocols is not applicable to http
               // and webdav protocols.
@@ -406,7 +408,7 @@ QString KProtocolManager::slaveProtocol(const KUrl &url, QString &proxy)
   }
 
   d->url = url;
-  d->proxy = proxy = QString();
+  d->proxy.clear(); proxy.clear();
   d->protocol = url.protocol();
   return d->protocol;
 }
@@ -466,7 +468,7 @@ QString KProtocolManager::defaultUserAgent( const QString &_modifiers )
     }
     if( modifiers.contains('l') )
     {
-      supp += QString("; %1").arg(acceptLanguagesHeader());
+      supp += QString("; %1").arg(KGlobal::locale()->language());
     }
   }
   d->modifiers = modifiers;
