@@ -97,9 +97,8 @@ static int  (*K_BIO_write) (BIO *b, const void *data, int len) = 0L;
 static int (*K_PEM_ASN1_write_bio) (int (*)(),const char *,BIO *,char *,
                                    const EVP_CIPHER *,unsigned char *,int ,
                                             pem_password_cb *, void *) = 0L;
-static ASN1_METHOD* (*K_X509_asn1_meth) (void) = 0L;
-static int (*K_ASN1_i2d_fp)(int (*)(),FILE *,unsigned char *) = 0L;
-static int (*K_i2d_ASN1_HEADER)(ASN1_HEADER *, unsigned char **) = 0L;
+static int (*K_ASN1_item_i2d_fp)(ASN1_ITEM *,FILE *,unsigned char *) = 0L;
+static ASN1_ITEM *K_NETSCAPE_X509_it = 0L;
 static int (*K_X509_print_fp)  (FILE *, X509*) = 0L;
 static int (*K_i2d_PKCS12)  (PKCS12*, unsigned char**) = 0L;
 static int (*K_i2d_PKCS12_fp)  (FILE *, PKCS12*) = 0L;
@@ -257,7 +256,7 @@ static QString findMostRecentLib(QString dir, QString name)
        QString best = 0L;
        // where do we start
        uint s = filter.length()-1;
-       for (QStringList::Iterator it = l.begin(); it != l.end(); ++it) {
+       for (QStringList::const_iterator it = l.begin(); it != l.end(); ++it) {
                QString numberpart = (*it).mid(s);
                uint endmaj = numberpart.indexOf('.');
                if (endmaj == -1)
@@ -380,11 +379,11 @@ KOpenSSLProxy::KOpenSSLProxy()
 	     ;
 #endif
 
-   for (QStringList::Iterator it = libpaths.begin();
-                              it != libpaths.end();
+   for (QStringList::const_iterator it = libpaths.constBegin();
+                              it != libpaths.constEnd();
                               ++it) {
-      for (QStringList::Iterator shit = libnamesc.begin();
-                                 shit != libnamesc.end();
+      for (QStringList::const_iterator shit = libnamesc.constBegin();
+                                 shit != libnamesc.constEnd();
                                  ++shit) {
          QString alib = *it;
          if (!alib.isEmpty() && !alib.endsWith('/'))
@@ -392,7 +391,7 @@ KOpenSSLProxy::KOpenSSLProxy()
          alib += *shit;
 	     // someone knows why this is needed?
 	     QString tmpStr(alib.toLatin1().constData());
-	     tmpStr.replace(QRegExp("\\(.*\\)"), "");
+	     tmpStr.remove(QRegExp("\\(.*\\)"));
          if (!access(tmpStr.toLatin1(), R_OK)) {
             d->cryptoLib = new KLibrary(alib);
             d->cryptoLib->setLoadHints(QLibrary::ExportExternalSymbolsHint);
@@ -441,9 +440,9 @@ KOpenSSLProxy::KOpenSSLProxy()
       K_BIO_ctrl = (long (*) (BIO *,int,long,void *)) d->cryptoLib->resolveFunction("BIO_ctrl");
       K_BIO_write = (int (*) (BIO *b, const void *data, int len)) d->cryptoLib->resolveFunction("BIO_write");
       K_PEM_ASN1_write_bio = (int (*)(int (*)(), const char *,BIO*, char*, const EVP_CIPHER *, unsigned char *, int, pem_password_cb *, void *)) d->cryptoLib->resolveFunction("PEM_ASN1_write_bio");
-      K_X509_asn1_meth = (ASN1_METHOD* (*)(void)) d->cryptoLib->resolveFunction("X509_asn1_meth");
-      K_ASN1_i2d_fp = (int (*)(int (*)(), FILE*, unsigned char *)) d->cryptoLib->resolveFunction("ASN1_i2d_fp");
-      K_i2d_ASN1_HEADER = (int (*)(ASN1_HEADER *, unsigned char **)) d->cryptoLib->resolveFunction("i2d_ASN1_HEADER");
+      K_ASN1_item_i2d_fp = (int (*)(ASN1_ITEM *, FILE*, unsigned char *))
+          d->cryptoLib->resolveFunction("ASN1_item_i2d_fp");
+      K_NETSCAPE_X509_it = (ASN1_ITEM *) d->cryptoLib->resolveFunction("NETSCAPE_X509_it");
       K_X509_print_fp = (int (*)(FILE*, X509*)) d->cryptoLib->resolveFunction("X509_print_fp");
       K_i2d_PKCS12 = (int (*)(PKCS12*, unsigned char**)) d->cryptoLib->resolveFunction("i2d_PKCS12");
       K_i2d_PKCS12_fp = (int (*)(FILE *, PKCS12*)) d->cryptoLib->resolveFunction("i2d_PKCS12_fp");
@@ -553,18 +552,18 @@ KOpenSSLProxy::KOpenSSLProxy()
    }
    }
 #else
-   for (QStringList::Iterator it = libpaths.begin();
-                              it != libpaths.end();
+   for (QStringList::const_iterator it = libpaths.constBegin();
+                              it != libpaths.constEnd();
                               ++it) {
-      for (QStringList::Iterator shit = libnamess.begin();
-                                 shit != libnamess.end();
+      for (QStringList::const_iterator shit = libnamess.constBegin();
+                                 shit != libnamess.constEnd();
                                  ++shit) {
          QString alib = *it;
          if (!alib.isEmpty() && !alib.endsWith('/'))
             alib += '/';
          alib += *shit;
 	     QString tmpStr(alib.toLatin1());
-	     tmpStr.replace(QRegExp("\\(.*\\)"), "");
+	     tmpStr.remove(QRegExp("\\(.*\\)"));
          if (!access(tmpStr.toLatin1(), R_OK)) {
          	d->sslLib = new KLibrary(alib);
             d->sslLib->setLoadHints(QLibrary::ExportExternalSymbolsHint);
@@ -623,7 +622,7 @@ KOpenSSLProxy::KOpenSSLProxy()
       K_SSL_set_session = (int (*)(SSL*,SSL_SESSION*)) d->sslLib->resolveFunction("SSL_set_session");
       K_d2i_SSL_SESSION = (SSL_SESSION* (*)(SSL_SESSION**,unsigned char**, long)) d->sslLib->resolveFunction("d2i_SSL_SESSION");
       K_i2d_SSL_SESSION = (int (*)(SSL_SESSION*,unsigned char**)) d->sslLib->resolveFunction("i2d_SSL_SESSION");
-      K_SSL_get_ciphers = (STACK *(*)(const SSL*)) d->sslLib->resolveFunction("SSL_get_ciphers");
+      K_SSL_get_ciphers = (STACK_OF(SSL_CIPHER) *(*)(const SSL*)) d->sslLib->resolveFunction("SSL_get_ciphers");
 #endif
 
 
@@ -996,16 +995,9 @@ int KOpenSSLProxy::PEM_write_bio_X509(BIO *bp, X509 *x) {
    else return -1;
 }
 
-
-ASN1_METHOD *KOpenSSLProxy::X509_asn1_meth(void) {
-   if (K_X509_asn1_meth) return (K_X509_asn1_meth)();
-   else return 0L;
-}
-
-
-int KOpenSSLProxy::ASN1_i2d_fp(FILE *out,unsigned char *x) {
-   if (K_ASN1_i2d_fp && K_i2d_ASN1_HEADER)
-        return (K_ASN1_i2d_fp)((int (*)())K_i2d_ASN1_HEADER, out, x);
+int KOpenSSLProxy::ASN1_item_i2d_fp(FILE *out,unsigned char *x) {
+   if (K_ASN1_item_i2d_fp && K_NETSCAPE_X509_it)
+        return (K_ASN1_item_i2d_fp)(K_NETSCAPE_X509_it, out, x);
    else return -1;
 }
 
