@@ -113,6 +113,7 @@ void JobTest::initTestCase()
     }
 #endif
 
+    qRegisterMetaType<KJob*>("KJob*");
     qRegisterMetaType<KIO::Job*>("KIO::Job*");
     qRegisterMetaType<KUrl>("KUrl");
     qRegisterMetaType<time_t>("time_t");
@@ -144,7 +145,7 @@ void JobTest::enterLoop()
     eventLoop.exec(QEventLoop::ExcludeUserInputEvents);
 }
 
-void JobTest::get()
+void JobTest::storedGet()
 {
     kDebug() ;
     const QString filePath = homeTmpDir() + "fileFromHome";
@@ -153,6 +154,8 @@ void JobTest::get()
     m_result = -1;
 
     KIO::StoredTransferJob* job = KIO::storedGet( u, KIO::NoReload, KIO::HideProgressInfo );
+    QSignalSpy spyPercent(job, SIGNAL(percent(KJob*, unsigned long)));
+    QVERIFY(spyPercent.isValid());
     job->setUiDelegate( 0 );
     connect( job, SIGNAL( result( KJob* ) ),
             this, SLOT( slotGetResult( KJob* ) ) );
@@ -160,6 +163,7 @@ void JobTest::get()
     QCOMPARE( m_result, 0 ); // no error
     QCOMPARE( m_data, QByteArray("Hello\0world", 11) );
     QCOMPARE( m_data.size(), 11 );
+    QVERIFY(!spyPercent.isEmpty());
 }
 
 void JobTest::slotGetResult( KJob* job )
@@ -216,6 +220,27 @@ void JobTest::slotResult( KJob* job )
     emit exitLoop();
 }
 
+void JobTest::storedPut()
+{
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    KUrl u(filePath);
+    QByteArray putData = "This is the put data";
+    KIO::TransferJob* job = KIO::storedPut( putData, u, 0600, KIO::Overwrite | KIO::HideProgressInfo );
+    QSignalSpy spyPercent(job, SIGNAL(percent(KJob*, unsigned long)));
+    QVERIFY(spyPercent.isValid());
+    QDateTime mtime = QDateTime::currentDateTime().addSecs( -30 ); // 30 seconds ago
+    mtime.setTime_t(mtime.toTime_t()); // hack for losing the milliseconds
+    job->setModificationTime(mtime);
+    job->setUiDelegate( 0 );
+    QVERIFY(job->exec());
+    QFileInfo fileInfo(filePath);
+    QVERIFY(fileInfo.exists());
+    QCOMPARE(fileInfo.size(), (long long)putData.size());
+    QCOMPARE((int)fileInfo.permissions(), (int)(QFile::ReadOwner | QFile::WriteOwner | QFile::ReadUser | QFile::WriteUser ));
+    QCOMPARE(fileInfo.lastModified(), mtime);
+    QVERIFY(!spyPercent.isEmpty());
+}
+
 ////
 
 void JobTest::copyLocalFile( const QString& src, const QString& dest )
@@ -591,6 +616,23 @@ void JobTest::listRecursive()
     QCOMPARE( joinedNames.toLatin1(), ref_names );
 }
 
+void JobTest::listFile()
+{
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    createTestFile( filePath );
+    KIO::ListJob* job = KIO::listDir(KUrl(filePath), KIO::HideProgressInfo);
+    job->setUiDelegate( 0 );
+    QVERIFY(!job->exec());
+    QCOMPARE(job->error(), static_cast<int>(KIO::ERR_IS_FILE));
+
+    // And list something that doesn't exist
+    const QString path = homeTmpDir() + "fileFromHomeDoesNotExist";
+    job = KIO::listDir(KUrl(path), KIO::HideProgressInfo);
+    job->setUiDelegate( 0 );
+    QVERIFY(!job->exec());
+    QCOMPARE(job->error(), static_cast<int>(KIO::ERR_DOES_NOT_EXIST));
+}
+
 void JobTest::killJob()
 {
     const QString src = homeTmpDir();
@@ -1088,7 +1130,7 @@ void JobTest::copyFileToSystem( bool resolve_local_urls )
 
 void JobTest::getInvalidUrl()
 {
-    KUrl url("file://\"\"");
+    KUrl url("http://[strange;hostname]/");
     QVERIFY(!url.isValid());
 
     KIO::SimpleJob* job = KIO::get(url, KIO::NoReload, KIO::HideProgressInfo);
@@ -1121,7 +1163,20 @@ void JobTest::deleteFile()
 void JobTest::deleteDirectory()
 {
     const QString dest = otherTmpDir() + "dirFromHome_copied";
-    QVERIFY(QFile::exists(dest));
+    if (!QFile::exists(dest))
+        createTestDirectory(dest);
+    // Let's put a few things in there to see if the recursive deletion works correctly
+    // A hidden file:
+    createTestFile(dest + "/.hidden");
+#ifndef Q_WS_WIN
+    // A broken symlink:
+    createTestSymlink(dest+"/broken_symlink");
+    // A symlink to a dir:
+    bool symlink_ok = symlink( KDESRCDIR, QFile::encodeName( dest + "/symlink_to_dir" ) ) == 0;
+    if ( !symlink_ok )
+        kFatal() << "couldn't create symlink: " << strerror( errno ) ;
+#endif
+
     KIO::Job* job = KIO::del(KUrl(dest), KIO::HideProgressInfo);
     job->setUiDelegate(0);
     bool ok = KIO::NetAccess::synchronousRun(job, 0);
@@ -1129,17 +1184,56 @@ void JobTest::deleteDirectory()
     QVERIFY(!QFile::exists(dest));
 }
 
+void JobTest::deleteSymlink(bool using_fast_path)
+{
+    extern KIO_EXPORT bool kio_resolve_local_urls;
+    kio_resolve_local_urls = !using_fast_path;
+
+#ifndef Q_WS_WIN
+    const QString src = homeTmpDir() + "dirFromHome";
+    createTestDirectory(src);
+    QVERIFY(QFile::exists(src));
+    const QString dest = homeTmpDir() + "/dirFromHome_link";
+    if (!QFile::exists(dest)) {
+        // Add a symlink to a dir, to make sure we don't recurse into those
+        bool symlinkOk = symlink(QFile::encodeName(src), QFile::encodeName(dest)) == 0;
+        QVERIFY( symlinkOk );
+        QVERIFY(QFile::exists(dest));
+    }
+    KIO::Job* job = KIO::del(KUrl(dest), KIO::HideProgressInfo);
+    job->setUiDelegate(0);
+    bool ok = KIO::NetAccess::synchronousRun(job, 0);
+    QVERIFY(ok);
+    QVERIFY(!QFile::exists(dest));
+    QVERIFY(QFile::exists(src));
+#endif
+
+    kio_resolve_local_urls = true;
+}
+
+
+void JobTest::deleteSymlink()
+{
+#ifndef Q_WS_WIN
+    deleteSymlink(true);
+    deleteSymlink(false);
+#endif
+}
+
 void JobTest::deleteManyDirs(bool using_fast_path)
 {
     extern KIO_EXPORT bool kio_resolve_local_urls;
     kio_resolve_local_urls = !using_fast_path;
 
+    const int numDirs = 50;
     KUrl::List dirs;
-    for (int i = 0; i < 50; ++i) {
+    for (int i = 0; i < numDirs; ++i) {
         const QString dir = homeTmpDir() + "dir" + QString::number(i);
         createTestDirectory(dir);
         dirs << KUrl(dir);
     }
+    QTime dt;
+    dt.start();
     KIO::Job* job = KIO::del(dirs, KIO::HideProgressInfo);
     job->setUiDelegate(0);
     bool ok = KIO::NetAccess::synchronousRun(job, 0);
@@ -1148,6 +1242,7 @@ void JobTest::deleteManyDirs(bool using_fast_path)
         QVERIFY(!QFile::exists(dir.path()));
     }
 
+    kDebug() << "Deleted" << numDirs << "dirs in" << dt.elapsed() << "milliseconds";
     kio_resolve_local_urls = true;
 }
 
@@ -1220,6 +1315,26 @@ void JobTest::deleteManyFilesTogether()
     deleteManyFilesTogether(false);
 }
 
+void JobTest::rmdirEmpty()
+{
+    const QString dir = homeTmpDir() + "dir";
+    QDir().mkdir(dir);
+    QVERIFY(QFile::exists(dir));
+    KIO::Job* job = KIO::rmdir(dir);
+    QVERIFY(job->exec());
+    QVERIFY(!QFile::exists(dir));
+}
+
+void JobTest::rmdirNotEmpty()
+{
+    const QString dir = homeTmpDir() + "dir";
+    createTestDirectory(dir);
+    createTestDirectory(dir + "/subdir");
+    KIO::Job* job = KIO::rmdir(dir);
+    QVERIFY(!job->exec());
+    QVERIFY(QFile::exists(dir));
+}
+
 void JobTest::stat()
 {
 #if 1
@@ -1274,6 +1389,25 @@ void JobTest::mimeType()
 #endif
 }
 
+void JobTest::moveFileDestAlreadyExists() // #157601
+{
+    const QString file1 = homeTmpDir() + "fileFromHome";
+    createTestFile( file1 );
+    const QString file2 = homeTmpDir() + "anotherFile";
+    createTestFile( file2 );
+    const QString existingDest = otherTmpDir() + "fileFromHome";
+    createTestFile( existingDest );
+
+    KUrl::List urls; urls << KUrl(file1) << KUrl(file2);
+    KIO::CopyJob* job = KIO::move(urls, otherTmpDir(), KIO::HideProgressInfo);
+    job->setUiDelegate(0);
+    job->setAutoSkip(true);
+    bool ok = KIO::NetAccess::synchronousRun(job, 0);
+    QVERIFY(ok);
+    QVERIFY(QFile::exists(file1)); // it was skipped
+    QVERIFY(!QFile::exists(file2)); // it was moved
+}
+
 void JobTest::moveAndOverwrite()
 {
     const QString sourceFile = homeTmpDir() + "fileFromHome";
