@@ -25,8 +25,8 @@ void recursive_print( const KArchiveDirectory * dir, const QString & path )
 {
   QStringList l = dir->entries();
   l.sort();
-  QStringList::ConstIterator it = l.begin();
-  for( ; it != l.end(); ++it )
+  QStringList::ConstIterator it = l.constBegin();
+  for( ; it != l.constEnd(); ++it )
   {
     const KArchiveEntry* entry = dir->entry( (*it) );
     printf("mode=%07o %s %s %s%s %lld isdir=%d\n", entry->permissions(), entry->user().toLatin1().constData(), entry->group().toLatin1().constData(), path.toLatin1().constData(), (*it).toLatin1().constData(),
