@@ -73,7 +73,24 @@ extern "C"
 using namespace Kross;
 
 /*********************************************************************************
- * FormDialog
+ * FormList
+ */
+
+FormListView::FormListView(QWidget* parent) : QListWidget(parent) {}
+FormListView::~FormListView() {}
+void FormListView::clear() { QListWidget::clear(); }
+void FormListView::remove(int index) { delete QListWidget::item(index); }
+void FormListView::addItem(const QString& text) { QListWidget::addItem(text); }
+int FormListView::count() { return QListWidget::count(); }
+int FormListView::current() { return QListWidget::currentRow(); }
+void FormListView::setCurrent(int row) { QListWidget::setCurrentRow(row); }
+QString FormListView::text(int row) {
+    QListWidgetItem *item = QListWidget::item(row);
+    return item ? item->text() : QString();
+}
+
+/*********************************************************************************
+ * FormFileWidget
  */
 
 namespace Kross {
@@ -83,6 +100,7 @@ namespace Kross {
     {
         public:
             KFileWidget* filewidget;
+            QString filename;
     };
 
 }
@@ -100,6 +118,9 @@ FormFileWidget::FormFileWidget(QWidget* parent, const QString& startDirOrVariabl
     //QMetaObject::invokeMethod(d->filewidget, "toggleSpeedbar", Q_ARG(bool,false));
     //KFileDialog::setMode( KFile::File | KFile::LocalOnly );
 
+    // slotOk() emits accepted, accept() emits fileSelected()
+    QObject::connect(d->filewidget, SIGNAL(fileSelected(const QString&)), this, SLOT(slotFileSelected(const QString&)));
+
     QObject::connect(d->filewidget, SIGNAL(fileSelected(const QString&)), this, SIGNAL(fileSelected(const QString&)));
     QObject::connect(d->filewidget, SIGNAL(fileHighlighted(const QString&)), this, SIGNAL(fileHighlighted(const QString&)));
     QObject::connect(d->filewidget, SIGNAL(selectionChanged()), this, SIGNAL(selectionChanged()));
@@ -150,19 +171,28 @@ void FormFileWidget::setMimeFilter(const QStringList& filter)
     d->filewidget->setMimeFilter(filter);
 }
 
+void FormFileWidget::slotFileSelected( const QString & fn )
+{
+    //kDebug()<<fn;
+    d->filename = fn;
+}
+
 QString FormFileWidget::selectedFile() const
 {
-    KUrl selectedUrl;
-    QString locationText = d->filewidget->locationEdit()->currentText();
-    if( locationText.contains( '/' ) ) { // relative path? -> prepend the current directory
-        KUrl u( d->filewidget->baseUrl(), KShell::tildeExpand(locationText) );
-        selectedUrl = u.isValid() ? u : selectedUrl = d->filewidget->baseUrl();
+    if ( d->filewidget->operationMode() != KFileWidget::Saving ) {
+      d->filewidget->accept();
+    } else {
+      //kDebug()<<d->filename<<d->filewidget->operationMode();
+      if ( d->filename.isEmpty() ) {
+        // make KFileWidget create an url for us (including extension if neccessary)
+        QObject::connect(d->filewidget, SIGNAL(accepted()), d->filewidget, SLOT(accept()));
+        d->filewidget->slotOk();
+        QObject::disconnect(d->filewidget, SIGNAL(accepted()), d->filewidget, SLOT(accept()));
+      }
     }
-    else { // simple filename -> just use the current URL
-        selectedUrl = d->filewidget->baseUrl();
-    }
-    QFileInfo fi( selectedUrl.path(), d->filewidget->locationEdit()->currentText() );
-    return fi.absoluteFilePath();
+    //kDebug()<<d->filename;
+    KUrl url( d->filename );
+    return url.path(); // strip file:// at least python chokes on it
 }
 
 /*********************************************************************************
@@ -324,6 +354,7 @@ FormDialog::FormDialog(const QString& caption)
 
 FormDialog::~FormDialog()
 {
+    kWarning()<<"dtor";
     delete d;
 }
 
@@ -339,6 +370,18 @@ bool FormDialog::setButtons(const QString& buttons)
     return true;
 }
 
+bool FormDialog::setButtonText(const QString& button, const QString& text)
+{
+    int i = metaObject()->indexOfEnumerator("ButtonCode");
+    Q_ASSERT( i >= 0 );
+    QMetaEnum e = metaObject()->enumerator(i);
+    int v = e.keysToValue( button.toUtf8() );
+    if( v < 0 )
+        return false;
+    KDialog::setButtonText( (KDialog::ButtonCode) v, text);
+    return true;
+}
+
 bool FormDialog::setFaceType(const QString& facetype)
 {
     int i = KPageView::staticMetaObject.indexOfEnumerator("FaceType");
@@ -370,21 +413,32 @@ QWidget* FormDialog::page(const QString& name) const
     return d->items.contains(name) ? d->items[name]->widget() : 0;
 }
 
-QWidget* FormDialog::addPage(const QString& name, const QString& header, const QString& iconname)
+//shared by FormDialog and FormAssistant
+static KPageWidgetItem* formAddPage(KPageDialog* dialog, const QString& name, const QString& header, const QString& iconname)
 {
-    QWidget* widget = new QWidget( mainWidget() );
+    QWidget* widget = new QWidget( dialog->mainWidget() );
     QVBoxLayout* boxlayout = new QVBoxLayout(widget);
     boxlayout->setSpacing(0);
     boxlayout->setMargin(0);
     widget->setLayout(boxlayout);
 
-    KPageWidgetItem* item = KPageDialog::addPage(widget, name);
+    KPageWidgetItem* item = dialog->addPage(widget, name);
     item->setHeader(header.isNull() ? name : header);
     if( ! iconname.isEmpty() )
         item->setIcon( KIcon(iconname) );
-    d->items.insert(name, item);
+    //d->items.insert(name, item);
 
-    return item->widget();
+    return item;
+}
+
+QWidget* FormDialog::addPage(const QString& name, const QString& header, const QString& iconname)
+{
+    return d->items.insert(name, formAddPage((KPageDialog*)this,name,header,iconname)).value()->widget();
+}
+
+void FormDialog::setMainWidget(QWidget *newMainWidget)
+{
+    KDialog::setMainWidget(newMainWidget);
 }
 
 QString FormDialog::result()
@@ -411,6 +465,123 @@ void FormDialog::slotCurrentPageChanged(KPageWidgetItem* current)
     //foreach(QWidget* widget, current->widget()->findChildren< QWidget* >("")) widget->setFocus();
 }
 
+
+namespace Kross {
+    /// \internal d-pointer class.
+    class FormAssistant::Private
+    {
+        public:
+            KDialog::ButtonCode buttoncode;
+            QHash<QString, KPageWidgetItem*> items;
+    };
+}
+FormAssistant::FormAssistant(const QString& caption)
+    : KAssistantDialog( /*0, Qt::WShowModal | Qt::WDestructiveClose*/ )
+    , d( new Private() )
+{
+    setCaption(caption);
+    setSizePolicy( QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding) );
+
+    connect(this, SIGNAL(currentPageChanged(KPageWidgetItem*,KPageWidgetItem*)),
+            this, SLOT(slotCurrentPageChanged(KPageWidgetItem*)));
+    /* unlike boost qt does not support defining of slot call order!
+    connect(this, SIGNAL(user2Clicked()), this, SIGNAL (nextClicked()));
+    connect(this, SIGNAL(user3Clicked()), this, SIGNAL (backClicked()));
+    */
+}
+
+FormAssistant::~FormAssistant()
+{
+    delete d;
+}
+
+void FormAssistant::showHelpButton(bool show)
+{
+    showButton(KDialog::Help, show);
+}
+
+void FormAssistant::back()
+{
+    emit backClicked();
+    KAssistantDialog::back();
+}
+void FormAssistant::next()
+{
+    emit nextClicked();
+    KAssistantDialog::next();
+}
+
+QString FormAssistant::currentPage() const
+{
+    KPageWidgetItem* item = KPageDialog::currentPage();
+    return item ? item->name() : QString();
+}
+
+bool FormAssistant::setCurrentPage(const QString& name)
+{
+    if( ! d->items.contains(name) )
+        return false;
+    KPageDialog::setCurrentPage( d->items[name] );
+    return true;
+}
+
+QWidget* FormAssistant::page(const QString& name) const
+{
+    return d->items.contains(name) ? d->items[name]->widget() : 0;
+}
+
+QWidget* FormAssistant::addPage(const QString& name, const QString& header, const QString& iconname)
+{
+    return d->items.insert(name, formAddPage((KPageDialog*)this,name,header,iconname)).value()->widget();
+}
+
+bool FormAssistant::isAppropriate (const QString& name) const
+{
+    return d->items.contains(name) && KAssistantDialog::isAppropriate(d->items[name]);
+}
+void FormAssistant::setAppropriate (const QString& name, bool appropriate)
+{
+    if (!d->items.contains(name))
+        return;
+
+    KAssistantDialog::setAppropriate(d->items[name],appropriate);
+}
+bool FormAssistant::isValid (const QString& name) const
+{
+    return d->items.contains(name) && KAssistantDialog::isValid(d->items[name]);
+}
+void FormAssistant::setValid (const QString& name, bool enable)
+{
+    if (!d->items.contains(name))
+        return;
+
+    KAssistantDialog::setValid(d->items[name],enable);
+}
+
+QString FormAssistant::result()
+{
+    int i = metaObject()->indexOfEnumerator("AssistantButtonCode");
+    if( i < 0 ) {
+        kWarning() << "Kross::FormAssistant::setButtons No such enumerator \"AssistantButtonCode\"";
+        return QString();
+    }
+    QMetaEnum e = metaObject()->enumerator(i);
+    return e.valueToKey(FormAssistant::AssistantButtonCode(int(d->buttoncode)));
+}
+
+void FormAssistant::slotButtonClicked(int button)
+{
+    d->buttoncode = (KDialog::ButtonCode) button;
+    KDialog::slotButtonClicked(button);
+}
+
+void FormAssistant::slotCurrentPageChanged(KPageWidgetItem* current)
+{
+    Q_UNUSED(current);
+    //kDebug() << "FormAssistant::slotCurrentPageChanged current=" << current->name();
+    //foreach(QWidget* widget, current->widget()->findChildren< QWidget* >("")) widget->setFocus();
+}
+
 /*********************************************************************************
  * FormModule
  */
@@ -516,6 +687,11 @@ QWidget* FormModule::createDialog(const QString& caption)
     return new FormDialog(caption);
 }
 
+QWidget* FormModule::createAssistant(const QString& caption)
+{
+    return new FormAssistant(caption);
+}
+
 QObject* FormModule::createLayout(QWidget* parent, const QString& layout)
 {
     QLayout* l = 0;
@@ -546,12 +722,46 @@ QWidget* FormModule::createWidget(QWidget* parent, const QString& className, con
     return widget;
 }
 
+
+QString FormModule::tr(const QString& str)
+{
+    return QObject::tr(str.toUtf8());
+}
+QString FormModule::tr(const QString& str, const QString& comment)
+{
+    return QObject::tr(str.toUtf8(),comment.toUtf8());
+}
+
 QWidget* FormModule::createWidgetFromUI(QWidget* parent, const QString& xml)
 {
     QFormBuilder builder;
-    QByteArray ba = xml.toUtf8();
+
+    QDomDocument doc("mydocument");
+    doc.setContent(xml.toUtf8());
+
+    QDomNodeList strings=doc.elementsByTagName("string");
+    int i=strings.size();
+    while(--i>=0)
+    {
+        QDomElement e=strings.at(i).toElement();
+        QString i18nd=e.attribute("comment").isEmpty()?QObject::tr(e.text().toUtf8()):QObject::tr(e.text().toUtf8(),e.attribute("comment").toUtf8());
+        if (i18nd==e.text())
+            continue;
+        QDomNode n = e.firstChild();
+        while (!n.isNull())
+        {
+            QDomNode nn=n.nextSibling();
+            if (n.isCharacterData())
+                e.removeChild(n);
+            n = nn;
+        }
+        e.appendChild(e.ownerDocument().createTextNode(i18nd));
+    }
+
+    QByteArray ba = doc.toByteArray();
     QBuffer buffer(&ba);
     buffer.open(QIODevice::ReadOnly);
+
     QWidget* widget = builder.load(&buffer, parent);
     if( widget && parent && parent->layout() )
         parent->layout()->addWidget(widget);
@@ -582,6 +792,19 @@ QWidget* FormModule::createFileWidget(QWidget* parent, const QString& startDirOr
     return widget;
 }
 
+QWidget* FormModule::createListView(QWidget* parent)
+{
+    FormListView* widget = new FormListView(parent);
+    if( parent && parent->layout() )
+        parent->layout()->addWidget(widget);
+    return widget;
+}
+
+QAction* FormModule::createAction(QObject* parent)
+{
+    return new QAction(parent);
+}
+
 QObject* FormModule::loadPart(QWidget* parent, const QString& name, const QUrl& url)
 {
     //name e.g. "libkghostview"
